--*****************************************************************************
--* File: lua/modules/ui/game/avatars.lua
--* Author: Ted Snook
--* Summary: In Game Avatar Icons
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Button = import('/lua/maui/button.lua').Button
local Group = import('/lua/maui/group.lua').Group
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local StatusBar = import('/lua/maui/statusbar.lua').StatusBar
local GameCommon = import('/lua/ui/game/gamecommon.lua')
local GameMain = import('/lua/ui/game/gamemain.lua')
local ToolTip = import('/lua/ui/game/tooltip.lua')
local TooltipInfo = import('/lua/ui/help/tooltips.lua')
local Prefs = import('/lua/user/prefs.lua')

local avatarData = {}
local avatars = {}
local avatarContainer = false
local dirtyTable = {}
local phaseCounter = 0
local avatarKey = {}
local showAvatars = true
local avatarToggle = false
local animationLock = false
local currentIndex = 0

function GetEngineerGeneric()
    local idleEngineers = GetIdleEngineers()
    if idleEngineers then
        local selEngineer = idleEngineers[currentIndex] or idleEngineers[1]
        currentIndex = currentIndex + 1
        if currentIndex > table.getn(idleEngineers) then
            currentIndex = 1
        end
        UISelectAndZoomTo(selEngineer)
    end
end

local mouseover = false

function CreateMouseover(control, data)
    if not Prefs.GetOption('tooltips') then return end
    if mouseover then mouseover:Destroy() end
    local text = ''
    local body = ''
    local tooltip_id = ""
    local pos_offset = 9
    if data.Type == 'Avatar' then
        tooltip_id = 'avatar_Avatar_ACU'
        pos_offset = 0
    elseif data.Type == 'Engineer' then
        local tier = string.lower(data.Tab)
        tooltip_id = 'avatar_Engineer_'..tier
    elseif data.Type == 'Group' then
        tooltip_id = 'avatar_group'
        text = LOCF(TooltipInfo['Tooltips'][tooltip_id]['title'], data.Tab)
        body = LOCF(TooltipInfo['Tooltips'][tooltip_id]['description'], data.Tab)
    end
    if tooltip_id != '' then
        if text == '' and body == '' then
            text = LOC(TooltipInfo['Tooltips'][tooltip_id]['title'])
            body = LOC(TooltipInfo['Tooltips'][tooltip_id]['description'])
        end
        mouseover = ToolTip.CreateExtendedToolTip(control, text, body)
        LayoutHelpers.AtVerticalCenterIn(mouseover, control, pos_offset)
        mouseover.Right:Set(function() return control.Left() - 1 end)
        mouseover:SetAlpha(0, true)
        mouseover:SetNeedsFrameUpdate(true)
        mouseover.OnFrame = function(self, deltaTime)
            local alpha = self:GetAlpha() + (deltaTime * 4)
            if alpha < 1 then
                self:SetAlpha(alpha, true)
            else
                self:SetAlpha(1, true)
                self:SetNeedsFrameUpdate(false)
            end
        end
    end
end

function CloseMouseover()
    mouseover:Destroy()
end

local SSCheckState = false

function Contract()
    showAvatars = false
    SSCheckState = avatarToggle:IsChecked()
    avatarToggle:Hide()
    for i, v in avatars do
        if v.icon then
            v.icon:Hide()
        end
    end
#   ToggleAvatars(false)
end

function Expand()
    showAvatars = true
    avatarToggle:Show()
    for i, v in avatars do
        if v.icon then
            v.icon:Show()
        end
    end
    avatarToggle:SetCheck(SSCheckState)
#   ToggleAvatars(true)
end

function FocusArmyChanged()
    for i, v in avatars do
        if v.icon then
            v.icon:Destroy()
        end
    end
    avatars = {}
end

function CreateAvatarIcons(parent)
    avatarContainer = Group(parent, "avatarContainer")
    avatarContainer.Top:Set(function() return parent.Top() + 53 end)
    avatarContainer.Left:Set(parent.Left)
    avatarContainer.Right:Set(function() return parent.Right() - 3 end)
    avatarContainer.Bottom:Set(parent.Bottom)
    avatarContainer:DisableHitTest()
    avatarContainer:SetNeedsFrameUpdate(true)
    
    avatarToggle = UIUtil.CreateCheckbox(avatarContainer, 
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_up.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_up.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_over.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_over.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_dis.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_dis.dds'), 
        'UI_Avatar_Hide')
    LayoutHelpers.AtRightTopIn(avatarToggle, parent, 3, 15)
    avatarToggle.OnCheck = function(self, checked)
        ToggleAvatars(not checked)
    end
    ToolTip.AddCheckboxTooltip(avatarToggle, 'avatar_toggle')
    
    GameMain.AddBeatFunction(AvatarUpdate)
end

function AvatarUpdate()
    dirtyTable = {}
	UpdateArmyAvatars()
	UpdateIdleBuilders()
	UpdateGroups()
	UpdateAvatarIcons()
end

function ToggleAvatars(state)
    showAvatars = state
    if UIUtil.GetAnimationPrefs() and avatarKey[1].icon and UIUtil.currentLayout != 'right' then
        avatarKey[1].icon:SetNeedsFrameUpdate(true)
        if state then
            animationLock = true
            avatarKey[1].icon.OnFrame = function(self, delta)
                if showAvatars then
                    local selfRight = self.Right()
                    if selfRight < avatarContainer.Right() then
                        animationLock = false
                        self.Right:Set(avatarContainer.Right)
                        self:SetNeedsFrameUpdate(false)
                    else
                        self.Right:Set(function() return selfRight - (delta * 500) end)
                    end
                end
            end
        else
            avatarKey[1].icon.OnFrame = function(self, delta)
                if not showAvatars then
                    local globalLeft = avatarKey[table.getsize(avatarKey)].icon.Left() - 10
                    local selfRight = self.Right()
                    if globalLeft < avatarContainer.Right() then
                        self.Right:Set(function() return selfRight + (delta * 500) end)
                    else
                        self:SetNeedsFrameUpdate(false)
                    end
                end
            end
        end
    else
        for i, v in avatarKey do
            if v.icon then
                if showAvatars then
                    v.icon:Show()
                else
                    v.icon:Hide()
                end
            end
        end
    end
end

function UpdateArmyAvatars()
    local tempAvatars = {}
    
    tempAvatars = GetArmyAvatars()
    
    for i, v in avatars do
        if v.Type == 'Avatar' then
            local found = false
            if tempAvatars then
                for h, x in tempAvatars do
                    if x:GetEntityId() == v.ID then
                        found = true
                        break
                    end
                end
            end
            if not found then
                table.insert(dirtyTable, i)
            end
        end
    end
    if tempAvatars then
        for i, v in tempAvatars do
            local found = false
            for h, x in avatars do
                if x.Type == 'Avatar' and x.ID == v:GetEntityId() then
                    found = true
                    if x.icon.Update then
                        x.icon.Update(x)
                    end
                    break
                end
            end
            if not found then
                local tempTable = {
                    Type = 'Avatar',
                    ID = v:GetEntityId(),
                    Units = { v },
                }
                table.insert(avatars, tempTable)
            end
        end
    end
end

function UpdateIdleBuilders()
    local idleEngineers = {}
    
    idleEngineers = GetIdleEngineers()
    
    local engineers = {}
    local EngLabels = {1,2,3,3}
    if idleEngineers then
        engineers[4] = EntityCategoryFilterDown(categories.SUBCOMMANDER, idleEngineers)
        engineers[3] = EntityCategoryFilterDown(categories.TECH3 - categories.SUBCOMMANDER, idleEngineers)
        engineers[2] = EntityCategoryFilterDown(categories.TECH2, idleEngineers)
        engineers[1] = EntityCategoryFilterDown(categories.TECH1, idleEngineers)
    end
    for i, v in avatars do
        if v.Type == 'Engineer' then
            local found = false
            for h, x in engineers do
                if h == v.ID and table.getsize(x) > 0 then
                    found = true
                    break
                end
            end
            if not found then
                table.insert(dirtyTable, i)
            end
        end
    end
    for i, v in engineers do
        if table.getsize(v) > 0 then
            local found = false
            local tempTable = {
                Type = 'Engineer',
                ID = i,
                Tab = LOCF("<LOC _T_digit>T%d", EngLabels[i]),
                Label = LOCF("<LOC avatar_0000>%d", table.getsize(v)),
                Units = {},
            }
            for index, unit in v do
                table.insert(tempTable.Units, unit)
            end
            for h, x in avatars do
                if x.Type == 'Engineer' and x.ID == i then
                    found = true
                    if x.icon.Update then
                        x.icon.Update(tempTable)
                    end
                    break
                end
            end
            if not found then
                table.insert(avatars, tempTable)
            end
        end
    end
end

function UpdateGroups()
    local groups = {}

    groups = import('/lua/ui/game/selection.lua').selectionSets
    
    for i, v in avatars do
        if v.Type == 'Group' then
            local found = false
            for h, x in groups do
                if h == v.ID and table.getsize(v.Units) > 0 then
                    found = true
                    break
                end
            end
            if not found then
                table.insert(dirtyTable, i)
            end
        end
    end
    for i, v in groups do
        if tonumber(i) and tonumber(i) >= 0 and tonumber(i) <= 9 then
            if table.getsize(v) > 0 then
                local found = false
                local tempTable = {
                    Type = 'Group',
                    ID = i,
                    Tab = LOCF("<LOC avatar_0001>%d", i),
                    Units = {},
                }
                for index, unit in v do
                    if unit != nil and not unit:IsDead() then
                        table.insert(tempTable.Units, unit)
                    end
                end
                if table.getsize(tempTable.Units) > 0 then
                    tempTable.Label = LOCF("<LOC avatar_0002>%d", table.getsize(tempTable.Units))
                    for h, x in avatars do
                        if x.Type == 'Group' and x.ID == i then
                            found = true
                            if x.icon.Update then
                                x.icon.Update(tempTable)
                            end
                            break
                        end
                    end
                    if not found then
                        table.insert(avatars, tempTable)
                    end
                else
                    for h, x in avatars do
                        if x.Type == 'Group' and x.ID == i then
                            table.insert(dirtyTable, h)
                        end
                    end
                end
            end
        end
    end
end

function UpdateAvatarIcons()
    if not animationLock and showAvatars then
        for i, v in dirtyTable do
            if avatars[v] and avatars[v].icon then
                avatars[v].icon:Destroy()
                table.remove(avatars, v)
            end
        end
        for i, v in avatars do
            local index = i
            if not v.icon then
                v.icon = CreateAvatar(v, index)
            end
        end
        LayoutAvatars()
    end
end

function LayoutAvatars()
    local index = 1
    avatarKey[index] = {}
    table.sort(avatars, function(a,b)
        if a.Type == 'Engineer' and b.Type == 'Engineer' then
            return a.ID > b.ID
        else
            return a.ID < b.ID
        end
    end)
    index = LayoutSort("Avatar", index)
    index = LayoutSort("Engineer", index)
    index = LayoutSort("Group", index)
end

function LayoutSort(Type, Index)
    local index = Index
    for i, v in avatars do
        local tempindex = index
        if v.Type == Type then
            if index == 1 then
                v.icon.Right:Set(avatarContainer.Right)
                v.icon.Top:Set(avatarContainer.Top)
            else
                local scoreTop =  avatarContainer.Bottom() -- import('/lua/modules/ui/game/score.lua').GetScoreTop() or
                if v.icon.Height() + avatarKey[index-1].icon.Bottom() > scoreTop then
                    v.icon.Right:Set(function() return avatarKey[tempindex-1].icon.Left() + 5 end)
                    v.icon.Top:Set(avatarContainer.Top)
                else
                    v.icon.Right:Set(avatarKey[index-1].icon.Right)
                    v.icon.Top:Set(function() return avatarKey[tempindex-1].icon.Bottom() - 5 end)
                end
            end
            avatarKey[index] = v
            index = index + 1
        end
    end
    return index
end

function CreateAvatar(Data, Index)
    local bg = Button(avatarContainer,
        UIUtil.UIFile('/game/avatar/avatar_bmp.dds'),
        UIUtil.UIFile('/game/avatar/avatar_bmp.dds'),
        UIUtil.UIFile('/game/avatar/avatar_bmp.dds'),
        UIUtil.UIFile('/game/avatar/avatar_bmp.dds'))

    bg.UnitIndex = 1
    bg.Units = Data.Units
	bg.Width:Set(60)
	bg.Height:Set(70)
    local function CommonHandler(control, event, data)
        if event.Type == 'MouseEnter' then
            CreateMouseover(control, data)
        elseif event.Type == 'MouseExit' then
            CloseMouseover()
        end
    end
    if Data.Type == 'Avatar' or Data.Type == 'Engineer' then
        bg.HandleEvent = function(self, event)
            if event.Type == "ButtonPress" or event.Type == "ButtonDClick" then
                if self.Units[self.UnitIndex + 1] then
                    self.UnitIndex = self.UnitIndex + 1
                else
                    self.UnitIndex = 1
                end
                if event.Modifiers.Left then
                    if event.Type == "ButtonPress" then
                        if event.Modifiers.Shift then
                            local curUnits = GetSelectedUnits()
                            if curUnits then
                                table.insert(curUnits, self.Units[self.UnitIndex])
                            else
                                curUnits = {self.Units[self.UnitIndex]}
                            end
                            SelectUnits(curUnits)
                        else
                            SelectUnits({self.Units[self.UnitIndex]})
                        end
                    else
                        if event.Modifiers.Shift then
                            local curUnits = GetSelectedUnits()
                            if curUnits then
                                table.insert(curUnits, self.Units[self.UnitIndex])
                            else
                                curUnits = {self.Units[self.UnitIndex]}
                            end
                            SelectUnits(curUnits)
                        else
                            SelectUnits(self.Units)
                        end
                    end
                elseif event.Modifiers.Right then
                    if event.Modifiers.Shift then
                        local curUnits = GetSelectedUnits()
                        if curUnits then
                            table.insert(curUnits, self.Units[self.UnitIndex])
                        else
                            curUnits = {self.Units[self.UnitIndex]}
                        end
                        UISelectAndZoomTo(self.Units[self.UnitIndex])
                        SelectUnits(curUnits)
                    else
                        UISelectAndZoomTo(self.Units[self.UnitIndex])
                    end
                end
            end
            CommonHandler(self, event, Data)
            return true
        end
    elseif Data.Type == 'Group' then
        bg.HandleEvent = function(self, event)
            if event.Type == "ButtonPress" or event.Type == "ButtonDClick" then
                if event.Modifiers.Left then
                    if self.Units[self.UnitIndex + 1] then
                        self.UnitIndex = self.UnitIndex + 1
                    else
                        self.UnitIndex = 1
                    end
                    if event.Type == "ButtonPress" then
                        SelectUnits(self.Units)
                    elseif event.Type == "ButtonDClick" then
                        UISelectAndZoomTo(self.Units[self.UnitIndex])
                        SelectUnits(self.Units)
                    end
                end
            end
            CommonHandler(self, event, Data)
            return true
        end
    end

    local icon = Bitmap(bg, GetAvatarIcon(Data))
    icon.Width:Set(47)--changed from 61
    icon.Height:Set(47)--changed from 61
    LayoutHelpers.AtCenterIn(icon, bg, -7, 0)

    if Data.Type == 'Avatar' then
        local healthbar = StatusBar(bg, 0, 1, false, false,
            UIUtil.UIFile('/game/avatar/health-bar-back_bmp.dds'),
            UIUtil.UIFile('/game/avatar/health-bar-green.dds'),
            true, "avatar RO Health Status Bar")
        healthbar.Left:Set(function() return bg.Left() + 10 end)
        healthbar.Right:Set(function() return bg.Right() - 10 end)
        healthbar.Bottom:Set(function() return bg.Bottom() - 6 end)
        healthbar.Top:Set(function() return healthbar.Bottom() - 10 end)
        healthbar.Height:Set(function() return healthbar.Bottom() - healthbar.Top() end)
        healthbar.Width:Set(function() return healthbar.Right() - healthbar.Left() end)
        healthbar:DisableHitTest(true)
        
        local alertSound = false
        if EntityCategoryFilterDown(categories.COMMAND, Data.Units[1]) then
            local name = ""
            local blueprintID = Data.Units[1]:GetBlueprint().BlueprintId
            local faction = string.sub(blueprintID, 2, 2)
    
            local factionData = import('/lua/factions.lua')
            local facKey = factionData.FactionDesToKey[string.upper(faction)]
            
            name = factionData.Factions[factionData.FactionIndexMap[facKey]].SoundPrefix
    
            alertSound = Sound {
                Bank = string.format('COMPUTER_%s_VO', string.upper(name)),
                Cue = string.format("%sComputer_Commanders_01314",name),
            }
        end
        
        local glow = Bitmap(bg, UIUtil.UIFile('/game/avatar/pulse-bars_bmp.dds'))
        LayoutHelpers.AtCenterIn(glow, bg, -7, 0)
        glow.Depth:Set(function() return bg.Depth() + 1 end)
        glow:DisableHitTest()
        glow.Time = 0
        glow.PhaseTime = 0
        glow:SetAlpha(0)
        glow.OnFrame = function(self, delta)
            if self.Time > 5 and self:GetAlpha() < .3 then
                self:SetAlpha(0)
                self:SetNeedsFrameUpdate(false)
            end
            self.Time = self.Time + delta
            self.PhaseTime = self.PhaseTime + delta
            self:SetAlpha(math.abs(math.sin(self.PhaseTime * 2)))
        end
        
        bg.Update = function(Data)
            local tempPrevHealth = healthbar._value()
            local tempHealth = Data.Units[1]:GetHealth()
            if tempPrevHealth > tempHealth then
                if not glow:NeedsFrameUpdate() then
                    if alertSound then
                        PlayVoice(alertSound)
                    end
                    glow:SetNeedsFrameUpdate(true)
                    glow.PhaseTime = 0
                end
                glow.Time = 0
            end
            healthbar:SetRange(0, Data.Units[1]:GetMaxHealth())
            healthbar:SetValue(tempHealth)
            if tempPrevHealth != tempHealth then
                SetHealthbarColor(healthbar, Data.Units[1]:GetHealth() / Data.Units[1]:GetMaxHealth())
            end
            bg.Units = Data.Units
        end
    elseif Data.Type == 'Engineer' or Data.Type == 'Group' then
        local label = UIUtil.CreateText(bg, Data.Label, 15, UIUtil.bodyFont)
        LayoutHelpers.AtHorizontalCenterIn(label, icon)
        label.Bottom:Set(function() return bg.Bottom() - 3 end)
        label:SetDropShadow(true)
        label:DisableHitTest()
        
        local tabBG = Bitmap(bg, UIUtil.UIFile('/game/avatar/tech-tab_bmp.dds'))
        LayoutHelpers.AtLeftTopIn(tabBG, bg, -16, 2)
        tabBG:DisableHitTest()
        
        local tab = UIUtil.CreateText(tabBG, Data.Tab, 14, UIUtil.bodyFont)
        LayoutHelpers.AtCenterIn(tab, tabBG, 0, -2)
        tab:SetDropShadow(true)
        tab:DisableHitTest()
        
        bg.Update = function(Data)
			if not table.equal(bg.Units, Data.Units) then
				icon:SetTexture(GetAvatarIcon(Data))
			end
            bg.Units = Data.Units
            label:SetText(Data.Label)
            tab:SetText(Data.Tab)
        end
    end
    
    return bg
end

function GetAvatarIcon(Data)
    if Data.Type == 'Avatar' then
        local blueprintID = Data.Units[1]:GetBlueprint().BlueprintId
        local blueprint = __blueprints[blueprintID]
        local iconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
        return iconName
    else
        local tempUnits = {}
        local iconName = ''
        tempUnits[1] = EntityCategoryFilterDown(categories.COMMAND, Data.Units)
        tempUnits[2] = EntityCategoryFilterDown(categories.EXPERIMENTAL, Data.Units)
        tempUnits[3] = EntityCategoryFilterDown(categories.TECH3, Data.Units)
        tempUnits[4] = EntityCategoryFilterDown(categories.TECH2, Data.Units)
        tempUnits[5] = EntityCategoryFilterDown(categories.TECH1, Data.Units)
        
        for i, v in tempUnits do
            if table.getsize(v) > 0 then
                if table.getsize(v) == 1 then
                    local blueprintID = v[1]:GetBlueprint().BlueprintId
                    local blueprint = __blueprints[blueprintID]
                    iconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
                    break
                else
                    local unitTable = {}
                    local topID = ''
                    local topIDNum = 0
                    for h, x in v do
                        local blueprintID = x:GetBlueprint().BlueprintId
                        local found = false
                        for unitID, count in unitTable do
                            if unitID == blueprintID then
                                count = count + 1
                            end
                        end
                        if not found then
                            unitTable[blueprintID] = 1
                        end
                    end
                    for h, x in unitTable do
                        if x > topIDNum then
                            topIDNum = x
                            topID = h
                        end
                    end
                    local blueprint = __blueprints[topID]
                    iconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
                    break
                end
            end
        end
        if iconName == '' then
            local blueprintID = Data.Units[1]:GetBlueprint().BlueprintId
            local blueprint = __blueprints[blueprintID]
            iconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
        end
        return iconName
    end
end

function SetHealthbarColor(control, value)
    if value > .75 then
        control._bar:SetTexture(UIUtil.UIFile('/game/avatar/health-bar-green.dds'))
    elseif value > .25 then
        control._bar:SetTexture(UIUtil.UIFile('/game/avatar/health-bar-yellow.dds'))
    elseif value > 0 then
        control._bar:SetTexture(UIUtil.UIFile('/game/avatar/health-bar-red.dds'))
    end
end