
local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Group = import('/lua/maui/group.lua').Group
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Button = import('/lua/maui/button.lua').Button
local StatusBar = import('/lua/maui/statusbar.lua').StatusBar
local parent = import('/lua/ui/game/economy.lua').savedParent
local pathui = '/mods/uimod/hook'
local pathuiimg='/mods/uimod/textures'
local sca_utils = '/mods/uimod/sca_utils/'
local UIManager = import(sca_utils .. 'uimanager.lua')
local skinning=import(sca_utils .. 'skinning.lua')

local profiles = GetPreference("profile.profiles") -- Gets our profiles table
local current = GetPreference("profile.current") -- gets the current profile
local Prefs = import('/lua/user/prefs.lua')--preferences

function CreateControls(controls, reinit)
	if controls.massBG.Resizer then
		controls.massBG.Resizer:Destroy()
		controls.massBG.Resizer = false
	end
	if controls.energyBG.Resizer then
		controls.energyBG.Resizer:Destroy()
		controls.energyBG.Resizer = false
	end
	if controls.massBG then
		controls.massBG:Destroy()
		controls.energyBG:Destroy()
		controls.massBG = false
		controls.energyBG = false
		controls.massDropOver = false
		controls.energyDropOver = false
        controls.massLabel = false
        controls.massRateContainer = false
        controls.massWarningBG = false
        controls.massDropdownTarget = false
        controls.energyLabel = false
        controls.energyRateContainer = false
        controls.energyWarningBG = false
        controls.energyDropdownTarget = false
	end
	if controls.genBG then
		controls.genBG:Destroy()
		controls.genBG = false 
	end

    if Prefs.GetFromCurrentProfile("uimanager_sav")["miniui_econ"]==1 or reinit then
		if not controls.group then
			controls.group = Group(parent, "economyBarGroup")
			controls.group:DisableHitTest()
		end
		
		controls.genBG = Group(parent)
		controls.genBGL = Bitmap(controls.genBG, UIUtil.UIFile('/game/mini-resource-back_bmp/center_bmp_l.dds'))
		controls.genBGR = Bitmap(controls.genBG, UIUtil.UIFile('/game/mini-resource-back_bmp/center_bmp_r.dds'))
		controls.genBGM = Bitmap(controls.genBG, UIUtil.UIFile('/game/mini-resource-back_bmp/center_bmp_m.dds'))
		controls.massMini = Group(controls.genBG)
		controls.massMini:DisableHitTest()
		controls.energyMini = Group(controls.genBG)
		controls.energyMini:DisableHitTest()
		
		controls.massDropdown = Group(controls.genBG)
		controls.massDropdown:DisableHitTest()
		
		controls.massIncome = UIUtil.CreateText(controls.genBG, "", 10, UIUtil.fixedFont)
		controls.massIncome:SetColor('ffb7e75f')
		controls.massIncome:SetDropShadow(true)

		controls.massExpense = UIUtil.CreateText(controls.genBG, "", 10, UIUtil.fixedFont)
		controls.massExpense:SetColor('fff30017')
		controls.massExpense:SetDropShadow(true)
		
		controls.massStatus = StatusBar(controls.genBG, 0, 100, false, false, UIUtil.UIFile('/game/resource-mini-bars/mini-mass-bar-back_bmp.dds'), UIUtil.UIFile('/game/resource-mini-bars/mini-mass-bar_bmp.dds'), false)
		
		controls.massRate = UIUtil.CreateText(controls.genBG, "", 20, UIUtil.bodyFont)
		controls.massRate:SetDropShadow(true)
		
		controls.massWarning = UIUtil.CreateText(controls.genBG, LOC("<LOC Econ_0000>Mass Needed"), 16, UIUtil.titleFont)   
		
		controls.massStorage = UIUtil.CreateText(controls.genBG, "", 10, UIUtil.bodyFont)
		controls.massStorage:SetColor('ffb7e75f')
		controls.massStorage:SetDropShadow(true)

		-- energy bar setup
		controls.energyDropdown = Group(controls.genBG)
		controls.energyDropdown:DisableHitTest()
		
		controls.energyIncome = UIUtil.CreateText(controls.genBG, "", 10, UIUtil.fixedFont)
		controls.energyIncome:SetColor('ffb7e75f')
		controls.energyIncome:SetDropShadow(true)
	   
		controls.energyExpense = UIUtil.CreateText(controls.genBG, "", 10, UIUtil.fixedFont)
		controls.energyExpense:SetColor('fff30017')
		controls.energyExpense:SetDropShadow(true)

		controls.energyStatus = StatusBar(controls.genBG, 0, 100, false, false, UIUtil.UIFile('/game/resource-mini-bars/mini-energy-bar-back_bmp.dds'), UIUtil.UIFile('/game/resource-mini-bars/mini-energy-bar_bmp.dds'), false)
		
		controls.energyStorage = UIUtil.CreateText(controls.genBG, "", 10, UIUtil.bodyFont)
		controls.energyStorage:SetColor('fff7c70f')
		controls.energyStorage:SetDropShadow(true)
				
		controls.energyRate = UIUtil.CreateText(controls.genBG, "", 20, UIUtil.bodyFont)
		controls.energyRate:SetDropShadow(true)

		controls.energyWarning = UIUtil.CreateText(controls.genBG, LOC("<LOC Econ_0001>Energy Needed"), 16, UIUtil.titleFont)
		
	else
		
		if not controls.group then
			controls.group = Group(parent, "economyBarGroup")
			controls.group:DisableHitTest()
		end

		-- mass setup
		controls.massBG = Bitmap(controls.group)
		UIManager.dragOnUI(controls.massBG, "mass")
		controls.massDropdown = Bitmap(controls.massBG)
		controls.massDropOver = Bitmap(controls.massBG)
		controls.massIncome = UIUtil.CreateText(controls.massBG, "", 10, UIUtil.fixedFont)
		controls.massExpense = UIUtil.CreateText(controls.massBG, "", 10, UIUtil.fixedFont)
		controls.massLabel = Bitmap(controls.massBG)
		controls.massRateContainer = Button(controls.massBG, nil, nil, nil, nil, "UI_Economy_Click", "UI_Economy_Rollover")
		controls.massRate = UIUtil.CreateText(controls.massBG, "", 20, UIUtil.bodyFont)
		controls.massWarning = UIUtil.CreateText(controls.massBG, LOC("<LOC Econ_0000>Mass Needed"), 16, UIUtil.titleFont)   
		controls.massWarningBG = Bitmap(controls.massBG)
		-- this exists only to act as a hit target for the dropdown data
		controls.massDropdownTarget = Group(controls.massBG, "massDropdown")
		controls.massStorage = UIUtil.CreateText(controls.massBG, "", 12, UIUtil.bodyFont)

		controls.massIncome:DisableHitTest()
		controls.massRate:DisableHitTest()
		controls.massExpense:DisableHitTest()
		controls.massWarning:Hide()
		controls.massWarning:DisableHitTest(true)
		controls.massDropOver:Hide()

		-- energy bar setup

		controls.energyBG = Bitmap(controls.group)
		UIManager.dragOnUI(controls.energyBG, "energy")
		controls.energyDropdown = Bitmap(controls.energyBG)
		controls.energyDropOver = Bitmap(controls.energyBG)
		controls.energyIncome = UIUtil.CreateText(controls.energyBG, "", 10, UIUtil.fixedFont)
		controls.energyExpense = UIUtil.CreateText(controls.energyBG, "", 10, UIUtil.fixedFont)
		controls.energyLabel = Bitmap(controls.energyBG)
		controls.energyRateContainer = Button(controls.energyBG, nil, nil, nil, nil, "UI_Economy_Click", "UI_Economy_Rollover")
		controls.energyRate = UIUtil.CreateText(controls.energyBG, "", 20, UIUtil.bodyFont)
		controls.energyWarning = UIUtil.CreateText(controls.energyBG, LOC("<LOC Econ_0000>Energy Needed"), 16, UIUtil.titleFont)   
		controls.energyWarningBG = Bitmap(controls.energyBG)
		-- this exists only to act as a hit target for the dropdown data
		controls.energyDropdownTarget = Group(controls.energyBG, "energyDropdown")
		controls.energyStorage = UIUtil.CreateText(controls.energyBG, "", 12, UIUtil.bodyFont)

		controls.energyIncome:DisableHitTest()
		controls.energyRate:DisableHitTest()
		controls.energyExpense:DisableHitTest()
		controls.energyWarning:Hide()
		controls.energyWarning:DisableHitTest(true)
		controls.energyDropOver:Hide()

	end
end

function SetLayout()
	local controls = import('/lua/ui/game/economy.lua').controls
	CreateControls(import('/lua/ui/game/economy.lua').controls)

	if Prefs.GetFromCurrentProfile("uimanager_sav")["miniui_econ"]==1 then
		
		--Set Up mass controls
		LayoutHelpers.AtRightTopIn(controls.massStatus, controls.massMini, 0, 5)
		LayoutHelpers.ResetLeft(controls.massStatus)
		LayoutHelpers.CenteredBelow(controls.massStorage, controls.massStatus)
		LayoutHelpers.LeftOf(controls.massRate, controls.massStatus, 5)
		LayoutHelpers.ResetLeft(controls.massRate)
		
		controls.massDropdown.Width:Set(50)
		controls.massDropdown.Bottom:Set(controls.massStorage.Bottom)
		LayoutHelpers.LeftOf(controls.massDropdown, controls.massRate)
		LayoutHelpers.ResetLeft(controls.massDropdown)
		
		LayoutHelpers.AtHorizontalCenterIn(controls.massIncome, controls.massDropdown)
		LayoutHelpers.AtHorizontalCenterIn(controls.massExpense, controls.massDropdown)
		LayoutHelpers.AtTopIn(controls.massIncome, controls.massDropdown)
		LayoutHelpers.AtBottomIn(controls.massExpense, controls.massDropdown)
			
		controls.massWarning.Depth:Set(100)
		LayoutHelpers.Below(controls.massWarning, controls.massStorage, -2)
		controls.massWarning.Right:Set(controls.massMini.Right)
		LayoutHelpers.ResetLeft(controls.massWarning)
		controls.massWarning:SetDropShadow(true)
		controls.massWarning:Hide()
		controls.massWarning:DisableHitTest(true)
		
		--Set up energy controls
		LayoutHelpers.AtLeftTopIn(controls.energyStatus, controls.energyMini, 0, 5)
		LayoutHelpers.CenteredBelow(controls.energyStorage, controls.energyStatus)
		LayoutHelpers.RightOf(controls.energyRate, controls.energyStatus)
		
		controls.energyDropdown.Width:Set(50)
		LayoutHelpers.RightOf(controls.energyDropdown, controls.energyRate)
		controls.energyDropdown.Bottom:Set(controls.energyStorage.Bottom)
		
		LayoutHelpers.AtHorizontalCenterIn(controls.energyIncome, controls.energyDropdown)
		LayoutHelpers.AtHorizontalCenterIn(controls.energyExpense, controls.energyDropdown)
		LayoutHelpers.AtTopIn(controls.energyIncome, controls.energyDropdown)
		LayoutHelpers.AtBottomIn(controls.energyExpense, controls.energyDropdown)
		
		controls.energyWarning.Depth:Set(100)
		LayoutHelpers.Below(controls.energyWarning, controls.energyStorage, -2)
		controls.energyWarning.Left:Set(controls.energyMini.Left)
		controls.energyWarning:SetDropShadow(true)
		controls.energyWarning:Hide()
		controls.energyWarning:DisableHitTest(true)
		
		--Set up parent group
		controls.massMini.Width:Set(function() return
			controls.massDropdown.Width() +
			controls.massStatus.Width() +
			45
		end)
		controls.massMini.Height:Set(function() return 
			50
		end)
		
		controls.energyMini.Width:Set(function() return
			controls.massDropdown.Width()  +
			controls.energyStatus.Width() + 
			45
		end)
		controls.energyMini.Height:Set(50)
		
		controls.energyMini.Top:Set(parent.Top)
		controls.massMini.Top:Set(parent.Top)
		controls.energyMini.Left:Set(function() return (parent.Width() / 2) + 20 end)
		controls.massMini.Right:Set(function() return (parent.Width() / 2) - 20 end)
		
		controls.genBG.Width:Set(function() return controls.energyMini.Right() - controls.massMini.Left() end)
		controls.genBG.Height:Set(controls.massMini.Height)
		LayoutHelpers.AtHorizontalCenterIn(controls.genBG, parent)
		LayoutHelpers.AtTopIn(controls.genBG, parent)
		
		LayoutHelpers.AtLeftTopIn(controls.genBGL, controls.massMini, -3)
		LayoutHelpers.AtRightTopIn(controls.genBGR, controls.energyMini)
		controls.genBGM.Top:Set(controls.genBGL.Top)
		controls.genBGM.Left:Set(controls.genBGL.Right)
		controls.genBGM.Right:Set(controls.genBGR.Left)
		
		controls.massIncome.Depth:Set(function() return controls.group.Depth() + 10 end)
		controls.massExpense.Depth:Set(function() return controls.group.Depth() + 10 end)
		controls.massRate.Depth:Set(function() return controls.group.Depth() + 10 end)
		controls.massStorage.Depth:Set(function() return controls.group.Depth() + 10 end)
		controls.energyIncome.Depth:Set(function() return controls.group.Depth() + 10 end)
		controls.energyExpense.Depth:Set(function() return controls.group.Depth() + 10 end)
		controls.energyRate.Depth:Set(function() return controls.group.Depth() + 10 end)
		controls.energyStorage.Depth:Set(function() return controls.group.Depth() + 10 end)
		
		LayoutHelpers.FillParent(controls.group, controls.genBG)
		
	else
		controls.massBG:SetTexture(pathuiimg.. '/blank.dds')
		controls.energyBG:SetTexture(pathuiimg.. '/blank.dds')
		controls.massBG.Depth:Set(200)
		controls.energyBG.Depth:Set(300)
		
		controls.massStorage.Depth:Set(function() return controls.massBG.Depth() + 10 end)
		controls.massRateContainer.Depth:Set(function() return controls.massBG.Depth() + 5 end)
		controls.massExpense.Depth:Set(function() return controls.massBG.Depth() + 10 end)
		controls.massIncome.Depth:Set(function() return controls.massBG.Depth() + 10 end)
		controls.massRate.Depth:Set(function() return controls.massBG.Depth() + 10 end)
		--controls.massTitle.Depth:Set(function() return controls.massBG.Depth() + 10 end)
		controls.massWarning.Depth:Set(function() return controls.massBG.Depth() + 10 end)
		controls.massDropdown.Depth:Set(function() return controls.massBG.Depth() + 5 end)
		controls.massDropOver.Depth:Set(function() return controls.massBG.Depth() + 10 end)
		controls.massLabel.Depth:Set(function() return controls.massBG.Depth() + 10 end)
		controls.massWarningBG.Depth:Set(function() return controls.massBG.Depth() + 15 end)
		controls.massDropdownTarget.Depth:Set(function() return controls.massBG.Depth() + 15 end)
		
		controls.energyStorage.Depth:Set(function() return controls.energyBG.Depth() + 10 end)
		controls.energyRateContainer.Depth:Set(function() return controls.energyBG.Depth() + 5 end)
		controls.energyExpense.Depth:Set(function() return controls.energyBG.Depth() + 10 end)
		controls.energyIncome.Depth:Set(function() return controls.energyBG.Depth() + 10 end)
		controls.energyRate.Depth:Set(function() return controls.energyBG.Depth() + 10 end)
		--controls.energyTitle.Depth:Set(function() return controls.energyBG.Depth() + 10 end)
		controls.energyWarning.Depth:Set(function() return controls.energyBG.Depth() + 10 end)
		controls.energyDropdown.Depth:Set(function() return controls.energyBG.Depth() + 5 end)
		controls.energyDropOver.Depth:Set(function() return controls.energyBG.Depth() + 10 end)
		controls.energyLabel.Depth:Set(function() return controls.energyBG.Depth() + 10 end)
		controls.energyWarningBG.Depth:Set(function() return controls.energyBG.Depth() + 15 end)
		controls.energyDropdownTarget.Depth:Set(function() return controls.energyBG.Depth() + 15 end)

		LayoutHelpers.FillParent(controls.group, parent)
		
		
		--position mass controls
		
		
		if Prefs.GetFromCurrentProfile("uimanager_sav")["verticalmass"]==1 then
			skinning.GetSkin("mass", controls.massBG, controls.massBG, "verticalbg")
			controls.massStatus = skinning.SetStatusBar("mass", controls.massBG, "storagebar_v")
			controls.massStorageMax = UIUtil.CreateText(controls.massBG, "", 11, UIUtil.bodyFont)
			controls.massRatePos = skinning.SetStatusBar("mass", controls.massBG, "rateposbar")
			controls.massRateNeg = skinning.SetStatusBar("mass", controls.massBG, "ratenegbar")
			skinning.GetSkin("mass", controls.massDropdown, controls.massBG, "dropdown_v")
			skinning.GetSkin("mass", controls.massDropOver, controls.massDropdown, "dropover_v")
			skinning.SetText("mass", controls.massStorage, controls.massDropdown, "storage_v")
			skinning.SetText("mass", controls.massStorageMax, controls.massDropdown, "storageMax_v")
			skinning.GetSkin("mass", controls.massRateContainer, controls.massBG, "rateContainer_v")
			skinning.SetText("mass", controls.massRate, controls.massRateContainer, "rate_v")
			skinning.SetText("mass", controls.massIncome, controls.massRateContainer, "income_v")
			skinning.SetText("mass", controls.massExpense, controls.massRateContainer, "expense_v")
			skinning.SetText("mass", controls.massWarning, controls.massBG, "warningText_v")
			controls.massWarning:SetDropShadow(false)
			skinning.GetSkin("mass", controls.massLabel, controls.massBG, "label_v")
			skinning.GetSkin("mass", controls.massWarningBG, controls.massLabel, "warning_v")
			LayoutHelpers.FillParent(controls.massDropdownTarget, controls.massDropdown)
			controls.massStorageMax.Depth:Set(function() return controls.massBG.Depth() + 10 end)
			controls.massRatePos.Depth:Set(function() return controls.massBG.Depth() + 5 end)
			controls.massRateNeg.Depth:Set(function() return controls.massBG.Depth() + 5 end)
		else
			skinning.GetSkin("mass", controls.massBG, controls.massBG, "bg")
			controls.massStatus = skinning.SetStatusBar("mass", controls.massBG, "storagebar")
			skinning.GetSkin("mass", controls.massDropdown, controls.massBG, "dropdown")
			skinning.GetSkin("mass", controls.massDropOver, controls.massDropdown, "dropover")
			skinning.SetText("mass", controls.massStorage, controls.massStatus, "storage")
			skinning.GetSkin("mass", controls.massRateContainer, controls.massBG, "rateContainer")
			skinning.SetText("mass", controls.massRate, controls.massRateContainer, "rate")
			skinning.SetText("mass", controls.massIncome, controls.massDropdown, "income")
			skinning.SetText("mass", controls.massExpense, controls.massDropdown, "expense")
			skinning.SetText("mass", controls.massWarning, controls.massStatus, "warningText")
			controls.massWarning:SetDropShadow(true)
			skinning.GetSkin("mass", controls.massLabel, controls.massBG, "label")
			skinning.GetSkin("mass", controls.massWarningBG, controls.massLabel, "warning")
			LayoutHelpers.FillParent(controls.massDropdownTarget, controls.massDropdown)
		end
		controls.massStatus.Depth:Set(function() return controls.massBG.Depth() + 5 end)
		
		
		--position energy controls
		
		
		if Prefs.GetFromCurrentProfile("uimanager_sav")["verticalenergy"]==1 then
			skinning.GetSkin("energy", controls.energyBG, controls.energyBG, "verticalbg")
			controls.energyStatus = skinning.SetStatusBar("energy", controls.energyBG, "storagebar_v")
			controls.energyStorageMax = UIUtil.CreateText(controls.energyBG, "", 11, UIUtil.bodyFont)
			controls.energyRatePos = skinning.SetStatusBar("energy", controls.energyBG, "rateposbar")
			controls.energyRateNeg = skinning.SetStatusBar("energy", controls.energyBG, "ratenegbar")
			skinning.GetSkin("energy", controls.energyDropdown, controls.energyBG, "dropdown_v")
			skinning.GetSkin("energy", controls.energyDropOver, controls.energyDropdown, "dropover_v")
			skinning.SetText("energy", controls.energyStorage, controls.energyDropdown, "storage_v")
			skinning.SetText("energy", controls.energyStorageMax, controls.energyDropdown, "storageMax_v")
			skinning.GetSkin("energy", controls.energyRateContainer, controls.energyBG, "rateContainer_v")
			skinning.SetText("energy", controls.energyRate, controls.energyRateContainer, "rate_v")
			skinning.SetText("energy", controls.energyIncome, controls.energyRateContainer, "income_v")
			skinning.SetText("energy", controls.energyExpense, controls.energyRateContainer, "expense_v")
			skinning.SetText("energy", controls.energyWarning, controls.energyBG, "warningText_v")
			controls.energyWarning:SetDropShadow(false)
			skinning.GetSkin("energy", controls.energyLabel, controls.energyBG, "label_v")
			skinning.GetSkin("energy", controls.energyWarningBG, controls.energyLabel, "warning_v")
			LayoutHelpers.FillParent(controls.energyDropdownTarget, controls.energyDropdown)
			controls.energyStorageMax.Depth:Set(function() return controls.energyBG.Depth() + 10 end)
			controls.energyRatePos.Depth:Set(function() return controls.energyBG.Depth() + 5 end)
			controls.energyRateNeg.Depth:Set(function() return controls.energyBG.Depth() + 5 end)
		elseif Prefs.GetFromCurrentProfile("uimanager_sav")["verticalenergy"]==0 and Prefs.GetFromCurrentProfile("uimanager_sav")["inverse_barre"]==1 then
			skinning.GetSkin("energy", controls.energyBG, controls.energyBG, "bg_i")
			controls.energyStatus = skinning.SetStatusBar("energy", controls.energyBG, "storagebar_i")
			skinning.GetSkin("energy", controls.energyDropdown, controls.energyBG, "dropdown_i")
			skinning.GetSkin("energy", controls.energyDropOver, controls.energyDropdown, "dropover_i")
			skinning.SetText("energy", controls.energyStorage, controls.energyStatus, "storage_i")
			skinning.GetSkin("energy", controls.energyRateContainer, controls.energyBG, "rateContainer_i")
			skinning.SetText("energy", controls.energyRate, controls.energyRateContainer, "rate_i")
			skinning.SetText("energy", controls.energyIncome, controls.energyDropdown, "income_i")
			skinning.SetText("energy", controls.energyExpense, controls.energyDropdown, "expense_i")
			skinning.SetText("energy", controls.energyWarning, controls.energyStatus, "warningText_i")
			controls.energyWarning:SetDropShadow(true)
			skinning.GetSkin("energy", controls.energyLabel, controls.energyBG, "label_i")
			skinning.GetSkin("energy", controls.energyWarningBG, controls.energyLabel, "warning_i")
			LayoutHelpers.FillParent(controls.energyDropdownTarget, controls.energyDropdown)
		else
			skinning.GetSkin("energy", controls.energyBG, controls.energyBG, "bg")
			controls.energyStatus = skinning.SetStatusBar("energy", controls.energyBG, "storagebar")
			skinning.GetSkin("energy", controls.energyDropdown, controls.energyBG, "dropdown")
			skinning.GetSkin("energy", controls.energyDropOver, controls.energyDropdown, "dropover")
			skinning.SetText("energy", controls.energyStorage, controls.energyStatus, "storage")
			skinning.GetSkin("energy", controls.energyRateContainer, controls.energyBG, "rateContainer")
			skinning.SetText("energy", controls.energyRate, controls.energyRateContainer, "rate")
			skinning.SetText("energy", controls.energyIncome, controls.energyDropdown, "income")
			skinning.SetText("energy", controls.energyExpense, controls.energyDropdown, "expense")
			skinning.SetText("energy", controls.energyWarning, controls.energyStatus, "warningText")
			controls.energyWarning:SetDropShadow(true)
			skinning.GetSkin("energy", controls.energyLabel, controls.energyBG, "label")
			skinning.GetSkin("energy", controls.energyWarningBG, controls.energyLabel, "warning")
			LayoutHelpers.FillParent(controls.energyDropdownTarget, controls.energyDropdown)
		end
		controls.energyStatus.Depth:Set(function() return controls.energyBG.Depth() + 5 end)

		controls.massWarningBG:SetAlpha(0)
		controls.energyWarningBG:SetAlpha(0)
			
	end
end
