local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Button = import('/lua/maui/button.lua').Button
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Group = import('/lua/maui/group.lua').Group
local CampMgr = import('/lua/ui/campaign/campaignmanager.lua')
local profiles = GetPreference("profile.profiles") -- Gets our profiles table
local current = GetPreference("profile.current") -- gets the current profile
local Prefs = import('/lua/user/prefs.lua')--preferences
local pathuiimg='/mods/uimod/textures'
local GUI = import('/lua/ui/game/options.lua').GUI
local Tooltip = import('/lua/ui/game/tooltip.lua')
local sca_utils = '/mods/uimod/sca_utils/'
local UIManager = import(sca_utils .. 'uimanager.lua')
local skinning=import(sca_utils .. 'skinning.lua')


function SetLayout()
	if Prefs.GetFromCurrentProfile("uimanager_sav")["miniui_econ"]==1 then
	    local buttondata = import('/lua/ui/game/options.lua').buttonData
	    local econControls = import('/lua/ui/game/economy.lua').controls
	    local econparent = import('/lua/ui/game/economy.lua').savedParent
	    
	    local function CreateButton(data)
	        local btn 
	        if data.checkbox then
	            btn = Checkbox(GUI.bg,
	                UIUtil.UIFile(data.on..'_btn_up.dds'),
	                UIUtil.UIFile(data.off..'_btn_down.dds'),
	                UIUtil.UIFile(data.on..'_btn_over.dds'),
	                UIUtil.UIFile(data.off..'_btn_over.dds'),
	                UIUtil.UIFile(data.on..'_btn_dis.dds'),
	                UIUtil.UIFile(data.off..'_btn_dis.dds'),
	                "UI_MFD_Click", "UI_MFD_Rollover")
	            if data.glow then
	                btn.glow = Bitmap(btn, UIUtil.UIFile('/game/options_btn/glow_bmp.dds'))
	                LayoutHelpers.AtCenterIn(btn.glow, btn)
	                btn.glow:SetAlpha(0)
	                btn.glow:DisableHitTest()
	            end
	        else
	            btn = UIUtil.CreateButtonStd(GUI.bg, data.texture, "", 12, 0, 0, "NO_SOUND", "UI_Options_Rollover")
	        end
	        btn.data = data
	        return btn
	    end
	    
		if GUI.bg.Resizer then GUI.bg.Resizer:Destroy() GUI.bg.Resizer = false end
	    if GUI.bg then GUI.bg:Destroy() GUI.bg = false end
	    if GUI.leftBg then GUI.leftBg:Destroy() GUI.leftBg = false end
	    if GUI.rightBg then GUI.rightBg:Destroy() GUI.rightBg = false end
	    
	    if not GUI.hideBtn then
	        GUI.hideBtn = UIUtil.CreateCheckbox(GUI.parent, 
	        UIUtil.UIFile('/game/min-control-tab-btn/tab-close_btn_up.dds'),
	        UIUtil.UIFile('/game/min-control-tab-btn/tab-open_btn_up.dds'),
	        UIUtil.UIFile('/game/min-control-tab-btn/tab-close_btn_over.dds'),
	        UIUtil.UIFile('/game/min-control-tab-btn/tab-open_btn_over.dds'),
	        UIUtil.UIFile('/game/min-control-tab-btn/tab-close_btn_dis.dds'),
	        UIUtil.UIFile('/game/min-control-tab-btn/tab-open_btn_dis.dds'), 
	        'UI_Avatar_Hide')
	    end
	    Tooltip.AddCheckboxTooltip(GUI.hideBtn, 'toggle_mini_expanded_options')
	    GUI.hideBtn.Depth:Set(function() return econControls.genBG.Depth() + 10 end)
	    GUI.hideBtn.OnCheck = function(self, checked)
	        GUI.bg:SetHidden(checked)
	        if checked then
	            econControls.energyMini.Left:Set(function() return (econparent.Width() / 2) + 20 end)
	            econControls.massMini.Right:Set(function() return (econparent.Width() / 2) - 20 end)
	        else
	            econControls.energyMini.Left:Set(function() return (econparent.Width() / 2) + 125 end)
	            econControls.massMini.Right:Set(function() return (econparent.Width() / 2) - 125 end)
	        end
	    end
	    
	    if not GUI.bg then
	        GUI.bg = Group(GetFrame(0))
	    end
	    GUI.bg.Depth:Set(function() return econControls.group.Depth() + 100 end)

	    GUI.buttons = {}
	    GUI.buttonsmfd = {}
	    for i, v in buttondata.options do
	        local index = i
	        if not GUI.buttons[index] then
	            GUI.buttons[index] = CreateButton(v)
	            GUI.buttons[index].Height:Set(32)
	            GUI.buttons[index].Width:Set(38)
	        end
	        if index == 1 then
	            GUI.buttons[index].Left:Set(function() return GUI.bg.Left() + 1 end)
	            GUI.buttons[index].Top:Set(function() return GUI.bg.Top() + 5 end)
	        else
	            LayoutHelpers.RightOf(GUI.buttons[index], GUI.buttons[index-1], 0)
	        end
	        if import('/lua/ui/campaign/campaignmanager.lua').campaignMode and v.disableInCampaign then
	            GUI.buttons[index]:Disable()
	        end
	    end
	    
	    GUI.mfdCenterGroup = Bitmap(GUI.bg, UIUtil.UIFile('/game/mini-filter-back_bmp/min-filter-back_bmp.dds'))
	    GUI.mfdCenterGroup:DisableHitTest()
	    
	    for i, v in buttondata.mfd do
	        local index = i
	        if not GUI.buttonsmfd[index] then
	            GUI.buttonsmfd[index] = CreateButton(v)
	            GUI.buttonsmfd[index].Height:Set(34)
	            GUI.buttonsmfd[index].Width:Set(55)
	        end
	        if index == 1 then
	            LayoutHelpers.AtLeftTopIn(GUI.buttonsmfd[index], GUI.mfdCenterGroup, -7, -2)
	        else
	            LayoutHelpers.RightOf(GUI.buttonsmfd[index], GUI.buttonsmfd[index-1], -19)
	        end
	    end
	    
	    LayoutHelpers.Below(GUI.mfdCenterGroup, GUI.buttons[1], 2)
	    LayoutHelpers.AtHorizontalCenterIn(GUI.mfdCenterGroup, GUI.bg)
	    GUI.bg.Width:Set(function() return table.getn(GUI.buttons) * GUI.buttons[1].Width() end)
	    GUI.bg.Height:Set(60)
	    
	    LayoutHelpers.AtHorizontalCenterIn(GUI.hideBtn, econControls.genBG)
	    LayoutHelpers.AtTopIn(GUI.hideBtn, econControls.genBG, -5)
	    LayoutHelpers.CenteredBelow(GUI.bg, GUI.hideBtn, -6)
	    GUI.hideBtn:SetCheck(true)
	    GUI.hideBtn.OnHide = function(self, hidden)
	        if hidden then
	            GUI.bg:Hide()
	        else
	            GUI.bg:SetHidden(self:IsChecked())
	        end
	    end
	else

		local buttondata = import('/lua/ui/game/options.lua').buttonData
		local econControls = import('/lua/ui/game/economy.lua').controls
		
		if GUI.bg.Resizer then GUI.bg.Resizer:Destroy() GUI.bg.Resizer = false end
		if GUI.bg then GUI.bg:Destroy() GUI.bg = false end
		if GUI.leftBg then GUI.leftBg:Destroy() GUI.leftBg = false end
		if GUI.rightBg then GUI.rightBg:Destroy() GUI.rightBg = false end
		if not GUI.bg then
			GUI.bg = Bitmap(GUI.parent)
			LayoutHelpers.AtLeftTopIn(GUI.bg, GUI.parent)
			GUI.bg.Depth:Set(400)
			UIManager.dragOnUI(GUI.bg, "options")
		end

		local function CreateButton(data)
			local btn 
			if data.checkbox then
				btn = Checkbox(GUI.bg, nil, nil, nil, nil, nil, nil, "NO_SOUND", "UI_Options_Rollover")
				btn.glow = Bitmap(btn, UIUtil.UIFile('/game/options_btn/glow_bmp.dds'))
				LayoutHelpers.AtCenterIn(btn.glow, btn)
				btn.glow:SetAlpha(0)
				btn.glow:DisableHitTest()
			else
				btn = Button(GUI.bg, nil, nil, nil, nil, "NO_SOUND", "UI_Options_Rollover")
			end
			btn.data = data
			return btn
		end

		if Prefs.GetFromCurrentProfile("uimanager_sav")["verticaloptions"]==1 then
			skinning.GetSkin("options", GUI.bg, GUI.bg, "vertical")
		
			GUI.buttons = {}
			GUI.buttonsmfd = {}
			for i, v in buttondata.options do
				local index = i
				if not GUI.buttons[index] then
					GUI.buttons[index] = CreateButton(v, GUI.bg)
					skinning.GetSkin("options",	GUI.buttons[index], GUI.bg, GUI.buttons[index].data.id.. '_v')
					GUI.buttons[index].Depth:Set(function() return GUI.bg.Depth() + 10 end)
				end
			end
			for i, v in GUI.buttonsmfd do
				if GUI.buttonsmfd[i] then
					GUI.buttonsmfd[i]:Destroy()
					GUI.buttonsmfd[i] = false
				end
			end
			if GUI.hideBtn then GUI.hideBtn:Destroy() GUI.hideBtn = false end
			if GUI.mfdCenterGroup then GUI.mfdCenterGroup:Destroy() GUI.mfdCenterGroup = false end
			GUI.bg:Show()
		else
			skinning.GetSkin("options", GUI.bg, GUI.bg, "horizontal")
	 
			GUI.buttons = {}
			GUI.buttonsmfd = {}
			for i, v in buttondata.options do
				local index = i
				if not GUI.buttons[index] then
					GUI.buttons[index] = CreateButton(v)
					skinning.GetSkin("options", GUI.buttons[index], GUI.bg, GUI.buttons[index].data.id)
					GUI.buttons[index].Depth:Set(function() return GUI.bg.Depth() + 10 end)
				end
			end
			for i, v in GUI.buttonsmfd do
				if GUI.buttonsmfd[i] then
					GUI.buttonsmfd[i]:Destroy()
					GUI.buttonsmfd[i] = false
				end
			end
			if GUI.hideBtn then GUI.hideBtn:Destroy() GUI.hideBtn = false end
			if GUI.mfdCenterGroup then GUI.mfdCenterGroup:Destroy() GUI.mfdCenterGroup = false end
			GUI.bg:Show()
		end
	end
end
