
local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Dragger = import('/lua/maui/dragger.lua').Dragger
local Combo = import('/lua/ui/controls/combo.lua').Combo
local IntegerSlider = import('/lua/maui/slider.lua').IntegerSlider
local sca_utils = '/mods/uimod/sca_utils/'
local UIManager = import(sca_utils .. 'uimanager.lua')

function CreateControls(controls)
	if controls.bg then controls.bg:Destroy() end
    controls.bg = Bitmap(controls.controlClusterGroup)
	controls.bg.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			--create the table of current marker positions
			local drag = Dragger()
			local offX = event.MouseX - self.Left()
			local offY = event.MouseY - self.Top()
			drag.OnMove = function(dragself, x, y)
				self.Left:Set(x - offX)
				self.Top:Set(y - offY)
				GetCursor():SetTexture(UIUtil.GetCursor('MOVE_WINDOW'))
			end
			drag.OnRelease = function(dragself)
				GetCursor():Reset()
				drag:Destroy()
			end
			PostDragger(self:GetRootFrame(), event.KeyCode, drag)
		end
	end

		
	controls.armycombo = Combo(controls.bg)
	controls.speedSlider = IntegerSlider(controls.bg, false, -10, 10, 1, 
		UIUtil.SkinnableFile('/slider02/slider_btn_up.dds'), 
		UIUtil.SkinnableFile('/slider02/slider_btn_over.dds'), 
		UIUtil.SkinnableFile('/slider02/slider_btn_down.dds'), 
		UIUtil.SkinnableFile('/dialogs/options/slider-back_bmp.dds'))    

	controls.currentSpeed = UIUtil.CreateText(controls.speedSlider, "", 12)
end

function SetLayout()
    CreateControls(import('/lua/ui/game/replay.lua').controls)
    
    local controls = import('/lua/ui/game/replay.lua').controls
    
    controls.bg:SetTexture(UIUtil.SkinnableFile('/game/replay/panel_bmp.dds'))

	controls.bg.Left:Set(250)
	controls.bg.Top:Set(450)
	LayoutHelpers.ResetRight(controls.bg)
	LayoutHelpers.ResetBottom(controls.bg)
    
    controls.armycombo.Left:Set(function() return controls.bg.Left() + 10 end)
    controls.armycombo.Top:Set(function() return controls.bg.Top() + 10 end)
    controls.armycombo.Width:Set(140)
    controls.armycombo.Height:Set(20)
        
    controls.speedSlider.Left:Set(function() return controls.bg.Left() + 10 end)
    controls.speedSlider.Right:Set(function() return controls.bg.Right() - 40 end)
    controls.speedSlider.Top:Set(function() return controls.bg.Bottom() - 40 end)
    controls.speedSlider._background.Left:Set(controls.speedSlider.Left)
    controls.speedSlider._background.Right:Set(controls.speedSlider.Right)
    controls.speedSlider._background.Top:Set(controls.speedSlider.Top)
    controls.speedSlider._background.Bottom:Set(controls.speedSlider.Bottom)
    
    LayoutHelpers.RightOf(controls.currentSpeed, controls.speedSlider)
end