local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Group = import('/lua/maui/group.lua').Group
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local GameCommon = import('/lua/ui/game/gamecommon.lua')
local ItemList = import('/lua/maui/itemlist.lua').ItemList
local Prefs = import('/lua/user/prefs.lua')

local profiles = GetPreference("profile.profiles") -- Gets our profiles table
local current = GetPreference("profile.current") -- gets the current profile

local pathui = '/mods/uimod/hook'
local UIManager= import('/mods/uimod/sca_utils/uimanager.lua')
local pathuiimg='/mods/uimod/textures'
local sca_utils = '/mods/uimod/sca_utils/'
local skinning=import(sca_utils .. 'skinning.lua')

function SetLayout()
	if Prefs.GetFromCurrentProfile("uimanager_sav")["new_unit_view"]==0 then
		SetLayout_sca()
	else
		SetLayout_mini()
	
	end
end

function CreateResourceGroup(parent, groupLabel)
    local group = Group(parent)
    
    # Group label
    group.Label = UIUtil.CreateText( group, groupLabel, 12, "Arial Bold" )
    group.Label:SetColor("FFBEBEBE")

    # Energy Icon
    group.EnergyIcon = Bitmap(group)
    group.EnergyIcon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-energy_bmp.dds'))

    # Energy Value
    group.EnergyValue = UIUtil.CreateText( group, "0", 12, UIUtil.bodyFont )
    group.EnergyValue:SetColor("FF00F000")

    # Mass Icon
    group.MassIcon = Bitmap(group)
    group.MassIcon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-mass_bmp.dds'))

    # Mass Value
    group.MassValue = UIUtil.CreateText( group, "0", 12, UIUtil.bodyFont )
    group.MassValue:SetColor("FF00F000")

    return group
end

# A 'Stat Group' is an icon or text label with a value on the right
# e.g.  Health 3000
#       Shield 6000
#       <icon> 24
function CreateStatGroup(parent, labelIcon)
    local group = Group(parent)
    
    group.Label = Bitmap(group)
    group.Label:SetTexture(labelIcon)

    group.Value = UIUtil.CreateText( group, "", 14, UIUtil.bodyFont )
    group.Value:SetColor("FF00F000")

    return group
end

function CreateTextbox(parent, label, bigBG)
    local group = Group(parent)
    
    if bigBG then
        group.BG = Bitmap(group)
        group.BGMid = Bitmap(group)
        group.BGBot = Bitmap(group)
        group.BG:SetTexture(UIUtil.UIFile('/game/build-ui-02/description_bmp_t.dds'))
        group.BGMid:SetTexture(UIUtil.UIFile('/game/build-ui-02/description_bmp_m.dds'))
        group.BGBot:SetTexture(UIUtil.UIFile('/game/build-ui-02/description_bmp_b.dds'))
    else
        group.bgtopleft = Bitmap(group)
        group.bgtopmiddle = Bitmap(group)
        group.bgtopright = Bitmap(group)
        
        group.bgmidleft = Bitmap(group)
        group.bgmidmiddle = Bitmap(group)
        group.bgmidright = Bitmap(group)
        
        group.bgbotleft = Bitmap(group)
        group.bgbotmiddle = Bitmap(group)
        group.bgbotright = Bitmap(group)
        
        group.bgtopleft:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_ul.dds'))
        group.bgtopmiddle:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_horz_um.dds'))
        group.bgtopright:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_ur.dds'))
        
        group.bgmidleft:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_vert_l.dds'))
        group.bgmidmiddle:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_m.dds'))
        group.bgmidright:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_vert_r.dds'))
        
        group.bgbotleft:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_ll.dds'))
        group.bgbotmiddle:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_lm.dds'))
        group.bgbotright:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_lr.dds'))
    end
    
    if label then
        group.Label = UIUtil.CreateText( group, LOC(label), 12, "Arial Bold" )
        group.Label:SetColor("FFBEBEBE")
    end

    group.Value = {}
    group.Value[1] = UIUtil.CreateText( group, "", 12, UIUtil.bodyFont)
    
    return group
end

function Create_mini(parent)
	# Main Window
    if not import('/lua/ui/game/unitviewDetail.lua').View then
        import('/lua/ui/game/unitviewDetail.lua').View = Group(parent)
    end
    
    local View = import('/lua/ui/game/unitviewDetail.lua').View
    
    if not View.BG then
        View.BG = Bitmap(View)
    end
    View.BG:SetTexture(UIUtil.UIFile(pathuiimg..'/blank.dds'))
	skinning.GetSkin("unitviewdetail", View.BG, View, "new")
    View.BG.Depth:Set(200)

    # Unit icon
    if false then
        View.UnitIcon = Bitmap(View)
    end
    
    if not View.UnitImg then
        View.UnitImg = Bitmap( View.BG )
    end
    
    # Unit Name
    if not View.UnitName then
        View.UnitName = UIUtil.CreateText( View.BG, "", 14, "Arial Bold" )
    end
    View.UnitName:SetColor("FFFF9E06")

    # Unit Description
    if not View.UnitShortDesc then
        View.UnitShortDesc = UIUtil.CreateText( View.BG, "", 14, UIUtil.bodyFont )
    end
    View.UnitShortDesc:SetColor("FFFF9E06")

    # Cost
    if not View.BuildCostGroup then
        View.BuildCostGroup = CreateResourceGroup(View.BG, "<LOC uvd_0000>Build Cost (Rate)")
    end

    # Upkeep
    if not View.UpkeepGroup then
        View.UpkeepGroup = CreateResourceGroup(View.BG, "<LOC uvd_0002>Yield")
    end
    
    # Health stat
    if not View.HealthStat then
        View.HealthStat = CreateStatGroup( View.BG, UIUtil.UIFile('/game/unit_view_icons/redcross.dds') )
    end
    
    if not View.ShieldStat then
        View.ShieldStat = CreateStatGroup( View.BG, UIUtil.UIFile('/game/unit_view_icons/shield.dds') )
    end
    # Tme stat
    if not View.TimeStat then
        View.TimeStat = CreateStatGroup( View.BG, UIUtil.UIFile('/game/unit-over/icon-clock_bmp.dds') )
    end
    
	if Prefs.GetFromCurrentProfile("uimanager_sav")["affiche_help"]==1 then
        # Description  "<LOC uvd_0003>Description"
        if not View.Description then
            View.Description = CreateTextbox(View.BG, nil, true)
        end
        
        # Ability List"<LOC _Abilities>"
        if not View.AbilityList then
            View.AbilityList = CreateTextbox(View.BG, nil)
        end
    else
        if View.Description then View.Description:Destroy() View.Description = false end
        if View.AbilityList then View.AbilityList:Destroy() View.AbilityList = false end
    end 
	
    View.BG:DisableHitTest(true)
end

function SetLayout_mini()
    import('/lua/ui/game/unitviewDetail.lua').ViewState = Prefs.GetOption('uvd_format')
    local mapGroup = import('/lua/ui/game/unitviewDetail.lua').MapView
    
    Create_mini(mapGroup)
    
    local control = import('/lua/ui/game/unitviewDetail.lua').View
	
	if UIManager.unitview then
		control.Left:Set(UIManager.unitview.Left)
		control.Bottom:Set(UIManager.unitview.Bottom)
	else
		LayoutHelpers.AtLeftIn(control, mapGroup, 0)
		LayoutHelpers.AtBottomIn(control, mapGroup, -2)
	end
	control.Width:Set( control.BG.Width )
    control.Height:Set( control.BG.Height )
    
    # Main window background
    LayoutHelpers.AtLeftTopIn( control.BG, control )

    # Unit Name
    LayoutHelpers.AtLeftTopIn( control.UnitName, control.BG, 20, 7 )
    
    # Unit Image
    LayoutHelpers.AtLeftTopIn( control.UnitImg, control.BG, 15, 33 )
    control.UnitImg.Height:Set(46)
    control.UnitImg.Width:Set(48)
    
    # Unit Description
    control.UnitShortDesc.Left:Set(function() return control.UnitName.Right() + 5 end)
    control.UnitShortDesc.Top:Set(control.UnitName.Top)

    # Health Stat
    LayoutHelpers.Below(control.HealthStat, control.UnitImg, 12)
    control.HealthStat.Left:Set(function() return control.UnitImg.Left() + 5 end)
    control.HealthStat.Height:Set(control.HealthStat.Label.Height)
    LayoutStatGroup_mini( control.HealthStat )
    
    LayoutHelpers.RightOf( control.TimeStat, control.HealthStat, 10 )
    control.TimeStat.Height:Set(control.TimeStat.Label.Height)
    LayoutStatGroup_mini( control.TimeStat )
    
    LayoutHelpers.RightOf(  control.ShieldStat, control.TimeStat, 10 )
    control.TimeStat.Height:Set(control.ShieldStat.Label.Height)
    LayoutStatGroup_mini( control.ShieldStat )
    
    # Build Resource Group
    LayoutHelpers.AtLeftTopIn( control.BuildCostGroup, control.BG, 75, 30 )
    control.BuildCostGroup.Width:Set( 131 )
    LayoutResourceGroup( control.BuildCostGroup )
    control.BuildCostGroup.Bottom:Set( function() return control.BuildCostGroup.MassValue.Bottom() + 1 end )

    # Upkeep Resource Group
    LayoutHelpers.RightOf( control.UpkeepGroup, control.BuildCostGroup )
    control.UpkeepGroup.Width:Set( 131 )
    control.UpkeepGroup.Bottom:Set( control.BuildCostGroup.Bottom )
    LayoutResourceGroup( control.UpkeepGroup )
    control:Hide()
   
    if control.AbilityList then
        # Ability List
        control.AbilityList.Left:Set(function() return control.BG.Left() + 20 end)
        control.AbilityList.Bottom:Set(function() return control.BG.Top() - 20 end)
        control.AbilityList.Width:Set(20)
        LayoutAbilityBox(control.AbilityList)
   end
        
    if control.Description then
        # Description
        control.Description.Left:Set(function() return control.BG.Right() end)
        control.Description.Bottom:Set(function() return control.BG.Bottom() end)
        control.Description.Width:Set(control.Description.BG.Width)
        LayoutTextbox(control.Description)
   end 
    
    control:Hide()
end

function Create_sca(parent)
    # Main window
    if not import('/lua/ui/game/unitviewDetail.lua').View then
        import('/lua/ui/game/unitviewDetail.lua').View = Group(parent)
    end
    
    local View = import('/lua/ui/game/unitviewDetail.lua').View
    
    if not View.BG then
        View.BG = Bitmap(View)
    end
    View.BG:SetTexture(pathuiimg.. '/blank.dds')
    skinning.GetSkin("unitviewdetail", View.BG, View, "classic")
	View.BG.Depth:Set(200)

    # Unit icon
    if false then
        View.UnitIcon = Bitmap(View)
    end
    
    if not View.UnitImg then
        View.UnitImg = Bitmap( View.BG )
    end
    
    # Unit Name
    if not View.UnitName then
        View.UnitName = UIUtil.CreateText( View.BG, "", 14, "Arial Bold" )
    end
    View.UnitName:SetColor("FFFF9E06")

    # Unit Description
    if not View.UnitShortDesc then
        View.UnitShortDesc = UIUtil.CreateText( View.BG, "", 14, UIUtil.bodyFont )
    end
    View.UnitShortDesc:SetColor("FFFF9E06")

    # Cost
    if not View.BuildCostGroup then
        View.BuildCostGroup = CreateResourceGroup(View.BG, "<LOC uvd_0000>Build Cost (Rate)")
    end

    # Upkeep
    if not View.UpkeepGroup then
        View.UpkeepGroup = CreateResourceGroup(View.BG, "<LOC uvd_0002>Yield")
    end
    
    # Health stat
    if not View.HealthStat then
        View.HealthStat = CreateStatGroup( View.BG, UIUtil.UIFile('/game/unit_view_icons/redcross.dds') )
    end
    
    if not View.ShieldStat then
        View.ShieldStat = CreateStatGroup( View.BG, UIUtil.UIFile('/game/unit_view_icons/shield.dds') )
    end
    # Tme stat
    if not View.TimeStat then
        View.TimeStat = CreateStatGroup( View.BG, UIUtil.UIFile('/game/unit-over/icon-clock_bmp.dds') )
    end
    
--    if Prefs.GetOption('uvd_format') == 'full' then
	if Prefs.GetFromCurrentProfile("uimanager_sav")["affiche_help"]==1 then
        # Description  "<LOC uvd_0003>Description"
        if not View.Description then
            View.Description = CreateTextbox(View.BG, nil, true)
        end
        
        # Ability List"<LOC _Abilities>"
        if not View.AbilityList then
            View.AbilityList = CreateTextbox(View.BG, nil)
        end
    else
        if View.Description then View.Description:Destroy() View.Description = false end
        if View.AbilityList then View.AbilityList:Destroy() View.AbilityList = false end
    end
    
    View.BG:DisableHitTest(true)
end


function SetLayout_sca()
    import('/lua/ui/game/unitviewDetail.lua').ViewState = Prefs.GetOption('uvd_format')
    local mapGroup = import('/lua/ui/game/unitviewDetail.lua').MapView
    
    Create_sca(mapGroup)
    
    local control = import('/lua/ui/game/unitviewDetail.lua').View

    local OrderGroup = import('/lua/ui/game/orders.lua').controls.bg

	

	if UIManager.unitview then
		control.Left:Set(UIManager.unitview.Left)
		control.Bottom:Set(UIManager.unitview.Bottom)
	else
	    LayoutHelpers.AtLeftIn(control, mapGroup, 0)
		LayoutHelpers.AtBottomIn(control, mapGroup, -2)
	end
    control.Width:Set( control.BG.Width )
    control.Height:Set( control.BG.Height )
    
    # Main window background
    LayoutHelpers.AtLeftTopIn( control.BG, control )

    # Unit Name
	skinning.SetPosition("unitviewdetail", control.UnitName, control.BG, "name")
    
    # Unit Image
	skinning.SetPosition("unitviewdetail", control.UnitImg, control.BG, "icon")
    
    # Unit Description
	skinning.SetPosition("unitviewdetail", control.UnitShortDesc, control.UnitName, "description", "rightof")

    # Health Stat
	skinning.SetPosition("unitviewdetail", control.HealthStat, control.BG, "healthstat")
    control.HealthStat.Width:Set(1)
    control.HealthStat.Height:Set(control.HealthStat.Label.Height)
    LayoutStatGroup_sca( control.HealthStat )
    
    # Shield stat
	skinning.SetPosition("unitviewdetail", control.TimeStat, control.BG, "timestat")
    control.TimeStat.Width:Set(1)
    control.TimeStat.Height:Set(control.TimeStat.Label.Height)
    LayoutStatGroup_sca( control.TimeStat )
    
	skinning.SetPosition("unitviewdetail", control.ShieldStat, control.BG, "shieldstat")
    control.TimeStat.Width:Set(1)
    control.TimeStat.Height:Set(control.ShieldStat.Label.Height)
    LayoutStatGroup_sca( control.ShieldStat )
    
    # Build Resource Group
	skinning.SetPosition("unitviewdetail", control.BuildCostGroup, control.BG, "buildcostgroup")
    control.BuildCostGroup.Width:Set(1)
    LayoutResourceGroup( control.BuildCostGroup )

    # Upkeep Resource Group
 	skinning.SetPosition("unitviewdetail", control.UpkeepGroup, control.BG, "upkeepgroup")
    control.UpkeepGroup.Width:Set(1)
    LayoutResourceGroup( control.UpkeepGroup )
   
    if control.AbilityList then
        # Ability List
		skinning.SetPosition("unitviewdetail", control.AbilityList, control.BG, "abilitybox")
        LayoutAbilityBox(control.AbilityList)
   end
        
    if control.Description then
        # Description
		skinning.SetPosition("unitviewdetail", control.Description, control.BG, "helpbox")
        control.Description.Width:Set(control.Description.BG.Width)
        LayoutTextbox(control.Description)
   end 
    
    control:Hide()
end


function LayoutResourceGroup(group)
    
    LayoutHelpers.AtTopIn( group.Label, group )
    LayoutHelpers.AtHorizontalCenterIn( group.Label, group )

    LayoutHelpers.Below( group.EnergyIcon, group.Label )
    group.EnergyIcon.Left:Set( function() return group.Label.Left() - 4 end )

    LayoutHelpers.RightOf( group.EnergyValue, group.EnergyIcon, 1 )
    group.EnergyValue.Top:Set( function() return group.EnergyIcon.Top() + 1 end )

    LayoutHelpers.RightOf( group.MassValue, group.MassIcon, 1 )
    group.MassValue.Right:Set( function() return group.Label.Right() end )
    group.MassValue.Top:Set( function() return group.MassIcon.Top() + 1 end )

    LayoutHelpers.Below( group.MassIcon, group.EnergyIcon )
end

function LayoutStatGroup_mini(group)
    group.Width:Set(function() return group.Label.Width() + group.Value.Width() end)
    group.Label.Left:Set(group.Left)
    group.Label.Top:Set(group.Top)
    group.Value.Left:Set(function() return group.Label.Right() + 4 end)
    LayoutHelpers.AtVerticalCenterIn(group.Value, group.Label)
end  

function LayoutStatGroup_sca(group)
    group.Label.Left:Set(group.Left)
    group.Label.Top:Set(group.Top)
    group.Value.Left:Set(function() return group.Label.Right() + 4 end)
    LayoutHelpers.AtVerticalCenterIn(group.Value, group.Label)
end  

function LayoutTextbox(group)
    group.BGBot.Bottom:Set(group.Bottom)
    group.BGBot.Left:Set(group.Left)
    group.BGMid.Left:Set(group.BGBot.Left)
    group.BGMid.Bottom:Set(group.BGBot.Top)
    group.BGMid.Height:Set(15)
    group.BG.Bottom:Set(function() return group.BGMid.Top() + 1 end)
    group.BG.Left:Set(group.BGBot.Left)
    
    group.BG.Depth:Set(function() return group.Depth() - 1 end)
    group.BGMid.Depth:Set(group.BG.Depth)
    group.BGBot.Depth:Set(group.BG.Depth)
    
    if group.Label then
        LayoutHelpers.AtRightTopIn(group.Label, group.BG, 10, 5)
        group.Label:SetColor("FFFF9E06")
    end
    
    LayoutHelpers.AtLeftTopIn(group.Value[1], group.BG, 8, 8)
    group.Value[1].Right:Set(function() return group.Right() - 15 end)
    group.Value[1].Width:Set(function() return group.Right() - group.Left() - 14 end)
    group.Value[1]:SetClipToWidth(true)
end

function LayoutAbilityBox(group)
    group.bgmidmiddle.Left:Set(group.Left)
    group.bgmidmiddle.Bottom:Set(group.Bottom)
    
    group.bgtopleft.Right:Set(group.bgmidmiddle.Left)
    group.bgtopleft.Bottom:Set(group.bgmidmiddle.Top)
    group.bgtopright.Left:Set(group.bgmidmiddle.Right)
    group.bgtopright.Bottom:Set(group.bgmidmiddle.Top)
    
    group.bgbotleft.Right:Set(group.bgmidmiddle.Left)
    group.bgbotleft.Top:Set(group.bgmidmiddle.Bottom)
    group.bgbotright.Left:Set(group.bgmidmiddle.Right)
    group.bgbotright.Top:Set(group.bgmidmiddle.Bottom)
     
    group.bgtopmiddle.Bottom:Set(group.bgmidmiddle.Top)
    group.bgtopmiddle.Left:Set(group.bgmidmiddle.Left)
    group.bgtopmiddle.Right:Set(group.bgmidmiddle.Right)
    group.bgbotmiddle.Top:Set(group.bgmidmiddle.Bottom)
    group.bgbotmiddle.Left:Set(group.bgmidmiddle.Left)
    group.bgbotmiddle.Right:Set(group.bgmidmiddle.Right)
    
    group.bgmidleft.Bottom:Set(group.bgmidmiddle.Bottom)
    group.bgmidleft.Top:Set(group.bgmidmiddle.Top)
    group.bgmidleft.Right:Set(group.bgmidmiddle.Left)
    group.bgmidright.Bottom:Set(group.bgmidmiddle.Bottom)
    group.bgmidright.Top:Set(group.bgmidmiddle.Top)
    group.bgmidright.Left:Set(group.bgmidmiddle.Right)
        
    LayoutHelpers.AtLeftTopIn(group.Value[1], group.bgmidmiddle, -5, -5)
end