
local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local GameCommon = import('/lua/ui/game/gamecommon.lua')
local Group = import('/lua/maui/group.lua').Group
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local StatusBar = import('/lua/maui/statusbar.lua').StatusBar

local MultiFunction = import('/lua/ui/game/multifunction.lua')
local profiles = GetPreference("profile.profiles") -- Gets our profiles table
local current = GetPreference("profile.current") -- gets the current profile
local Prefs = import('/lua/user/prefs.lua')--preferences

local pathui = '/mods/uimod/hook'
local UIManager= import('/mods/uimod/sca_utils/uimanager.lua')
local sca_utils = '/mods/uimod/sca_utils/'
local skinning=import(sca_utils .. 'skinning.lua')

local pathuiimg='/mods/uimod/textures'

function SetLayout()
	if Prefs.GetFromCurrentProfile("uimanager_sav")["new_unit_view"]==0 then
		SetLayout_sca()
	else
		SetLayout_mini()
	end
end

function SetLayout_mini()
  local controls = import('/lua/ui/game/unitview.lua').controls
    local numRoDisplayFields = import('/lua/ui/game/unitview.lua').numRoDisplayFields
    if UIManager.unitview then controls.rolloverGroup.Bottom:Set(UIManager.unitview.Bottom) end
    controls.rolloverGroup.Height:Set(50)
    if UIManager.unitview then controls.rolloverGroup.Left:Set(UIManager.unitview.Left) end
    controls.rolloverGroup.Depth:Set(900)
	
    if not controls.roBG then
        controls.roBG = Bitmap(controls.rolloverGroup)
    end
    controls.roBG:SetTexture(UIUtil.UIFile(pathuiimg..'/blank.dds'))
	skinning.GetSkin("unitview", controls.roBG, controls.rolloverGroup, "new")
    controls.roBG.Bottom:Set(controls.rolloverGroup.Bottom)
    controls.roBG.Left:Set(controls.rolloverGroup.Left)
    
    if controls.roBG2 then controls.roBG2:Destroy() controls.roBG2 = false end

    if not controls.roIcon then
        controls.roIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roIcon.Top:Set(function() return controls.roBG.Top() + 10 end)
    controls.roIcon.Left:Set(function() return controls.rolloverGroup.Left() + 10 end)
    controls.roIcon.Height:Set(46)
    controls.roIcon.Width:Set(48)
    
    if not controls.roStratIcon then
        controls.roStratIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roStratIcon.Top:Set(function() return controls.roIcon.Bottom() end)
    controls.roStratIcon.Left:Set(function() return controls.roIcon.Left() end)
    controls.roStratIcon.Width:Set(14)
    controls.roStratIcon.Height:Set(14)
    

    local previousPip = false
    for i = 1, 3 do
        if not controls.roTechLevelPips[i] then
            controls.roTechLevelPips[i] = Bitmap(controls.rolloverGroup)
        end
        if i == 1 then
            controls.roTechLevelPips[i].Left:Set(function() return controls.roStratIcon.Right() + 4 end)
        else
            controls.roTechLevelPips[i].Left:Set(previousPip.Right)
        end
        LayoutHelpers.AtVerticalCenterIn(controls.roTechLevelPips[i], controls.roStratIcon)
        controls.roTechLevelPips[i].Depth:Set(function() return controls.roIcon.Depth() + 1 end)
        previousPip = controls.roTechLevelPips[i]
    end

    if not controls.roHealthBar then
        controls.roHealthBar = StatusBar(controls.rolloverGroup, 0, 1, false, false, UIUtil.UIFile('/game/unit-over/health-bars-back-1_bmp.dds'), UIUtil.UIFile('/game/unit-over/bar01_bmp.dds'), true, "Unit RO Health Status Bar")
    end
    controls.roHealthBar.Left:Set(function() return controls.roIcon.Right() + 10 end)
    controls.roHealthBar.Right:Set(function() return controls.roHealthBar.Left() + 60 end)
    controls.roHealthBar.Bottom:Set(function() return controls.roIcon.Bottom() - 15 end)
    controls.roHealthBar.Top:Set(function() return controls.roHealthBar.Bottom() - 4 end)
    controls.roHealthBar.Height:Set(function() return controls.roHealthBar.Bottom() - controls.roHealthBar.Top() end)
    controls.roHealthBar.Width:Set(function() return controls.roHealthBar.Right() - controls.roHealthBar.Left() end)

    if not controls.roProgBar then
        controls.roProgBar = StatusBar(controls.rolloverGroup, 0, 1, false, false, UIUtil.UIFile('/game/unit-over/health-bars-back-1_bmp.dds'), UIUtil.UIFile('/game/unit-over/bar01_bmp.dds'), true, "Unit RO Health Status Bar")
    end
    LayoutHelpers.Below(controls.roProgBar, controls.roHealthBar, 1)
    controls.roProgBar.Height:Set(controls.roHealthBar.Height)
    controls.roProgBar.Width:Set(controls.roHealthBar.Width)

    if not controls.roName then
        controls.roName = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
    end
    controls.roName.Top:Set(function() return controls.roIcon.Top() + 0 end)
    controls.roName.Left:Set(function() return controls.roIcon.Right() + 12 end)
    controls.roName.Right:Set(function() return controls.roBG.Right() - 10 end)
    controls.roName:SetClipToWidth(true)
    controls.roName:SetDropShadow(true)
    
    local prevVetIcon = false
    for i=1, 5 do
        local index = i
        if not controls.roVeteranIcons[i] then
            controls.roVeteranIcons[index] = Bitmap(controls.rolloverGroup)
        end
        if index == 1 then
            controls.roVeteranIcons[index].Left:Set(function() return controls.roIcon.Right() + 1 end)
        else
            LayoutHelpers.RightOf(controls.roVeteranIcons[index], controls.roVeteranIcons[index-1], -3)
        end
        controls.roVeteranIcons[index].Top:Set(function() return controls.roStratIcon.Top() - 6 end)
        controls.roVeteranIcons[index]:SetTexture(UIUtil.UIFile('/game/veteran-logo_bmp/uef-veteran_bmp.dds'))
        controls.roVeteranIcons[index]:Hide()
        prevVetIcon = controls.roVeteranIcons[index]
    end
    
    if not controls.roId then
        controls.roId = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
    end
    controls.roId.Top:Set(function() return controls.roName.Bottom() + 12 end)
    LayoutHelpers.AtHorizontalCenterIn(controls.roId, controls.roHealthBar)
    controls.roId:SetDropShadow(true)


    if not controls.roHealth then
        controls.roHealth = UIUtil.CreateText(controls.rolloverGroup , "", 10, UIUtil.bodyFont)
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.roHealth, controls.roHealthBar)
    controls.roHealth.Top:Set(function() return controls.roHealthBar.Top() - controls.roHealth.Height() + 2 end)
    controls.roHealth.Depth:Set(function() return controls.roHealthBar.Depth() + 10 end)
    controls.roHealth:SetDropShadow(true)

    if not controls.roDisplay['mass'] then
        controls.roDisplay['mass'] = {}
        controls.roDisplay['mass'].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
        controls.roDisplay['mass'].icon = Bitmap(controls.rolloverGroup)
    end
    controls.roDisplay['mass'].icon.Left:Set(function() return controls.roHealthBar.Right() + 6 end)
    controls.roDisplay['mass'].icon.Top:Set(function() return controls.roIcon.Top() + 19 end)
    controls.roDisplay['mass'].text.Left:Set(function() return controls.roDisplay['mass'].icon.Right()  end)
    LayoutHelpers.AtVerticalCenterIn(controls.roDisplay['mass'].text, controls.roDisplay['mass'].icon, 1)

    if not controls.roDisplay['energy'] then
        controls.roDisplay['energy'] = {}
        controls.roDisplay['energy'].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
        controls.roDisplay['energy'].icon = Bitmap(controls.rolloverGroup)
    end
    controls.roDisplay['energy'].icon.Left:Set(function() return controls.roDisplay['mass'].icon.Right() + 43 end)
    controls.roDisplay['energy'].icon.Top:Set(controls.roDisplay['mass'].icon.Top)
    controls.roDisplay['energy'].text.Left:Set(function() return controls.roDisplay['energy'].icon.Right() end)
    LayoutHelpers.AtVerticalCenterIn(controls.roDisplay['energy'].text, controls.roDisplay['energy'].icon, 1)


    for i=1, numRoDisplayFields do
		local index = i
        if not controls.roDisplay[index] then
            controls.roDisplay[index] = {}
            controls.roDisplay[index].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
            controls.roDisplay[index].icon = Bitmap(controls.rolloverGroup)
        end
        if i == 1 then
            controls.roDisplay[index].icon.Left:Set(function() return controls.roHealthBar.Left() + 5 end)
            controls.roDisplay[index].icon.Top:Set(function() return controls.roHealthBar.Bottom() + 13 end)
        else
            local prevControl = controls.roDisplay[index - 1].icon
            controls.roDisplay[index].icon.Left:Set(function() return prevControl.Right() + 46 end)
            LayoutHelpers.AtVerticalCenterIn(controls.roDisplay[index].icon, prevControl)
        end
        controls.roDisplay[index].text.Left:Set(function() return controls.roDisplay[index].icon.Right() + 2 end)
        LayoutHelpers.AtVerticalCenterIn(controls.roDisplay[i].text, controls.roDisplay[index].icon, 1)
    end

    if controls.roFocusBG then controls.roFocusBG:Destroy() controls.roFocusBG = false end
    if controls.roFocusHealthBar then controls.roFocusHealthBar:Destroy() controls.roFocusHealthBar = false end
    if controls.roFocusName then controls.roFocusName:Destroy() controls.roFocusName = false end

    if not controls.roFocusIcon then
        controls.roFocusIcon = Bitmap(controls.rolloverGroup)
    end
    LayoutHelpers.AtLeftTopIn(controls.roFocusIcon, controls.roBG, 260, 28)
    controls.roFocusIcon.Height:Set(46)
    controls.roFocusIcon.Width:Set(46)
    controls.roFocusIcon.Depth:Set(function() return controls.roBG.Depth() + 1 end)
    
    if not controls.roFocusHealthPercent then
        controls.roFocusHealthPercent = UIUtil.CreateText(controls.rolloverGroup, "", 14, UIUtil.bodyFont)
    end
    controls.roFocusHealthPercent.Bottom:Set(function() return controls.roFocusIcon.Bottom() - 2 end)
    controls.roFocusHealthPercent.Left:Set(function() return controls.roFocusIcon.Left() + 2 end)
    LayoutHelpers.ResetRight(controls.roFocusHealthPercent)
    controls.roFocusHealthPercent:SetDropShadow(true)
    controls.roFocusHealthPercent.Depth:Set(function() return controls.roFocusIcon.Depth() + 1 end)
    
    controls.rolloverGroup:DisableHitTest(true)
	
	if Prefs.GetFromCurrentProfile("uimanager_sav")["affiche_help"]==1 then
		if not controls.AbilityList then
			controls.AbilityList = CreateTextbox(controls.roBG)
		end
		controls.AbilityList.Left:Set(function() return controls.roBG.Left() + 19 end)
		controls.AbilityList.Bottom:Set(function() return controls.roBG.Top() - 11 end)
		controls.AbilityList.Width:Set(20)
		LayoutAbilityBox(controls.AbilityList)
	else
	    if controls.AbilityList then controls.AbilityList:Destroy() controls.AbilityList = false end
	end
end

function SetLayout_sca()
    local controls = import('/lua/ui/game/unitview.lua').controls
    local numRoDisplayFields = import('/lua/ui/game/unitview.lua').numRoDisplayFields
    if UIManager.unitview then controls.rolloverGroup.Bottom:Set(UIManager.unitview.Bottom) end
    controls.rolloverGroup.Height:Set(50)
    if UIManager.unitview then controls.rolloverGroup.Left:Set(UIManager.unitview.Left) end
    controls.rolloverGroup.Depth:Set(900)
	
    
    if not controls.roBG then
        controls.roBG = Bitmap(controls.rolloverGroup)
    end
	controls.roBG:SetTexture(pathuiimg.. '/blank.dds')
 	skinning.GetSkin("unitview", controls.roBG, controls.rolloverGroup, "classic")
	controls.roBG.Bottom:Set(controls.rolloverGroup.Bottom)
    controls.roBG.Left:Set(controls.rolloverGroup.Left)
    
    if not controls.roIcon then
        controls.roIcon = Bitmap(controls.rolloverGroup)
    end
    
    if not controls.roStratIcon then
        controls.roStratIcon = Bitmap(controls.rolloverGroup)
    end
    
    local previousPip = false
    for i = 1, 3 do
        if not controls.roTechLevelPips[i] then
            controls.roTechLevelPips[i] = Bitmap(controls.rolloverGroup)
        end
        if i == 1 then
            controls.roTechLevelPips[i].Left:Set(function() return controls.roBG.Left() + 4 end)
			controls.roTechLevelPips[i].Top:Set(function() return controls.roBG.Top() + 17 end)
        else
            controls.roTechLevelPips[i].Left:Set(previousPip.Right)
			controls.roTechLevelPips[i].Top:Set(previousPip.Top)
       end
        controls.roTechLevelPips[i].Depth:Set(function() return controls.roIcon.Depth() + 1 end)
        previousPip = controls.roTechLevelPips[i]
    end

    if controls.roHealthBar then
        controls.roHealthBar:Destroy()
    end

    if controls.roProgBar then
        controls.roProgBar:Destroy()
    end

    if not controls.roName then
        controls.roName = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
    end
    controls.roName:SetClipToWidth(true)
    controls.roName:SetDropShadow(true)
    
    local prevVetIcon = false
    for i=1, 5 do
        local index = i
        if not controls.roVeteranIcons[index] then
            controls.roVeteranIcons[index] = Bitmap(controls.rolloverGroup)
        end
        if index == 1 then
            controls.roVeteranIcons[index].Left:Set(function() return controls.roBG.Left() + 1 end)
			controls.roVeteranIcons[index].Top:Set(function() return controls.roBG.Top() end)
        else
            LayoutHelpers.RightOf(controls.roVeteranIcons[index], controls.roVeteranIcons[index-1], -3)
			LayoutHelpers.AtVerticalCenterIn(controls.roVeteranIcons[index], controls.roVeteranIcons[index-1])
        end
        controls.roVeteranIcons[index]:SetTexture(UIUtil.UIFile('/game/veteran-logo_bmp/uef-veteran_bmp.dds'))
        controls.roVeteranIcons[index]:Hide()
        prevVetIcon = controls.roVeteranIcons[index]
    end
    
    if not controls.roId then
        controls.roId = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
    end
    controls.roId:SetDropShadow(true)


    if not controls.roHealth then
        controls.roHealth = UIUtil.CreateText(controls.rolloverGroup, "", 10, UIUtil.bodyFont)
    end

    if not controls.roDisplay['mass'] then
        controls.roDisplay['mass'] = {}
        controls.roDisplay['mass'].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
        controls.roDisplay['mass'].icon = Bitmap(controls.rolloverGroup)
    end

    if not controls.roDisplay['energy'] then
        controls.roDisplay['energy'] = {}
        controls.roDisplay['energy'].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
        controls.roDisplay['energy'].icon = Bitmap(controls.rolloverGroup)
    end

    for i=1, numRoDisplayFields do
		local index = i
		if controls.roDisplay[index] then  -- SCA cleaning
			controls.roDisplay[index].text:Destroy()
			controls.roDisplay[index].icon:Destroy()
		end
	
		controls.roDisplay[index] = {}
		controls.roDisplay[index].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
		controls.roDisplay[index].icon = Bitmap(controls.rolloverGroup)
		
		
		
        if i == 1 then
            controls.roDisplay[index].icon.Left:Set(function() return controls.roBG.Left() + 44 end)
            controls.roDisplay[index].icon.Top:Set(function() return controls.roBG.Top() end)
        else
            local prevControl = controls.roDisplay[index - 1].text
            LayoutHelpers.RightOf(controls.roDisplay[index].icon, prevControl, 4)
            LayoutHelpers.AtVerticalCenterIn(controls.roDisplay[index].icon, prevControl)
        end
        controls.roDisplay[index].icon.Height:Set(14)
        controls.roDisplay[index].icon.Width:Set(14)	
        controls.roDisplay[index].text.Left:Set(function() return controls.roDisplay[index].icon.Right() + 2 end)
        LayoutHelpers.AtVerticalCenterIn(controls.roDisplay[index].text, controls.roDisplay[index].icon, 1)
    end

    if not controls.roFocusBG then
        controls.roFocusBG = Bitmap(controls.rolloverGroup)
    end

    if not controls.roFocusIcon then
        controls.roFocusIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roFocusIcon.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)
    
    if not controls.roFocusHealthPercent then
        controls.roFocusHealthPercent = UIUtil.CreateText(controls.rolloverGroup, "", 14, UIUtil.bodyFont)
    end
    controls.roFocusHealthPercent:SetDropShadow(true)
    controls.roFocusHealthPercent.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)
    
    if controls.roFocusHealthBar then
        controls.roFocusHealthBar:Destroy()
    end

    if not controls.roFocusName then
        controls.roFocusName = UIUtil.CreateText(controls.rolloverGroup, "", 14, UIUtil.bodyFont)
    end
    controls.roFocusName:SetDropShadow(true)
    controls.roFocusName.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)

    controls.roBG:DisableHitTest(true)
    controls.rolloverGroup:DisableHitTest(true)
	
	skinning.GetSkin("unitview", controls.roFocusBG, controls.rolloverGroup, "focusbg")
	skinning.SetPosition("unitview", controls.roIcon, controls.roBG, "icon")
	skinning.SetPosition("unitview", controls.roStratIcon, controls.roBG, "straticon")
	skinning.SetPosition("unitview", controls.roTechLevelPips[1], controls.roBG, "techpips")
	controls.roHealthBar = skinning.SetStatusBar("unitview", controls.roBG, "healthbar")
	controls.roProgBar = skinning.SetStatusBar("unitview", controls.roBG, "progbar")
	skinning.SetPosition("unitview", controls.roName, controls.roBG, "name")
	skinning.SetPosition("unitview", controls.roVeteranIcons[1], controls.roBG, "veticons")
	skinning.SetPosition("unitview", controls.roDisplay['mass'].icon, controls.roBG, "mass")
	skinning.SetPosition("unitview", controls.roDisplay['energy'].icon, controls.roBG, "energy")
	skinning.SetPosition("unitview", controls.roDisplay[1].icon, controls.roBG, "extrainfo")
	skinning.SetPosition("unitview", controls.roFocusIcon, controls.roFocusBG, "focusicon")
	skinning.SetPosition("unitview", controls.roFocusHealthPercent, controls.roFocusBG, "focushealthtext")
	controls.roFocusHealthBar = skinning.SetStatusBar("unitview", controls.roFocusBG, "focushealthbar")
	skinning.SetPosition("unitview", controls.roFocusName, controls.roFocusBG, "focusname")

    controls.roDisplay['mass'].text.Left:Set(function() return controls.roDisplay['mass'].icon.Right()  end)
    LayoutHelpers.AtVerticalCenterIn(controls.roDisplay['mass'].text, controls.roDisplay['mass'].icon, 1)
    controls.roDisplay['energy'].text.Left:Set(function() return controls.roDisplay['energy'].icon.Right() end)
    LayoutHelpers.AtVerticalCenterIn(controls.roDisplay['energy'].text, controls.roDisplay['energy'].icon, 1)

    LayoutHelpers.AtHorizontalCenterIn(controls.roHealth, controls.roHealthBar)
    controls.roHealth.Top:Set(function() return controls.roHealthBar.Bottom() + 3 end)
    controls.roHealth.Depth:Set(function() return controls.roHealthBar.Depth() + 10 end)
    controls.roHealth:SetDropShadow(true)
    controls.roId.Top:Set(function() return controls.roName.Bottom() + 12 end)
    LayoutHelpers.AtHorizontalCenterIn(controls.roId, controls.roHealthBar)
	
	if Prefs.GetFromCurrentProfile("uimanager_sav")["affiche_help"]==1 then
		if not controls.AbilityList then
			controls.AbilityList = CreateTextbox(controls.roBG)
		end
		skinning.SetPosition("unitview", controls.AbilityList, controls.roBG, "abilitybox")
		LayoutAbilityBox(controls.AbilityList)
	else
	    if controls.AbilityList then controls.AbilityList:Destroy() controls.AbilityList = false end
	end
		
end

function LayoutAbilityBox(group)
    group.bgmidmiddle.Left:Set(group.Left)
    group.bgmidmiddle.Bottom:Set(group.Bottom)
    
    group.bgtopleft.Right:Set(group.bgmidmiddle.Left)
    group.bgtopleft.Bottom:Set(group.bgmidmiddle.Top)
    group.bgtopright.Left:Set(group.bgmidmiddle.Right)
    group.bgtopright.Bottom:Set(group.bgmidmiddle.Top)
    
    group.bgbotleft.Right:Set(group.bgmidmiddle.Left)
    group.bgbotleft.Top:Set(group.bgmidmiddle.Bottom)
    group.bgbotright.Left:Set(group.bgmidmiddle.Right)
    group.bgbotright.Top:Set(group.bgmidmiddle.Bottom)
     
    group.bgtopmiddle.Bottom:Set(group.bgmidmiddle.Top)
    group.bgtopmiddle.Left:Set(group.bgmidmiddle.Left)
    group.bgtopmiddle.Right:Set(group.bgmidmiddle.Right)
    group.bgbotmiddle.Top:Set(group.bgmidmiddle.Bottom)
    group.bgbotmiddle.Left:Set(group.bgmidmiddle.Left)
    group.bgbotmiddle.Right:Set(group.bgmidmiddle.Right)
    
    group.bgmidleft.Bottom:Set(group.bgmidmiddle.Bottom)
    group.bgmidleft.Top:Set(group.bgmidmiddle.Top)
    group.bgmidleft.Right:Set(group.bgmidmiddle.Left)
    group.bgmidright.Bottom:Set(group.bgmidmiddle.Bottom)
    group.bgmidright.Top:Set(group.bgmidmiddle.Top)
    group.bgmidright.Left:Set(group.bgmidmiddle.Right)
        
    LayoutHelpers.AtLeftTopIn(group.Value[1], group.bgmidmiddle, -5, -5)
end

function CreateTextbox(parent)
    local group = Group(parent)
    
    group.bgtopleft = Bitmap(group)
    group.bgtopmiddle = Bitmap(group)
    group.bgtopright = Bitmap(group)
    
    group.bgmidleft = Bitmap(group)
    group.bgmidmiddle = Bitmap(group)
    group.bgmidright = Bitmap(group)
    
    group.bgbotleft = Bitmap(group)
    group.bgbotmiddle = Bitmap(group)
    group.bgbotright = Bitmap(group)
    
    group.bgtopleft:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_ul.dds'))
    group.bgtopmiddle:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_horz_um.dds'))
    group.bgtopright:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_ur.dds'))
    
    group.bgmidleft:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_vert_l.dds'))
    group.bgmidmiddle:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_m.dds'))
    group.bgmidright:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_vert_r.dds'))
    
    group.bgbotleft:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_ll.dds'))
    group.bgbotmiddle:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_lm.dds'))
    group.bgbotright:SetTexture(UIUtil.UIFile('/game/chat_brd/chat_brd_lr.dds'))

    group.Value = {}
    group.Value[1] = UIUtil.CreateText( group, "", 12, UIUtil.bodyFont)
    
    return group
end
