--*****************************************************************************
--* File: lua/modules/ui/game/multifunction.lua
--* Author: Chris Blackwell
--* Summary: UI for the multifunction display
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Group = import('/lua/maui/group.lua').Group
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Button = import('/lua/maui/button.lua').Button
local Grid = import('/lua/maui/grid.lua').Grid
local Text = import('/lua/maui/text.lua').Text
local GameCommon = import('/lua/ui/game/gamecommon.lua')
local Tooltip = import('/lua/ui/game/tooltip.lua')
local econ = import("/lua/ui/game/economy.lua")
local Prefs = import('/lua/user/prefs.lua')

local worldView = import('/lua/ui/game/borders.lua').GetMapGroup()

local minimap = Prefs.GetFromCurrentProfile('stratview') or false
local mouseoverDisplay = false
local needExpand = false

buttonData = {
    [1] = {
        btnName = 'military',
        label = 'mfd_military',
        button = false,
        clickBehavior = function(self, checked)
            RenderOverlayMilitary(checked)
        end
    },
    [2] = {
        btnName = 'intelligence',
        label = 'mfd_intel',
        button = false,
        clickBehavior = function(self, checked)
            RenderOverlayIntel(checked)
        end
    },
    [3] = {
        btnName = 'economy',
        label = 'mfd_economy',
        button = false,
        clickBehavior = function(self, checked)
            toggleEconDisplay(checked)
        end
    },
    [4] = {
        btnName = 'team-color',
        label = 'mfd_defense',
        button = false,
        clickBehavior = function(self, checked)
            TeamColorMode(checked)
        end
    },
    [5] = {
        btnName = 'control',
        label = 'mfd_strat_view',
        button = true,
        clickBehavior = function(self, checked)
            import('/lua/ui/game/mapcontrol.lua').CreateDialog()
        end
    },
}

controls = {
    controlClusterGroup = false,
    group = false,
    miniMap = false,
    bg = false,
    glow = false,
    displayGroup = false,   
    infoText = false,
    minimapBtns = {},
}

local econDisplayState = 0
function toggleEconDisplay(state)
    RenderOverlayEconomy(state)
end

function SetLayout(layout, onToggleMinimap)
    import(UIUtil.GetLayoutFilename('multifunction')).SetLayout(minimap, onToggleMinimap)
end

function SetupMultifunctionControl(parent)
    controls.controlClusterGroup = parent
    SetLayout(UIUtil.currentLayout)
    return controls.group
end

function ToggleMilitary()
    if controls.minimapBtns[1] then
        controls.minimapBtns[1]:ToggleCheck()
    else
        import('/lua/ui/game/options.lua').ToggleMFD('mfd_military')
    end
end

function ToggleDefense()
    if controls.minimapBtns[4] then
        controls.minimapBtns[4]:ToggleCheck()
    else
        import('/lua/ui/game/options.lua').ToggleMFD('mfd_defense')
    end
end

function ToggleEconomy()
    if controls.minimapBtns[3] then
        controls.minimapBtns[3]:OnClick()
    else
        import('/lua/ui/game/options.lua').ToggleMFD('mfd_economy')
    end
end

function ToggleIntel()
    if controls.minimapBtns[2] then
        controls.minimapBtns[2]:ToggleCheck()
    else
        import('/lua/ui/game/options.lua').ToggleMFD('mfd_intel')
    end
end

function Expand()
    if UIUtil.currentLayout == 'minimal' and needExpand then
        controls.displayGroup:Show()
        needExpand = false
    else
        if needExpand then
            controls.displayGroup:Show()
        else
            controls.displayGroup:Hide()
        end
        needExpand = false
    end
end

function Contract()
    if UIUtil.currentLayout == 'minimal' then
        if minimap then
            controls.displayGroup:Hide()
            needExpand = true
        else
            needExpand = false
        end    
    else
        needExpand = minimap
    end
end

function ToggleMinimap()

    # disable when in Screen Capture mode
    if import('/lua/ui/game/gamemain.lua').gameUIHidden then
        return
    end

    minimap = not minimap
    if minimap then
        controls.displayGroup:Show()
        SetLayout(UIUtil.currentLayout, true)
    else
        controls.displayGroup:Hide()
        SetLayout(UIUtil.currentLayout, true)
    end
    Prefs.SetToCurrentProfile('stratview', minimap)
end

function DisableMiniMap()
    if controls.displayGroup then
        minimap = false
        controls.displayGroup:Hide()
        SetLayout(UIUtil.currentLayout)
        Prefs.SetToCurrentProfile('stratview', minimap)
    end
end

function GetMinimapState()
    return not controls.displayGroup:IsHidden()
end
