
--*****************************************************************************
--* File: lua/ui/game/pause.lua
--* Author: Ted Snook
--* Summary: Paused Game HUD
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/ui/uiutil.lua')
local Group = import('/lua/maui/group.lua').Group
local Bitmap = import('/lua/maui/Bitmap.lua').Bitmap
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local MultiLineText = import('/lua/maui/multilinetext.lua').MultiLineText
local Prefs = import('/lua/user/prefs.lua')
local WinMgr = import('/lua/ui/game/windowmanager.lua')

local parent = false
local hideBool = false

# How many timeouts we think we probably have (only sim knows for sure). We keep
# track of this to address some UI issues
local likelyTimeouts = -1

local controls = {
    bg = false,
    pausedText = false,
    pausesRemaining = false,
    pauseResumeButton = false,
    
}

function Expand()
    hideBool = false
end

function Contract()
    hideBool = true
end

function GetLikelyTimeouts()
    return likelyTimeouts
end

function CreatePauseUI(inParent)
    if SessionIsMultiplayer() then
        local scenInfo = SessionGetScenarioInfo()
        likelyTimeouts = tonumber(scenInfo.Options.Timeouts)
    end
    
    parent = inParent
    if not controls.bg then
        controls.bg = Bitmap(parent, UIUtil.UIFile('/dialogs/pause/panel_bmp_t.dds'))
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.bg, parent)
    controls.bg.Top:Set(function() return parent.Top() + 20 end)
    controls.bg.Depth:Set(1500)
    controls.bg:DisableHitTest()
    
    if not controls.bgBot then
        controls.bgBot = Bitmap(controls.bg, UIUtil.UIFile('/dialogs/pause/panel_bmp_b.dds'))
        controls.bgBot:DisableHitTest()
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.bgBot, parent)
    controls.bgBot.Depth:Set(1500)
    
    if not controls.bgMid then
        controls.bgMid = Bitmap(controls.bg, UIUtil.UIFile('/dialogs/pause/panel_bmp_m.dds'))
        controls.bgMid:DisableHitTest()
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.bgMid, parent)
    controls.bg.Depth:Set(1500)
    
    if not controls.pausedText then
        controls.pausedText = MultiLineText(controls.bg, UIUtil.bodyFont, 18, UIUtil.fontColor)
        controls.pausedText:DisableHitTest()
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.pausedText, controls.bg)
    controls.pausedText.Width:Set(function() return controls.bg.Width() - 30 end)
    controls.pausedText.Height:Set(140)
    controls.pausedText.Top:Set(function() return controls.bg.Bottom() end)
    controls.pausedText:SetCenteredHorizontally(true)
    
    if not controls.pausesRemaining then
        controls.pausesRemaining = UIUtil.CreateText(controls.bg, LOC("<LOC pause_screen_0002>Timeouts Remaining"), 14, UIUtil.bodyFont)
        controls.pausesRemaining:DisableHitTest()
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.pausesRemaining, controls.bg)
    controls.pausesRemaining.Top:Set(function() return controls.bgBot.Top() + 20 end)
    
    if not controls.pauseResumeButton then
        controls.pauseResumeButton = UIUtil.CreateButtonStd(controls.bgBot, '/widgets/small', LOC("<LOC pause_screen_0003>Resume"), 12)
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.pauseResumeButton, controls.bgBot)
    controls.pauseResumeButton.Top:Set(function() return controls.bgBot.Top() + 40 end)
    controls.pauseResumeButton.OnClick = function(self, modifiers)
        SessionResume()
    end
    
    controls.bgMid.Top:Set(function() return controls.bg.Bottom() - 1 end)
    controls.bgMid.Height:Set(function() return controls.pausedText._text[1].Height() * controls.pausedText._numLines() end)
    
    controls.bgBot.Top:Set(function() return controls.bgMid.Bottom() end)
    
    controls.bg:Hide()
end

function OnPause(owner, timeouts, isOwner)
    likelyTimeouts = timeouts
    if WinMgr.IsWindowOpen() then return end
    if not hideBool then
        if UIUtil.GetAnimationPrefs() then
            controls.bg:Show()
        else
            controls.bg:Show()
        end
        
        if owner then
            controls.pausedText:SetText(LOCF("<LOC pause_screen_0004>Game Paused: %s", SessionGetCommandSourceNames()[owner]))
        end
        if timeouts then
            if timeouts == -1 then
                controls.pausesRemaining:SetText(LOCF("<LOC pause_screen_0006>Infinite Timeouts"))
            else
                controls.pausesRemaining:SetText(LOCF("<LOC pause_screen_0005>Timeouts Remaining: %d", timeouts))
            end
        end
    end
end

function OnResume()
    if UIUtil.GetAnimationPrefs() then
        controls.bg:Hide()
    else
        controls.bg:Hide()
    end
end        