--*****************************************************************************
--* File: lua/ui/game/queue.lua
--* Author: Chris Blackwell / Ted Snook
--* Summary: Construction Queue UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Group = import('/lua/maui/group.lua').Group
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Grid = import('/lua/maui/grid.lua').Grid
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Button = import('/lua/maui/button.lua').Button
local GameCommon = import('/lua/ui/game/gamecommon.lua')
local RadioGroup = import('/lua/maui/mauiutil.lua').RadioGroup
local Tooltip = import('/lua/ui/game/tooltip.lua')
local Effect = import('/lua/maui/effecthelpers.lua')
local sca_utils = '/mods/uimod/sca_utils/'
local UIManager = import(sca_utils .. 'uimanager.lua')
local Prefs = import('/lua/user/prefs.lua')--preferences

local constructionControl = false
local currentSelection = false
currentQueue = false
local pathuiimg='/mods/uimod/textures'

boutonH = 48
boutonW = 48

# vars added in by mod
local dragging = false
local index = nil			# index of the item in the queue currently being dragged
local originalIndex = false	# original index of selected item (so that UpdateBuildQueue knows where to modify it from)
local oldQueue = {}
local modifiedQueue = {}
local updateQueue = true	# if false then queue won't update in the ui
local modified = false		# if false then buttonrelease will increase buildcount in queue
local dragLock = false		# to disable quick successive drags, which doubles the units in the queue

local currentInfiniteQueueCheckStatus = false
local layoutVar = false
controls = {
    queueGroup = false,
    queueBGCapMin = false,
    queueBGMiddle = false,
    queueBGCapMax = false,
    queueGrid = false,
    scrollQueueFwdButton = false,
    scrollQueueBackButton = false,
    infiniteQueueToggle = false,
    pauseToggle = false,
    mfdControl = false,
    ordersControl = false,
    controlClusterGroup = false,
}

local function CreateQueueGrid(parent)
    controls.queueGrid = Grid(parent, boutonH, boutonW)
    controls.queueGrid:SetName("Queue Grid")
    controls.queueGrid:AppendRows(1)
end

function ToggleUnitPause()
    controls.pauseToggle:ToggleCheck()
end

function CreateControls()
    if not controls.queueGroup then
        controls.queueGroup = Bitmap(controls.controlClusterGroup, nil, "queueGroup")
		controls.queueGroup.HandleEvent = function(self, event)
			if event.Type == "ButtonRelease" then
				ButtonReleaseCallback()
			end
			if event.Type=="WheelRotation" then
				if controls.scrollQueueBackButton:IsDisabled() == false then
					if event.WheelRotation > 0 then
						controls.queueGrid:ScrollLines("Vert", -1)
					else
						controls.queueGrid:ScrollLines("Vert", 1)
					end
				end
			end
			return true
		end
		UIManager.dragOnUI(controls.queueGroup, "queue")
    end
    if not controls.queueBGCapMin then
        controls.queueBGCapMin = Bitmap(controls.queueGroup)
    end
    if not controls.queueBGCapMax then
        controls.queueBGCapMax = Bitmap(controls.queueGroup)
    end
    if not controls.queueBGMiddle then
        controls.queueBGMiddle = Bitmap(controls.queueGroup)
    end
    if not controls.queueGrid then
        CreateQueueGrid(controls.queueBGMiddle)
    end

    if not controls.queueScrollBG then
        controls.queueScrollBG = Bitmap(controls.queueGroup)
    end
    if controls.scrollQueueBackButton then controls.scrollQueueBackButton:Destroy() end
    controls.scrollQueueBackButton = UIUtil.CreateButton(controls.queueBGCapMin
        , '/game/small-side_scr/arrow-left_scr_up.dds'
        , '/game/small-side_scr/arrow-left_scr_down.dds'
        , '/game/small-side_scr/arrow-left_scr_over.dds'
        , '/game/small-side_scr/arrow-left_scr_dis.dds'
        , nil, nil, nil, nil, "UI_Arrow_Click")
    if controls.scrollQueueFwdButton then controls.scrollQueueFwdButton:Destroy() end
    controls.scrollQueueFwdButton = UIUtil.CreateButton(controls.queueBGCapMin
        , '/game/small-side_scr/arrow-right_scr_up.dds'
        , '/game/small-side_scr/arrow-right_scr_down.dds'
        , '/game/small-side_scr/arrow-right_scr_over.dds'
        , '/game/small-side_scr/arrow-right_scr_dis.dds'
        , nil, nil, nil, nil, "UI_Arrow_Click")
    if controls.infiniteQueueToggle then controls.infiniteQueueToggle:Destroy() end
    controls.infiniteQueueToggle = UIUtil.CreateCheckbox(controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_up.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_down.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_over.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_down.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_dis.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_dis.dds'),
        "UI_Construct_Pause")
    if controls.pauseToggle then controls.pauseToggle:Destroy() end
    controls.pauseToggle = UIUtil.CreateCheckbox(controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_up.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_down.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_over.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_down.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_dis.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_dis.dds'),
        "UI_Construct_Infinite")
end

function CommonLogic()
    controls.scrollQueueBackButton.OnClick = function(self)
        controls.queueGrid:ScrollLines("Horz", -1)
    end
    controls.scrollQueueBackButton:Disable()
    controls.scrollQueueFwdButton.OnClick = function(self)
        controls.queueGrid:ScrollLines("Horz", 1)
    end
    controls.scrollQueueFwdButton:Disable()
    controls.infiniteQueueToggle.OnClick = function(self)
        if currentInfiniteQueueCheckStatus then
            currentInfiniteQueueCheckStatus = false
        else
            currentInfiniteQueueCheckStatus = true
        end
        OnInfiniteQueueToggle(currentInfiniteQueueCheckStatus)
    end
    controls.infiniteQueueToggle.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "infinite_toggle")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Checkbox.HandleEvent(self, event)
    end
    controls.pauseToggle.OnCheck = function(self, checked)
        SetPaused(currentSelection, checked)
    end
    controls.pauseToggle.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "pause")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Checkbox.HandleEvent(self, event)
    end

	controls.scrollQueueBackButton.OnClick = function(self)
    controls.queueGrid:ScrollLines("Vert", -1)
    end
    controls.scrollQueueBackButton:Disable()
    controls.scrollQueueFwdButton.OnClick = function(self)
        controls.queueGrid:ScrollLines("Vert", 1)
    end
    controls.scrollQueueFwdButton:Disable()
end

function SetLayout(layout)
    CreateControls()
    layoutVar = layout
    import(UIUtil.GetLayoutFilename('queue')).SetLayout()
    CommonLogic()
end

function SetupQueueControl(parent, inMFDControl, inOrdersControl)
    controls.mfdControl = inMFDControl
    controls.ordersControl = inOrdersControl
    controls.controlClusterGroup = parent

    SetLayout(UIUtil.currentLayout)
    return controls.queueGroup
end

function SetQueueState(state)
    controls.queueGroup.Mini(state)
end

function SetInfiniteQueueToggle(state)
    if state then
        controls.infiniteQueueToggle:Enable()
    else
        controls.infiniteQueueToggle:Disable()
    end
end

function SetInfinityCheck(state)
    currentInfiniteQueueCheckStatus = state
    controls.infiniteQueueToggle:SetCheck(state)
end



-- command queue is a table of unit id's and how many to build, in sequence to be built
local currentCommandQueue = false

function CreateQueueIcon(parent)
    local bg = Bitmap(parent) -- TODO unit specific BG here
    bg:SetSolidColor('00000000')
    #bg.Width:Set(GameCommon.iconBmpWidth)
    #bg.Height:Set(GameCommon.iconBmpHeight)
    local icon = Bitmap(bg)
    LayoutHelpers.AtLeftTopIn(icon, bg)
    #LayoutHelpers.FillParent(icon, bg)
    local text = UIUtil.CreateText(icon, "", 18, UIUtil.bodyFont)
    LayoutHelpers.AtLeftTopIn(text, icon, 5, 5)
	
	
    if UIUtil.currentLayout == "minimal" then
		icon.Height:Set(48)
        icon.Width:Set(48)
	elseif UIUtil.currentLayout == "bottom" or UIUtil.currentLayout == "left" or UIUtil.currentLayout == "right" then 
		icon.Height:Set(62)
        icon.Width:Set(62)	
	elseif UIUtil.currentLayout == "sca" then
		bg.Width:Set(boutonW)
		bg.Height:Set(boutonH)
		icon.Height:Set(boutonH)
        icon.Width:Set(boutonW)	
	end	
		

	
    text:SetDropShadow(true)
    text:SetColor('white')
    text:DisableHitTest()
    bg.text = text
    bg.unitIcon = icon
    bg:Hide()

    return bg
end

function ClearQueueGrid()
    ClearSessionExtraSelectList()
    controls.queueGrid:DeleteAndDestroyAll(true)
    controls.queueGrid:AppendRows(1, true)
    controls.queueGrid:EndBatch()
end

function PopulateQueueGrid(numIcons)
    if numIcons == 0 then
        controls.scrollQueueFwdButton:Disable()
        controls.scrollQueueBackButton:Disable()
        ClearQueueGrid()
        return
    end
	ClearQueueGrid()
    local cols = 0
	local rows = 0
    local maxcol, maxrow = controls.queueGrid:GetVisible()
	local currentcol = 0
	local currentrow = 1
    controls.queueGrid:AppendCols(numIcons, true)
    controls.queueGrid:AppendRows(1, true)
    for i = 1, numIcons do
        if currentcol >= maxcol then
            controls.queueGrid:AppendRows(1, true)
            currentrow = currentrow + 1
				currentcol = 1
		else
			currentcol = math.mod(i-1, maxcol)+1
        end
        controls.queueGrid:SetItem(CreateQueueIcon(controls.queueGrid), currentcol, currentrow, true)
    end
end

function MoveItemInQueue(queue, indexfrom, indexto, selection)
	modified = true
	if indexfrom < indexto then
		#take indexfrom out and shunt all indices from indexfrom to indexto up one
		moveditem = queue[indexfrom]
		for i = indexfrom, (indexto - 1) do
			queue[i] = queue[i+1]
		end
		queue[indexto] = moveditem
		modifiedQueue = queue
		# update buttons in the UI
		SetQueueGrid(queue, selection, true)
	elseif indexfrom > indexto then
		#take indexfrom out and shunt all indices from indexto to indexfrom down one
		moveditem = queue[indexfrom]
		for i = indexfrom, (indexto + 1), -1 do
			queue[i] = queue[i-1]
		end
		queue[indexto] = moveditem
		modifiedQueue = queue
		# update buttons in the UI
		SetQueueGrid(queue, selection, true)
	end
end


function UpdateBuildList(newqueue, from)
    # The way this does this is UGLY but I can only find functions to remove things from the build queue and to add them at the end
	# Thus the only way I can see to modify the build queue is to delete it back to the point it is modified from (the from argument) and then 
    # add the modified version back in. Unfortunately this causes a momentary 'skip' in the displayed build cue as it is deleted and replaced

	for i = table.getn(oldQueue), from, -1  do
		DecreaseBuildCountInQueue(i, oldQueue[i].count)	
	end
	for i = from, table.getn(newqueue)  do
        blueprint = __blueprints[newqueue[i].id]
        if blueprint.General.UpgradesFrom == 'none' then
		    IssueBlueprintCommand("UNITCOMMAND_BuildFactory", newqueue[i].id, newqueue[i].count)
		else
		    IssueBlueprintCommand("UNITCOMMAND_Upgrade", newqueue[i].id, 1, false)
        end
	end
	ForkThread(dragPause)
	# FIX ME: the factory will pause (I am guessing due to some time out) before it starts building again, once the build queue has been modified
end

function dragPause()
	WaitSeconds(0.3)
	dragLock = false
end

function ButtonReleaseCallback(event)
    if dragging == true then
		gameParent.HandleEvent = gameParent.oldHandleEvent
		# don't update the queue next time round, to avoid a list of 0 builds
		updateQueue = false
		
		# disable dragging until the queue is rebuilt
		dragLock = true
		
		# reset modified so buildcount increasing can be used again
		modified = false
		
        # mouse button released so end drag
        dragging = false

		if originalIndex <= index then
	        first_modified_index = originalIndex
		else 
			first_modified_index = index
		end

		# on the release of the mouse button we want to update the ACTUAL build queue that the factory does. So far, only the UI has been changed,
		# the original build queue will be reverted to once the factory starts building the next unit unless we change it.
		UpdateBuildList(modifiedQueue, first_modified_index)

		# nothing is now selected
		index = nil    
    end  

end

function SetQueueGrid(queue, selection, moved)
	if updateQueue then

		currentQueue = queue
	    
	    local numIcons = table.getn(queue)
	    if Prefs.GetFromCurrentProfile("uimanager_sav").const_sizing == 1 then
	        controls.queueGroup.Resizer.Right:Set(function() return math.min(controls.queueGrid.Left() + boutonW*numIcons + 28, GetFrame(0).Right()) end)
	    end

	    PopulateQueueGrid(numIcons)
	    local maxcol, maxrow = controls.queueGrid:GetVisible()

	    for i = 1, numIcons do
			local j = math.mod(i-1, maxcol)+1
			local k = math.ceil(i/maxcol)
	        local icon = controls.queueGrid:GetItem(j, k)
	        if not icon then
	            LOG("SetQueueGrid attempt to get item ", i, " but it's not there.")
	            continue
	        end
	        local bpid = queue[i].id
	        local blueprint = __blueprints[bpid]
	        local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
	        icon.unitIcon:SetTexture(upIconName)
	        icon.text:SetText(tostring(queue[i].count))
	        icon._blueprintId = bpid
	        icon._index = i -- store array index for determining who was clicked on

			import('/mods/uimod/sca_utils/uimanager.lua').addStrategicIconTo(icon.unitIcon, blueprint)

			icon.HandleEvent = function(self, event)
				if event.Type == 'MouseEnter' then
	                local blueprint = __blueprints[self._blueprintId]
	                import('/lua/ui/game/unitviewDetail.lua').Show(blueprint, currentSelection[1], self._blueprintId)
					
					# if we have entered the button and are dragging something then we want to replace it with what we are dragging
					if dragging == true then
	                    # move item from old location (index) to new location (this button's index)
						MoveItemInQueue(currentQueue, index, self._index, selection) 
	                    # since the currently selected button has now moved, update the index
	                    index = self._index
						
						self.dragMarker = Bitmap(self, pathuiimg..'/queuedragger.dds')
						LayoutHelpers.FillParent(self.dragMarker, icon)
						self.dragMarker:DisableHitTest()
						Effect.Pulse(self.dragMarker, 1.5, 0.6, 0.8)
		
	              	end
				elseif event.Type == 'MouseExit' then
	                    import('/lua/ui/game/unitviewDetail.lua').Hide()
	            elseif event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
	                local count = 1
	                if event.Modifiers.Ctrl == true and event.Modifiers.Shift == nil then
	                    count = 5
	                elseif event.Modifiers.Shift == true and event.Modifiers.Ctrl == nil then
	                    count = 5
	                elseif event.Modifiers.Shift == true and event.Modifiers.Ctrl == true then
	                    count = 5
	                end

	                if event.Modifiers.Left then
						if not dragLock then
							gameParent = import('gamemain.lua').GetGameParent()
							gameParent.oldHandleEvent = gameParent.HandleEvent
							gameParent.HandleEvent = function(self, event)
								if event.Type == 'ButtonRelease' then
									import('/lua/ui/game/queue.lua').ButtonReleaseCallback(event)
								end 
							end 
							
							# only perform drag and drop on factory build queues.
		                    if EntityCategoryContains(categories.FACTORY, currentSelection[1]) then
		                        # left button pressed so start dragging procedure
		                        dragging = true
		                        index = self._index
								originalIndex = index
								
								self.dragMarker = Bitmap(self, pathuiimg..'/queuedragger.dds')
								LayoutHelpers.FillParent(self.dragMarker, icon)
								self.dragMarker:DisableHitTest()
								Effect.Pulse(self.dragMarker, 1.5, 0.6, 0.8)
								
		                        # copy un modified queue so that current build order is recorded (for deleting it)
		                        oldQueue = table.copy(currentQueue)
							else
								IncreaseBuildCountInQueue(self._index, count)
							end
						end
	                else
	                    DecreaseBuildCountInQueue(self._index, count)
	                end
	            end
		 		if event.Type == 'ButtonRelease' then
					if dragging then
	                    if EntityCategoryContains(categories.FACTORY, currentSelection[1]) then
							# if queue has changed then update queue, else increase build count (like default)
							if modified then
								ButtonReleaseCallback(event)
							else
								gameParent.HandleEvent = gameParent.oldHandleEvent
								dragging = false
							    local count = 1
				                if event.Modifiers.Ctrl == true and event.Modifiers.Shift == nil then
				                    count = 5
				                elseif event.Modifiers.Shift == true and event.Modifiers.Ctrl == nil then
				                    count = 5
				                elseif event.Modifiers.Shift == true and event.Modifiers.Ctrl == true then
				                    count = 5
				                end
								IncreaseBuildCountInQueue(self._index, count)
							end
							if self.dragMarker then
								self.dragMarker:Destroy()
								self.dragMarker = false
							end
						end
					end
				end 
				Bitmap.HandleEvent(self, event)
	        end

	    end
	    controls.queueGrid:ScrollSetTop('Horz', 1)
	    controls.queueGrid:EndBatch()
	    local visCols, visRows = controls.queueGrid:GetVisible()
	    if (visCols * visRows) >= numIcons then
	        controls.scrollQueueFwdButton:Disable()
	        controls.scrollQueueBackButton:Disable()
	    else
	        controls.scrollQueueFwdButton:Enable()
	        controls.scrollQueueBackButton:Enable()
	    end
	end
	updateQueue = true
end

function RemovedLastQueuedUnit(bpId)
    if bpId and currentQueue then
        local i = table.getn(currentQueue)
        while i > 0 do
            local icon = controls.queueGrid:GetItem(i, 1)
            if icon._blueprintId == bpId then
                DecreaseBuildCountInQueue(icon._index, 1)
                break
            end
            i = i - 1
        end
    end
end

function SetSelectionIcons(sortedUnitTable, sortedObjectTables)
    ClearQueueGrid()

    local numTypes = table.getn(sortedUnitTable)
    local numIcons = 0

    if numTypes then
        local iconBPID = nil
        local cols, rows = controls.queueGrid:GetDimensions()
        numIcons = table.getn(sortedUnitTable)

        PopulateQueueGrid(numIcons)

        local i = 1
        for index, v in sortedUnitTable do
            local icon = controls.queueGrid:GetItem(i, 1)
            if not icon then
                LOG("SetQueueGrid attempt to get item ", i, " but it's not there.")
            end
            local blueprint = __blueprints[index]
            local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
            icon.unitIcon:SetTexture(upIconName)
            icon.text:SetText(v)
            icon._unitType = index
            i = i + 1

            icon.HandleEvent = function(self, event)
                if event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
                    if event.Modifiers.Left then
                        SelectUnits(sortedObjectTables[self._unitType])
                    else
                        local index = 1
                        local newUnitSelectTable = {}
                        for i, v in sortedObjectTables do
                            if i != self._unitType then
                                for y, z in v do
                                    newUnitSelectTable[index] = z
                                    index = index + 1
                                end
                            end
                        end
                        SelectUnits(newUnitSelectTable)
                    end
                end
                Bitmap.HandleEvent(self, event)
            end
        end
    end

    controls.queueGrid:EndBatch()
    local visCols, visRows = controls.queueGrid:GetVisible()
    if (visCols * visRows) >= numIcons then
        controls.scrollQueueFwdButton:Disable()
        controls.scrollQueueBackButton:Disable()
    else
        controls.scrollQueueFwdButton:Enable()
        controls.scrollQueueBackButton:Enable()
    end
end

function EnablePauseToggle()
    controls.pauseToggle:Enable()
end

-- called on new selection, determines what, if anything, goes in the order queue
function CheckForOrderQueue(selection, newSelection)
    currentSelection = newSelection
    if controls.pauseToggle then controls.pauseToggle:Disable() end
    controls.pauseToggle:SetCheck(GetIsPaused(currentSelection),true)
    if currentSelection[1] and EntityCategoryContains(categories.FACTORY, currentSelection[1]) then
        controls.infiniteQueueToggle:Enable()
    else
        controls.infiniteQueueToggle:Disable()
    end
    -- only show a queue if single item is selected
    if table.getn(selection) == 1 then
        currentCommandQueue = SetCurrentFactoryForQueueDisplay(selection[1])
        -- render the command queue
        if currentCommandQueue then
            SetQueueGrid(currentCommandQueue, selection)
        else
            ClearQueueGrid()
        end
        SetQueueState(false)
    else
        ClearCurrentFactoryForQueueDisplay()
        ClearQueueGrid()
    end
end

function OnQueueChanged(newQueue)
    if newQueue then
        SetQueueGrid(newQueue)
    else
        ClearQueueGrid()
	end
end

function OnInfiniteQueueToggle(checked)
    controls.infiniteQueueToggle:SetCheck(checked)
    for i,v in currentSelection do
        if checked then
            v:ProcessInfo('SetRepeatQueue', 'true')
        else
            v:ProcessInfo('SetRepeatQueue', 'false')
        end
    end
end

function ToggleEnhancementWindow(state, enhancementWindow)
    if state then
        if UIUtil.currentLayout == 'bottom' then
            controls.queueBGCapMin.Left:Set(function() return enhancementWindow.Right() - 23 end)
        end
    else
        if UIUtil.currentLayout == 'bottom' then
            LayoutHelpers.AtLeftIn(controls.queueBGCapMin, controls.constructionGroup, -2)
            LayoutHelpers.AtLeftIn(controls.queueBGCapMin, controls.queueGroup, -2)
        end
    end
end


function CreateResizeIcon()
    local bg = Bitmap(controls.queueGrid)
	local sca_utils = '/mods/uimod/sca_utils/'
	local uimod= import(sca_utils .. 'uimod.lua')
	bg:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/iconSize_' .. uimod.factionTable[uimod.factionIndex] .. '.dds'))
	
	if UIUtil.currentLayout == "sca" then
		bg.Height:Set(boutonH)
		bg.Width:Set(boutonW)
	end	

	return bg
end

function Resizing(iconsize)
    ClearQueueGrid()
    local cols, rows = controls.queueGrid:GetDimensions()
    controls.queueGrid:AppendCols(200 - cols, true)
    local currentcol = 1
    local currentrow = 1
    for i = 1, 200 do
		controls.queueGrid:SetItem(CreateResizeIcon(), currentrow, currentcol, true)
        if currentrow >= 20 then
            controls.queueGrid:AppendRows(1, true)
            currentrow = 1
            currentcol = currentcol + 1
		else
			currentrow = currentrow + 1
		end
	end
    
	controls.queueGrid:EndBatch()
end


--toggle function used to map key to infinite queue toggle
function toggleLoop()
	OnInfiniteQueueToggle(currentInfiniteQueueCheckStatus)
end
