--*****************************************************************************
--* File: lua/ui/game/score.lua
--* Author: Chris Blackwell
--* Mod: Saya and Cleopatre --> make movable the score
--* Summary: In game score dialog
--*
--* Copyright  :005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

--[[
Armies table looks like this:
{
    numArmies (int),
    focusArmy (index),
    armiesTable (table) {
        [1] {
            name (string),
            nickname (string)
            faction (index),
            color (color),
            iconColor (color),
            showScore (bool),
        }
        etc... for all armies
    }
}
--]]

-- current score will contain the most recent score update from the sync
currentScores = false


local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local GameMain = import('/lua/ui/game/gamemain.lua')
local Group = import('/lua/maui/group.lua').Group
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Text = import('/lua/maui/text.lua').Text
local Grid = import('/lua/maui/Grid.lua').Grid
local Prefs = import('/lua/user/prefs.lua')

local timeThread = false
local walltime = Prefs.GetFromCurrentProfile("scorewalltime") or false
local dialog = false
local overlay = false
local focusArmy = 0
local scoreGrid = false
local scoreFields = {}
local animationTable = {}
local bg = false
local bgGlow = false
local bgBrackets = false
local ThreadHandle = false
local needExpand = false
local init = false

local pathuiimg='/mods/uimod/textures'
local uimod= import('/mods/uimod/sca_utils/uimod.lua')
local UIManager= import('/mods/uimod/sca_utils/uimanager.lua')
local counter = 1 -- nomber of army display
local sca_utils = '/mods/uimod/sca_utils/'
local skinning=import(sca_utils .. 'skinning.lua')


local overlay = {
    group = false,
    armyLines = {},
    time = false,
    units = false,
}

function GetScoreTop()
    if overlay.topMiddle:IsHidden() then
        return nil
    else
        return overlay.topMiddle.Top()
    end
end

function CreateArmyLine(parent)
    local group = Group(parent)
    
    group.faction = Bitmap(group)    
    group.teamcolor = Bitmap(group)    
    group.playername = UIUtil.CreateText(group, "", 14, UIUtil.bodyFont)    
    group.score = UIUtil.CreateText(group, "", 14, UIUtil.bodyFont)    
    group.namebg = Bitmap(group)
    
    group.middlebg = Bitmap(group)
    group.rightbg = Bitmap(group)
    group.leftbg = Bitmap(group)
        
	return group
end

function LayoutLine(line)
    line.score.Right:Set(line.Right)
    
    line.playername.Right:Set(function() return line.score.Left() - 10 end)
    
    line.faction.Right:Set(function() return line.playername.Left() - 11 end)
    line.faction.Bottom:Set(function() return line.Bottom() - 3 end)
    line.faction.Height:Set(15)
    line.faction.Width:Set(15)
    LayoutHelpers.FillParent(line.teamcolor, line.faction)
    line.teamcolor.Depth:Set(function() return line.faction.Depth() - 1 end)
    
    LayoutHelpers.AtVerticalCenterIn(line.score, line)
    LayoutHelpers.AtVerticalCenterIn(line.playername, line)
    
    line.namebg.Left:Set(function() return line.playername.Left() - 3 end)
    line.namebg.Right:Set(function() return line.score.Right() + 3 end)
    line.namebg.Bottom:Set(line.playername.Bottom)
    line.namebg.Height:Set(function () return line.playername.Height() + 1 end)
    line.namebg.Depth:Set(function() return line.playername.Depth() - 1 end)
    
    line.middlebg.Left:Set(line.playername.Left)
    line.middlebg.Right:Set(line.namebg.Right)
    line.middlebg.Bottom:Set(line.Bottom)
    line.middlebg.Depth:Set(function() return line.namebg.Depth() - 1 end)
    
    line.rightbg.Left:Set(function() return line.middlebg.Right() end)
    line.rightbg.Bottom:Set(line.middlebg.Bottom)
    line.rightbg.Depth:Set(line.middlebg.Depth)
    
    line.leftbg.Right:Set(function() return line.middlebg.Left() end)
    line.leftbg.Bottom:Set(line.middlebg.Bottom)
    line.leftbg.Depth:Set(line.middlebg.Depth)
end

function SetLayout()
	if not(init) then CreateScoreOverlay()
	else
		local nom_sav = "sca"
		if Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Left"] then
			if (Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Left"] ) < (GetFrame(0).Right()/2) and (Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Top"] ) < (GetFrame(0).Bottom()/2)then
				overlay.group.Left:Set(Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Left"] )
				overlay.group.Top:Set(Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Top"])
				overlay.group.Right:Set(function() return overlay.group.Left() + (overlay.armyLines[1].playername.Width() + overlay.armyLines[(counter-1)].score.Width() + 45) end)
				overlay.group.Bottom:Set(function() return overlay.group.Top() + ((counter) * overlay.armyLines[1].rightbg.BitmapHeight() + 5) end)
			elseif (Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Left"] ) > (GetFrame(0).Right()/2) and (Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Top"] ) < (GetFrame(0).Bottom()/2)then
				overlay.group.Right:Set(Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Right"])
				overlay.group.Top:Set(Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Top"])
				overlay.group.Left:Set(function() return overlay.group.Right() - (overlay.armyLines[1].playername.Width() + overlay.armyLines[(counter-1)].score.Width() + 45) end)
				overlay.group.Bottom:Set(function() return overlay.group.Top() + ((counter) * overlay.armyLines[1].rightbg.BitmapHeight() + 5) end)
			elseif (Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Left"] ) < (GetFrame(0).Right()/2) and (Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Top"] ) > (GetFrame(0).Bottom()/2)then
				overlay.group.Left:Set(Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Left"] )
				overlay.group.Bottom:Set(Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Bottom"] )
				overlay.group.Right:Set(function() return overlay.group.Left() + (overlay.armyLines[1].playername.Width() + overlay.armyLines[(counter-1)].score.Width() + 45) end)
				overlay.group.Top:Set(function() return overlay.group.Bottom() - ((counter) * overlay.armyLines[1].rightbg.BitmapHeight() + 5) end)
			else
				overlay.group.Right:Set(Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Right"])
				overlay.group.Bottom:Set(Prefs.GetFromCurrentProfile("uimod_SCA_" ..nom_sav)["score"]["Bottom"] )
				overlay.group.Left:Set(function() return overlay.group.Right() - (overlay.armyLines[1].playername.Width() + overlay.armyLines[(counter-1)].score.Width() + 45) end)
				overlay.group.Top:Set(function() return overlay.group.Bottom() - ((counter) * overlay.armyLines[1].rightbg.BitmapHeight() + 5) end)
			end
		else
			overlay.group.Top:Set(150)
			overlay.group.Left:Set(150)
		end

		UIManager.score_grp=overlay.group
  	end
	if overlay.bottomLeft then
		skinning.GetSkin("score", overlay.bottomRight, overlay.group, "br")
		skinning.GetSkin("score", overlay.bottomLeft, overlay.group, "bl")
		skinning.GetSkin("score", overlay.bottomMiddle, overlay.group, "bm")	
		skinning.GetSkin("score", overlay.topRight, overlay.group, "tr")
		skinning.GetSkin("score", overlay.topLeft, overlay.group, "tl")
		skinning.GetSkin("score", overlay.topMiddle, overlay.group, "tm")
		for i, v in overlay.armyLines do
			skinning.GetSkin("score", v.middlebg, v, "m")
			skinning.GetSkin("score", v.leftbg, v, "lm")
			skinning.GetSkin("score", v.rightbg, v, "rm")
		end
		skinning.SetPosition("score", overlay.unitsBG, overlay.bottomLeft, "units")
		skinning.SetPosition("score", overlay.timeBG, overlay.topRight, "time", "righttop")
	end
end

function CreateScoreOverlay()

  if not(init) then
	init=true
    local worldView = import('/lua/ui/game/borders.lua').GetMapGroup()
    
    local armiesInfo = GetArmiesTable()
    
    overlay.group = Group(worldView, "scoreOverlayGroup")
    overlay.group:SetRenderPass(UIUtil.UIRP_PostGlow)
	overlay.group.Depth:Set(900)
    
    SetLayout()
    
   
    for i, v in armiesInfo.armiesTable do
        if v.showScore and not v.civilian then
            overlay.armyLines[counter] = CreateArmyLine(overlay.group)
            counter = counter + 1
        end
    end
    
    local index = 0
    for i, v in overlay.armyLines do
        index = i
        if index == 1 then
            v.Bottom:Set(overlay.group.Bottom)
        else
            v.Bottom:Set(overlay.armyLines[index-1].Top)
        end
        v.Right:Set(overlay.group.Right)
        v.Height:Set(v.middlebg.Height)
        v.Left:Set(v.faction.Left)
        LayoutLine(v)
    end
	
       -- to display the size of overlay if needed
	--overlay.oftest = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/score-overlay/score_brd_lm.dds'))
       --overlay.oftest.Right:Set(overlay.group.Right)
	--overlay.oftest.Left:Set(overlay.group.Left)
	--overlay.oftest.Top:Set(overlay.group.Top)
	--overlay.oftest.Bottom:Set(overlay.group.Bottom)
    
    overlay.bottomRight = Bitmap(overlay.group)
    overlay.bottomRight.Top:Set(overlay.armyLines[1].Bottom)
    overlay.bottomRight.Right:Set(function() return overlay.armyLines[1].rightbg.Right() end)
    
    overlay.bottomLeft = Bitmap(overlay.group)
    overlay.bottomLeft.Top:Set(overlay.armyLines[1].Bottom)
    overlay.bottomLeft.Left:Set(function() return overlay.armyLines[1].leftbg.Left() end)
    
    overlay.bottomMiddle = Bitmap(overlay.group)
    overlay.bottomMiddle.Top:Set(overlay.bottomRight.Top)
    overlay.bottomMiddle.Right:Set(overlay.bottomRight.Left)
    overlay.bottomMiddle.Left:Set(function() return overlay.bottomLeft.Right() end)
    
    overlay.topRight = Bitmap(overlay.group)
	overlay.topRight.Bottom:Set(function() return overlay.armyLines[index].Top() end)
    overlay.topRight.Right:Set(function() return overlay.armyLines[1].rightbg.Right() end)
    
    overlay.topLeft = Bitmap(overlay.group)
    overlay.topLeft.Bottom:Set(function() return overlay.topRight.Bottom() end)
    overlay.topLeft.Left:Set(function() return overlay.armyLines[1].leftbg.Left() end)
    
    overlay.topMiddle = Bitmap(overlay.group)
    overlay.topMiddle.Bottom:Set(function() return overlay.armyLines[index].Top() end)
    overlay.topMiddle.Left:Set(overlay.topLeft.Right)
    overlay.topMiddle.Right:Set(overlay.topRight.Left)
    overlay.topMiddle.Height:Set(function() return overlay.topMiddle.BitmapHeight() end)
    
    overlay.unitsBG = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/score-overlay/tank_bmp.dds'))
    LayoutHelpers.AtLeftTopIn(overlay.unitsBG, overlay.bottomLeft, 15, 6)
    
    overlay.units = UIUtil.CreateText(overlay.group, "", 14, UIUtil.bodyFont)
    overlay.units:SetColor('ffff9900')
    overlay.units:SetDropShadow(true)
    LayoutHelpers.RightOf(overlay.units, overlay.unitsBG)
    
    overlay.timeBG = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/score-overlay/clock_bmp.dds'))
    LayoutHelpers.AtRightTopIn(overlay.timeBG, overlay.topRight, 16, 18)
    
    overlay.time = UIUtil.CreateText(overlay.timeBG, "", 14, UIUtil.bodyFont)
    LayoutHelpers.LeftOf(overlay.time, overlay.timeBG)
    overlay.time.Top:Set(function() return overlay.timeBG.Top() + 4 end)
    overlay.time:SetDropShadow(true)
    overlay.time:SetColor('ffff9900')
    
    overlay.group:DisableHitTest(true)
    overlay.group:Hide()
    overlay.timeBG:Hide()
    overlay.group:SetNeedsFrameUpdate(true)
    overlay.group.OnFrame = function(self, deltatime)
        if Prefs.GetFromCurrentProfile("scoreoverlay") != nil and Prefs.GetFromCurrentProfile("scoreoverlay") != false then
            overlay.group:Show()
            overlay.timeBG:Show()
        end
        overlay.group:SetNeedsFrameUpdate(false)
    end
    
    GameMain.AddBeatFunction(_BeatFunction)
    
    overlay.group.OnDestroy = function(self)
        GameMain.RemoveBeatFunction(_BeatFunction)
    end
  else
	--LOG("********WARNING Recreate score ?")
  end
end


function _BeatFunction()
    if overlay.group:IsHidden() then
        overlay.timeBG:Hide()
        overlay.group:Hide()
    else
        overlay.time:SetText(GetGameTime())
        local armiesInfo = GetArmiesTable()
        if armiesInfo then
            focusArmy = armiesInfo.focusArmy
            local sortedFocus = 0
            local sortedTable = {}
            if currentScores != nil then
                local index = 1
                for i, v in armiesInfo.armiesTable do
                    if v.showScore and not v.civilian then
                        sortedTable[index] = v
                        sortedTable[index].ID = i
                        sortedTable[index].score = currentScores[i].general.score or 0
                        if focusArmy == index then
                            if currentScores[i].general.currentunits.count and currentScores[i].general.currentcap.count then
                                overlay.units:SetText(string.format("%d/%d", 
                                    currentScores[i].general.currentunits.count, 
                                    currentScores[i].general.currentcap.count))
                            end
                            sortedTable[index].focus = true
                        else
                            sortedTable[index].focus = false
                        end
                        index = index + 1
                    end
                end
                table.sort(sortedTable, function(table1, table2)
                    if table1.score >= table2.score then
                        return false
                    else
                        return true
                    end
                end)
            end
            for i, v in sortedTable do
                local index = i
                overlay.armyLines[index].score:SetText(v.score)
                if v.outOfGame and not overlay.armyLines[index].OOG then
                    overlay.armyLines[index].OOG = true
                    overlay.armyLines[index].faction:SetTexture(UIUtil.UIFile('/game/unit-over/icon-skull_bmp.dds'))
                    overlay.armyLines[index].teamcolor:Hide()
                    overlay.armyLines[index].playername:SetColor('ffa0a0a0')
                    overlay.armyLines[index].score:SetColor("ffa0a0a0")
                    overlay.armyLines[index].playername:SetText(v.nickname)
                end
                if overlay.armyLines[index].ID != v.ID then
                    overlay.armyLines[index].ID = v.ID
                    if not v.outOfGame then
                        overlay.armyLines[index].OOG = false
                        SetBackgroundColor(overlay.armyLines[index], v.focus)
                        overlay.armyLines[index].playername:SetText(v.nickname)
                        overlay.armyLines[index].teamcolor:Show()
                        overlay.armyLines[index].teamcolor:SetSolidColor(v.color)
                        overlay.armyLines[index].faction:SetTexture(UIUtil.UIFile(UIUtil.GetFactionIcon(v.faction)))
                    end
                end
            end
            local maxPlayerWidth = 0
            local index = 0
            for i, v in overlay.armyLines do
                index = i
                if v.playername.Width() > maxPlayerWidth then
                    maxPlayerWidth = v.playername.Width()
                end
            end
            for i, v in overlay.armyLines do
                v.playername.Width:Set(math.max(maxPlayerWidth, 70))
                v.playername.Right:Set(function() return overlay.armyLines[index].score.Left() - 10 end)
            end
        end
    end
end



local unitColor = 'red'

function SetBackgroundColor(control, focus)
    if focus then
        control.playername:SetColor("FFFFFFFF")
        control.score:SetColor("FFFFFFFF")
        control.namebg:SetSolidColor("70FFFFFF")
    else
        control.playername:SetColor("FFFFFFFF")
        control.score:SetColor("FFFFFFFF")
        control.namebg:SetSolidColor("70000000")
    end
end

function Expand()
    if needExpand then
        overlay.group:Show()
        overlay.timeBG:Show()
        local sound = Sound({Cue = "UI_Score_Window_Open", Bank = "Interface",})
        PlaySound(sound)
        needExpand = false
    end
end

function Contract()
    if not overlay.group:IsHidden() then
        local sound = Sound({Cue = "UI_Score_Window_Close", Bank = "Interface",})
        PlaySound(sound)
        overlay.timeBG:Hide()
        overlay.group:Hide()
        needExpand = true
    else
        needExpand = false
    end
end

function ToggleScoreControl()

    # disable when in Screen Capture mode
    if import('/lua/ui/game/gamemain.lua').gameUIHidden then
        return
    end

    if overlay.group:IsHidden() then
        Prefs.SetToCurrentProfile("scoreoverlay", true)
        overlay.group:Show()
        overlay.timeBG:Show()
        local sound = Sound({Cue = "UI_Score_Window_Open", Bank = "Interface",})
        PlaySound(sound)
    else
        Prefs.SetToCurrentProfile("scoreoverlay", false)
        local sound = Sound({Cue = "UI_Score_Window_Close", Bank = "Interface",})
        PlaySound(sound)
        overlay.timeBG:Hide()
        overlay.group:Hide()
    end
end
