--*****************************************************************************
--* File: lua/ui/game/selectionmanager.lua
--* Author: Chris Blackwell / Ted Snook
--* Summary: Unit selection manager UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Group = import('/lua/maui/group.lua').Group
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Grid = import('/lua/maui/grid.lua').Grid
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Button = import('/lua/maui/button.lua').Button
local GameCommon = import('/lua/ui/game/gamecommon.lua')
local RadioGroup = import('/lua/maui/mauiutil.lua').RadioGroup
local Tooltip = import('/lua/ui/game/tooltip.lua')
local pathuiimg='/mods/uimod/textures'

local currentSelection = false

controls = {
    panel = false,
    panelmin = false,
    panelstretch = false,
    panelmax = false,
    iconGrid = false,
    panelLogo = false,
    scrollup = false,
    scrolldown = false,
    mfdControl = false,
    ordersControl = false,
    queueControl = false,
    controlClusterGroup = false,
}    

boutonH = 48
boutonW = 48
          
function SelectionManager(selection, newSelection)
    import('/lua/ui/game/unitviewDetail.lua').Hide()
    currentSelection = newSelection
    local tempBuildableUnits = EntityCategoryGetUnitList(selection)
    if table.getsize(newSelection) == 1 then
        local bp = currentSelection[1]:GetBlueprint()
        local attachedList = EntityCategoryFilterDown(categories.MOBILE, GetAttachedUnitsList(newSelection))
        local tempFocusUnit = currentSelection[1]:GetFocus() 
        local isUpgrading = false
        if tempFocusUnit and currentSelection[1] then
            if tempFocusUnit:GetBlueprint().General.UpgradesFrom == bp.BlueprintId then
                isUpgrading = true
            end
        end
        if table.getsize(tempBuildableUnits) > 0 or bp.Enhancements != nil or isUpgrading then
            import('/lua/ui/game/construction.lua').SetAvailableGridButtons(selection, newSelection, tempBuildableUnits)
            controls.panelmin:Hide()
        elseif table.getsize(attachedList) > 0 and not EntityCategoryContains(categories.FACTORY, newSelection[1]) then
            controls.panelmin:Show()
            SetAttachedIcons(attachedList) 
        else
            local tempSelectionSet = {{currentSelection[1]:GetBlueprint().BlueprintId, 1, {currentSelection[1]}}}
            SetSelectionIcons(tempSelectionSet) 
            controls.panelmin:Show()
        end
    else
        controls.panelmin:Show()
        --Sort out units by type
        local sortedUnits = {}
        for i, v in currentSelection do
            local blueprintID = v:GetBlueprint().BlueprintId
            if table.getn(sortedUnits) > 0 then
                local newEntry = true
                for h, x in sortedUnits do
                    if x[1] == blueprintID then
                        x[2] = x[2] + 1
                        table.insert(x[3], v)
                        newEntry = false
                    end
                end
                if newEntry then
                    table.insert(sortedUnits, {blueprintID, 1, {v}})
                end
            else
                table.insert(sortedUnits, {blueprintID, 1, {v}})
            end
        end

        if table.getsize(sortedUnits) == 1  then
			--ENHANCE MOD, so multiple selections of units without building options show their enhancement (cybran shield gens)
            local bp = currentSelection[1]:GetBlueprint()
            if table.getsize(tempBuildableUnits) > 0 or bp.Enhancements then
                import('/lua/ui/game/construction.lua').SetAvailableGridButtons(selection,newSelection, tempBuildableUnits)
                controls.panelmin:Hide()
                return
            else
                SetSelectionIcons(sortedUnits)
                return
            end
        end
        
        local categoryList = { categories.COMMAND, categories.EXPERIMENTAL, categories.TECH3, categories.TECH2, categories.TECH1, categories.OPERATION }
        
        local newSortedUnits = {}
        for i, v in categoryList do
            for h, x in sortedUnits do
                if EntityCategoryContains(v, x[3][1]) then
                    table.insert(newSortedUnits, x)
                end
            end
        end
        
        SetSelectionIcons(newSortedUnits) 
    end
end


function CreateQueueButton(parent)
    local bg = Bitmap(parent) -- TODO unit specific BG here
    bg:SetSolidColor('00000000')
    #bg.Width:Set(GameCommon.iconBmpWidth)
    #bg.Height:Set(GameCommon.iconBmpHeight)
    local icon = Button(bg)
    LayoutHelpers.AtLeftTopIn(icon, bg)
    #LayoutHelpers.FillParent(icon, bg)
	
   if UIUtil.currentLayout == "minimal" then
        bg.Height:Set(48)
        bg.Width:Set(48)
        icon.Height:Set(48)
        icon.Width:Set(48)
	elseif UIUtil.currentLayout == "bottom" or UIUtil.currentLayout == "left" or UIUtil.currentLayout == "right" then 
        icon.Height:Set(62)
        icon.Width:Set(62)	
	elseif UIUtil.currentLayout == "sca" then
		bg.Height:Set(boutonH)
		bg.Width:Set(boutonW)
		icon.Height:Set(boutonH)
		icon.Width:Set(boutonW)
	end	


	
    bg.unitIcon = icon
    bg:Hide()

    return bg
end

function CreateQueueIcon(parent)
    local bg = Bitmap(parent) -- TODO unit specific BG here
    bg:SetSolidColor('00000000')
    #bg.Width:Set(GameCommon.iconBmpWidth)
    #bg.Height:Set(GameCommon.iconBmpHeight)
    local icon = Bitmap(bg)
    LayoutHelpers.AtLeftTopIn(icon, bg)
    #LayoutHelpers.FillParent(icon, bg)
    local text = UIUtil.CreateText(icon, "", 18, UIUtil.bodyFont)
    LayoutHelpers.AtLeftTopIn(text, icon, 5, 5)
    text:SetDropShadow(true)
    text:DisableHitTest()
    text:SetColor('white')
	
	
   if UIUtil.currentLayout == "minimal" then
        bg.Height:Set(48)
        bg.Width:Set(48)
        icon.Height:Set(48)
        icon.Width:Set(48)
	elseif UIUtil.currentLayout == "bottom" or UIUtil.currentLayout == "left" or UIUtil.currentLayout == "right" then 
        icon.Height:Set(62)
        icon.Width:Set(62)	
	elseif UIUtil.currentLayout == "sca" then
		bg.Height:Set(boutonH)
		bg.Width:Set(boutonW)
		icon.Height:Set(boutonH)
		icon.Width:Set(boutonW)
	end	


    bg.text = text
    bg.unitIcon = icon
    bg:Hide()

    return bg
end

function SetSelectionIcons(sortedUnitTable)
    ClearQueueGrid()
    import('/lua/ui/game/construction.lua').SetConstructionState(true)
    import('/lua/ui/game/queue.lua').SetQueueState(true)
    local numTypes = table.getn(sortedUnitTable)
    local numIcons = 0
    if numTypes and numTypes > 0 then
        local iconBPID = nil
        local cols, rows = controls.iconGrid:GetDimensions()
        local maxcols, maxrows = controls.iconGrid:GetVisible()
        numIcons = table.getn(sortedUnitTable)
        
        PopulateQueueGrid(numIcons)
        
        local i = 1
        local curcol = 1
        for index, v in sortedUnitTable do
            local icon = controls.iconGrid:GetItem(i, curcol)
            if not icon then
                WARN("Selection Manager: SetSelectionIcons attempt to get item ", i, " but it's not there.")
            end
            local blueprint = __blueprints[v[1]]
            local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
            icon.unitIcon:SetTexture(upIconName)
            icon.text:SetText(v[2])
            icon._position = i
            icon._unitType = v[1]
            icon._units = v[3]
            if i >= maxcols then
                i = 1
                curcol = curcol + 1
            else
                i = i + 1
            end
            
			import('/mods/uimod/sca_utils/uimanager.lua').addStrategicIconTo(icon, blueprint)

				
            icon.HandleEvent = function(self, event)
                if event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
                    if event.Modifiers.Left then
                        SelectUnits(self._units)
                    else
                        local ind = 1
                        local newUnitSelectTable = {}
                        for i, v in sortedUnitTable do
                            if self._unitType != v[1] then
                                for y, z in v[3] do
                                    newUnitSelectTable[ind] = z
                                    ind = ind + 1
                                end
                            end
                        end
                        SelectUnits(newUnitSelectTable)
                    end
                elseif event.Type == 'MouseEnter' then
                    local blueprint = __blueprints[self._unitType]
                    --import('/lua/ui/game/unitviewDetail.lua').Show(blueprint, nil, self._unitType)
                    local tempText = blueprint.Description
                    local tempDesc = ''
                    if blueprint.General.UnitName then
                        tempText = blueprint.General.UnitName
                        tempDesc = blueprint.Description
                    end
                    Tooltip.CreateMouseoverDisplay(self.unitIcon, {text = tempText, body = tempDesc}, 0, true)
                elseif event.Type == 'MouseExit' then
                    --import('/lua/ui/game/unitviewDetail.lua').Hide()
                    Tooltip.DestroyMouseoverDisplay()
                end
                Bitmap.HandleEvent(self, event)
            end
        end
    else
        controls.scrollup:Disable()
        controls.scrolldown:Disable()
        if controls.panelmin.Mini then
            controls.panelmin.Mini(true)
        end
    end
        
    controls.iconGrid:EndBatch()
end

function SetAttachedIcons(attachedUnits)
    ClearQueueGrid()
    local numTypes = table.getn(attachedUnits)
    local numIcons = 0
    if numTypes then
        local iconBPID = nil
        local cols, rows = controls.iconGrid:GetDimensions()
        local maxcols, maxrows = controls.iconGrid:GetVisible()
        numIcons = table.getn(attachedUnits)
        
        PopulateQueueGrid(numIcons, true)
        
        local i = 1
        local curcol = 1
        for index, v in attachedUnits do
            local blueprintID = v:GetBlueprint().BlueprintId
            local blueprint = __blueprints[blueprintID]
            local icon = controls.iconGrid:GetItem(i, curcol)
            local userUnit = v
            if not icon then
                WARN("Selection Manager: SetSelectionIcons attempt to get item ", i, ":", curcol, " but it's not there.")
            end
            local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
            icon.unitIcon:SetTexture(upIconName)
            icon.unitIcon:SetNewTextures(upIconName, downIconName, overIconName, iconName)
            icon.unitIcon:SetUpAltButtons( downIconName, downIconName, downIconName, downIconName )
            icon.unitIcon._unitType = blueprintID
            if i >= maxcols then
                i = 1
                curcol = curcol + 1
            else
                i = i + 1
            end
            
			import('/mods/uimod/sca_utils/uimanager.lua').addStrategicIconTo(icon, blueprint)

            icon.unitIcon.OnClick = function(self, modifiers)
                if modifiers.Left then
                    -- Toggling selection of the entity
                    self:OnAltToggle()
    
                    -- Add or Remove the entity to the session selection
                    if self.mAltToggledFlag then
                        AddToSessionExtraSelectList(userUnit)
                    else
                        RemoveFromSessionExtraSelectList(userUnit)
                    end
                end
            end
            icon.unitIcon.HandleEvent = function(self, event)
                if event.Type == 'MouseEnter' then
                    local blueprint = __blueprints[self._unitType]
                    --import('/lua/ui/game/unitviewDetail.lua').Show(blueprint, nil, self._unitType)
                    local tempText = blueprint.Description
                    local tempDesc = ''
                    if blueprint.General.UnitName then
                        tempText = blueprint.General.UnitName
                        tempDesc = blueprint.Description
                    end
                    Tooltip.CreateMouseoverDisplay(self, {text = tempText, body = tempDesc}, 0, true)
                elseif event.Type == 'MouseExit' then
                    --import('/lua/ui/game/unitviewDetail.lua').Hide()
                    Tooltip.DestroyMouseoverDisplay()
                end
                Button.HandleEvent(self, event)
            end
        end
    end
    controls.iconGrid:EndBatch()
end


function CreateResizeIcon()
    local bg = Bitmap(controls.iconGrid)
	local sca_utils = '/mods/uimod/sca_utils/'
	local uimod= import(sca_utils .. 'uimod.lua')
	bg:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/iconSize_' .. uimod.factionTable[uimod.factionIndex] .. '.dds'))
	
	if UIUtil.currentLayout == "sca" then
		bg.Height:Set(boutonH)
		bg.Width:Set(boutonW)
	end	

	return bg
end

function Resizing(iconsize)
    ClearQueueGrid()
    local cols, rows = controls.iconGrid:GetDimensions()
    controls.iconGrid:AppendCols(200 - cols, true)
    local currentcol = 1
    local currentrow = 1
    for i = 1, 200 do
		controls.iconGrid:SetItem(CreateResizeIcon(), currentrow, currentcol, true)
        if currentrow >= 20 then
            controls.iconGrid:AppendRows(1, true)
            currentrow = 1
            currentcol = currentcol + 1
		else
			currentrow = currentrow + 1
		end
	end
    
	controls.iconGrid:EndBatch()
end
