local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Button = import('/lua/maui/button.lua').Button
local Effect = import('/lua/maui/effecthelpers.lua')
local Prefs = import('/lua/user/prefs.lua')--preferences
local Edit = import('/lua/maui/edit.lua').Edit
local Window = import('/lua/maui/window.lua').Window
local Grid = import('/lua/maui/grid.lua').Grid

local sca_utils = '/mods/uimod/sca_utils/'
local uimanager = import(sca_utils .. 'uimanager.lua')
local uimod = import(sca_utils .. 'uimod.lua')
local skinning= import(sca_utils .. 'skinning.lua')
local skinfile = import(skinning.GetActualSkinFile())
local skintable = skinfile.skinlist

local CreateText_sca = import(sca_utils .. 'modcontrols.lua').CreateText_sca
local windowparent = import('/lua/ui/game/gamemain.lua').GetGameParent()
local pathuiimg='/mods/uimod/textures'
local modcontrols= import(sca_utils .. 'modcontrols.lua')
local Make_ListBox = import(sca_utils .. 'modcontrols.lua').ListBox
local Make_CheckBox = import(sca_utils .. 'modcontrols.lua').Make_CheckBox
local trad = import(sca_utils .. 'modcontrols.lua').trad

local window = false
local basicwindow = false
local dialog = false
local factions = false
local init = true
local textLine = {}
local defaultLayoutTable = {}

function Init()
	defaultLayoutTable = {}
	local allLayouts = Prefs.GetFromCurrentProfile("uimanager_sav")["layouts"]
	--make table of premade layouts, add any new ones to the list of all saved layouts
	for i, skin in skinfile.skinlist do
		if skinfile[skin].layouts then
			for layout in skinfile[skin].layouts do
				if layout == 'UseDefaults' then
					for i, defaultLayout in skinfile[skin].layouts.UseDefaults do
						if i != 'preview' then
							local found = false
							for i, layoutData in allLayouts do
								if layoutData[1] == skin and layoutData[2] == defaultLayout then
									found = true
								end
							end
							if not found then
								LOG('new layout found: ' ..defaultLayout)
								table.insert(uimanager.uimanager_sav.layouts, {skin, defaultLayout, false})
								table.insert(uimanager.uimanager_sav.basicLayouts, {skin, defaultLayout, false})
								Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
							end
						end
					end
				else
					table.insert(defaultLayoutTable, {skin, layout})
					local found = false
					for i, layoutData in allLayouts do
						if layoutData[1] == skin and layoutData[2] == layout then
							found = true
						end
					end
					if not found then
						LOG('new layout found: ' ..layout)
						table.insert(uimanager.uimanager_sav.layouts, {skin, layout, false})
						table.insert(uimanager.uimanager_sav.basicLayouts, {skin, layout, false})
						Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
					end
				end
			end
		end
	end
	--check for any saved premade layouts which have been removed and remove them from the saved list (make a list and remove backwards to avoid index problems)
	local deletedList = {}
	for i, layoutData in Prefs.GetFromCurrentProfile("uimanager_sav")["layouts"] do
		if not layoutData[3] then
			local foundskin = false
			local foundlayout = false
			for i, layoutData2 in defaultLayoutTable do
				for i, skin in skinfile.skinlist do
					if layoutData[1] == skin then
						foundskin = true
					end
				end
				if layoutData[2] == layoutData2[2] then
					foundlayout = true
				end
			end
			if not foundskin or not foundlayout then
				table.insert(deletedList, i)
				LOG('Removing default layout: ' ..layoutData[2])
			end
		end
	end
	local deletedBasicList = {}
	for i, layoutData in Prefs.GetFromCurrentProfile("uimanager_sav")["basicLayouts"] do
		if not layoutData[3] then
			local foundskin = false
			local foundlayout = false
			for i, layoutData2 in defaultLayoutTable do
				for i, skin in skinfile.skinlist do
					if layoutData[1] == skin then
						foundskin = true
					end
				end
				if layoutData[2] == layoutData2[2] then
					foundlayout = true
				end
			end
			if not foundskin or not foundlayout then
				table.insert(deletedBasicList, i)
				LOG('Removing default layout: ' ..layoutData[2])
			end
		end
	end

	table.sort(deletedList, function(a, b) return a>b end)
	for i, v in deletedList do
		table.remove(uimanager.uimanager_sav.layouts, v)
		Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
	end
	for i, v in deletedBasicList do
		table.remove(uimanager.uimanager_sav.basicLayouts, v)
		Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
	end
end

function CreateLayoutChooser(parent,left,top)
	--layout selection list
	currentskin = skinChooser.currentValue
	local equivalentSkins = {currentskin}
	local savedLayouts = Prefs.GetFromCurrentProfile("uimanager_sav")["layouts"]
	local tab = {}
	--if autofaction then display all layouts, else display only layouts linked to the displayed skin, and any skins declared equivalent to the displayed one (in the skinfile)
	if currentskin == 'Auto Faction' then
		for i, data in savedLayouts do
			table.insert(tab, data[2])
		end
	else
		if skinfile[currentskin].equivalentSkins then
			for i, skin in skinfile[currentskin].equivalentSkins do
				table.insert(equivalentSkins, skin)
			end
		end
		for i, data in defaultLayoutTable do
			for i, skin in equivalentSkins do
				if data[1] == skin then
					table.insert(tab, data[2])
				end
			end
		end
		for i, data in savedLayouts do
			for i, skin in equivalentSkins do
				if data[1] == skin and data[3] then
					table.insert(tab, data[2])
				end
			end
		end
	end
	layoutList = Make_ListBox(parent, left, top, 105, "normal",15,false,tab, 11,'')
	layoutList.setValue(uimanager.uimanager_sav.currentscaLayout[2])
end

function CreateSkinGroup(parent, left, top)
	local skinGroup = Bitmap(parent)
	LayoutHelpers.AtLeftTopIn(skinGroup, parent, left, top)
	skinGroup:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/layoutmanager/skins_bg.dds'))
	local skinText = CreateText_sca(parent, ("skin"), 28, UIUtil.titleFont)
	LayoutHelpers.AtLeftTopIn(skinText, skinGroup, 10, 7)
	skinIcon_bg = Bitmap(skinGroup)
	skinIcon_bg:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/layoutmanager/icon_bg.dds'))
	LayoutHelpers.AtLeftTopIn(skinIcon_bg, skinGroup, 139, 3)
	skinIcon = Bitmap(skinIcon_bg)
	LayoutHelpers.AtCenterIn(skinIcon, skinIcon_bg)
	skinIcon.Height:Set(58)
	skinIcon.Width:Set(58)
	skinIcon:SetTexture('/mods/uimod/skins/default_icon.dds')
	skinInfoBox = Bitmap(skinGroup)
	LayoutHelpers.AtLeftTopIn(skinInfoBox, skinGroup, 2, 62)
	skinInfoBox:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/layoutmanager/info_bg.dds'))

	
	factions = Bitmap(skinGroup)
	LayoutHelpers.AtLeftTopIn(factions, skinGroup, 2, 130)
	factions.Depth:Set(skinGroup.Depth() + 100)
	factions:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/layoutmanager/factions_bg.dds'))
	local faction_title= CreateText_sca(factions, "factionskin" , 16, UIUtil.titleFont)
	LayoutHelpers.AtLeftTopIn(faction_title, factions, 6,5)
	AText = UIUtil.CreateText(factions, 'Aeon:', 13)
	LayoutHelpers.AtLeftTopIn(AText, factions, 6, 30)
	AskinChooser = Make_ListBox(factions, 62, 25, 100, "normal", 15, false, skintable, 8, '', nil, nil, nil, true)
	CText = UIUtil.CreateText(factions, 'Cybran:', 13)
	LayoutHelpers.AtLeftTopIn(CText, factions, 6, 61)
	CskinChooser = Make_ListBox(factions, 62, 56, 100, "normal", 15, false, skintable, 8, '', nil, nil, nil, true)
	UText = UIUtil.CreateText(factions, 'UEF:', 13)
	LayoutHelpers.AtLeftTopIn(UText, factions, 6, 92)
	UskinChooser = Make_ListBox(factions, 62, 87, 100, "normal", 15, false, skintable, 8, '', nil, nil, nil, true)

	AskinChooser.minimizedBtn.OnHide = function(self, hidden)
		if hidden then
			CskinChooser:Hide()
			CskinChooser.minimizedBtn:Show()
			UskinChooser:Hide()
			UskinChooser.minimizedBtn:Show()		
		end
	end
	CskinChooser.minimizedBtn.OnHide = function(self, hidden)
		if hidden then
			AskinChooser:Hide()
			AskinChooser.minimizedBtn:Show()
			UskinChooser:Hide()
			UskinChooser.minimizedBtn:Show()		
		end
	end	UskinChooser.minimizedBtn.OnHide = function(self, hidden)
		if hidden then
			CskinChooser:Hide()
			CskinChooser.minimizedBtn:Show()
			AskinChooser:Hide()
			AskinChooser.minimizedBtn:Show()		
		end
	end
	AskinChooser.setValue(uimanager.uimanager_sav.AEONskin)
	CskinChooser.setValue(uimanager.uimanager_sav.CYBRANskin)
	UskinChooser.setValue(uimanager.uimanager_sav.UEFskin)
	CreateSkinChooser(skinGroup, 198, 5)
	
	if skinChooser.currentValue == 'Auto Faction' then
		factions:SetAlpha(1, true)
	else
		factions:SetAlpha(0.5, true)
		factions:DisableHitTest(true)
	end
end

function CreateSkinChooser(parent, left, top)
	skinChooser = Make_ListBox(parent, left, top, 107, "normal", 15, false, skintable, 9, '', nil, nil, nil)
	skinChooser.insertElement('Auto Faction')
	skinChooser.OnSelect = function()
		local val = skinChooser.currentValue
		local icontex = '/mods/uimod/skins/default_icon.dds'
		local skininfo = ''
		if val == 'Auto Faction' then
			icontex = '/mods/uimod/skins/auto-fac_icon.dds'
			skininfo = 'Automatically switches between selected skins depending on current faction.'
			factions:SetAlpha(1, true)
			factions:EnableHitTest(true)
		else
			if skinfile[val].icon then
				icontex = skinfile[val].path .. skinfile[val].icon
			end
			if skinfile[val].info then
				skininfo = skinfile[val].info
			end
			factions:SetAlpha(0.5, true)
			factions:DisableHitTest(true)
		end
		if not init then
			layoutList:Destroy()
			CreateLayoutChooser(window, 420, 52)
		end
		skinIcon:SetTexture(icontex)
		updateSkinInfo(skininfo)
	end

	local Group = import('/lua/maui/group.lua').Group

	skinChooser.setValue(uimanager.uimanager_sav.skin)
	skinChooser.OnSelect()
end

function updateSkinInfo(infoText)
	for i, v in textLine do
		v:Destroy()
	end
	textLine = {}
	textLine[1] = UIUtil.CreateText(skinInfoBox, "", 11)
	textLine[1].Top:Set(skinInfoBox.Top() + 4)
	textLine[1].Left:Set(skinInfoBox.Left() + 5)

	local tempTable = import('/lua/maui/text.lua').WrapText(infoText, 190,
	function(text)
		return textLine[1]:GetStringAdvance(text)
	end)

	local prevControl = false
	for i, v in tempTable do
		if i == 1 then
			textLine[1]:SetText(v)
			prevControl = textLine[1]
		else
			textLine[i] = UIUtil.CreateText(skinInfoBox, v, 11)
			LayoutHelpers.Below(textLine[i], prevControl)
			textLine[i].Left:Set(skinInfoBox.Left() + 5)
			prevControl = textLine[i]
		end
	end
end

function CreateBasicManagerGroup(parent, left, top)
	local managerGroup = Bitmap(parent)
	LayoutHelpers.AtLeftTopIn(managerGroup, parent, left, top)
	managerGroup.Width:Set(200)
	managerGroup.Height:Set(100)
	local managerText = CreateText_sca(parent, ("manager"), 20, UIUtil.titleFont)
	LayoutHelpers.AtLeftTopIn(managerText, managerGroup, 10, 7)
	local basicLayouts = Prefs.GetFromCurrentProfile("uimanager_sav")["basicLayouts"]
	local boxTable = {}
	for i, data in basicLayouts do
		table.insert(boxTable, data[1]..': '..data[2])
	end
	managerBox = Make_ListBox(managerGroup, 10, 30, 220, "normal", 30, false, boxTable, 4, '', nil, nil, nil)
	local addBtn = uimanager.Make_Button_generic(managerGroup, 'add', false, -167, 40, modcontrols.trad("LM_add_layout"))
	addBtn.OnQuit = function()
		for i, data in uimanager.uimanager_sav.basicLayouts do
			if data[1] == uimanager.uimanager_sav.currentscaLayout[1] and data[2] == uimanager.uimanager_sav.currentscaLayout[2] then
				UIUtil.QuickDialog(window, 'WARNING: Layout is already in the list.',  "<LOC _OK>", function() end, nil, nil, nil, nil, true, {escapeButton = 1, enterButton = 1})
				return
			end
		end
		managerBox.insertElement(uimanager.uimanager_sav.currentscaLayout[1]..': '..uimanager.uimanager_sav.currentscaLayout[2])
		table.insert(uimanager.uimanager_sav.basicLayouts, {uimanager.uimanager_sav.currentscaLayout[1], uimanager.uimanager_sav.currentscaLayout[2], uimanager.uimanager_sav.defaulLayout or false, uimanager.uimanager_sav["AEONskin"], uimanager.uimanager_sav["CYBRANskin"], uimanager.uimanager_sav["UEFskin"]})
		Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
	end
	local removeBtn = uimanager.Make_Button_generic(managerGroup, 'remove', false, -167, 0, modcontrols.trad("LM_remove_layout"))
	removeBtn.OnQuit = function()
		local delSkin = string.gsub(managerBox.currentValue, '(.*): (.*)', '%1')
		local delLayout = string.gsub(managerBox.currentValue, '(.*): (.*)', '%2')
		for i, data in uimanager.uimanager_sav.basicLayouts do
			if data[1] == delSkin and data[2] == delLayout then
				table.remove(uimanager.uimanager_sav.basicLayouts, i)
			end
		end
		managerBox.removeItem(managerBox.currentValue)
		Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
	end
	local resetBtn = uimanager.Make_Button_generic(managerGroup, 'reset', false, -167, -40, modcontrols.trad("LM_reset_layout"))
	resetBtn.OnQuit = function()
		uimanager.uimanager_sav.basicLayouts = table.copy(uimanager.uimanager_sav.layouts)
		for i, value in managerBox.getValues() do
			managerBox.removeItem(value)
		end
		for i, data in uimanager.uimanager_sav.basicLayouts do
			managerBox.insertElement(data[1]..': '..data[2])
		end
		Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
	end
end
	
function selectLayout(skin, name, override)
	--select layout
	--defaultSkin is to override the skin choice to allow loading of default layouts in skins they weren't made in
	local default = false
	local factional = false
	local defaultSkin = false
	print("selecting layout: " ..skin.. ' - '.. name)
	for i, v in defaultLayoutTable do
		if v[2] == name then
			default = true
			defaultSkin = v[1]
			if skin == defaultSkin then
				if skinfile[defaultSkin].layouts[name].factionskins and not override then
					factional = true
					print("skin is faction dependant")
				end
			end
		end
	end
		
	if default then
		uimod.loadDefaultLayout(defaultSkin, name)
	else
		if Prefs.GetFromCurrentProfile('uimod_SCA_' .. name) then
			uimod.loadUserLayout(skin, name)
		else
			print("undefined layout")
		end
	end
	
	uimanager.uimanager_sav = Prefs.GetFromCurrentProfile("uimanager_sav")
	uimanager.uimanager_sav.currentscaLayout = {skin, name}
	if factional then
		uimanager.uimanager_sav.skin = 'Auto Faction'
	else
		uimanager.uimanager_sav.skin = skin
	end
	if default then
		uimanager.uimanager_sav.defaultLayout = true
	else
		uimanager.uimanager_sav.defaultLayout = false
	end
	Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
	uimod.RefreshLayout()
	uimanager.resizeIcons(math.ceil(64*(uimanager.uimanager_sav["iconsize"]) + 32))
	uimanager.bRefresh_allow = false
	uimanager.reset()
	uimanager.bRefresh_allow = true
end

function Save()
	local tempIndex = false
	for i, layoutData in uimanager.uimanager_sav.layouts do
		if layoutData[1] == skinChooser.currentValue and layoutData[2] == layoutList.currentValue then
			tempIndex = i
		end
	end
	if tempIndex then
		local tempBasicIndex = false
		for i, layoutData in uimanager.uimanager_sav.basicLayouts do
			if layoutData[1] == skinChooser.currentValue and layoutData[2] == layoutList.currentValue then
				tempBasicIndex = i
			end
		end
		if tempBasicIndex then
			uimanager.uimanager_sav.basicLayouts[tempBasicIndex] = {skinChooser.currentValue, layoutList.currentValue, true, AskinChooser.currentValue, CskinChooser.currentValue, UskinChooser.currentValue}
		end
		uimanager.uimanager_sav.layouts[tempIndex] = {skinChooser.currentValue, layoutList.currentValue, true, AskinChooser.currentValue, CskinChooser.currentValue, UskinChooser.currentValue}
		uimanager.uimanager_sav.currentscaLayout = {skinChooser.currentValue, layoutList.currentValue}
		uimanager.uimanager_sav.defaultLayout = false
		Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
		uimod.saveModValues(layoutList.currentValue)
	else
		print('WARNING: Error saving layout.')
	end
end

--shows a dialog asking for a name and saves the current layout under that name
function SaveAs()
    -- Dialog already showing? Don't show another one
    if dialog then return end
	
    dialog = Bitmap(window, UIUtil.SkinnableFile('/dialogs/dialog_02/panel_bmp.dds'), "Rename Dialog")
    LayoutHelpers.AtCenterIn(dialog, window)
    dialog.Depth:Set(function() return window.Depth() + 1000 end)
	
    local label = UIUtil.CreateText(dialog, "Save your Layout", 16, UIUtil.buttonFont)
    label.Top:Set(function() return dialog.Top() + 30 end)
    label.Left:Set(function() return dialog.Left() + 35 end)
    
    local cancelButton = UIUtil.CreateButtonStd(dialog, '/widgets02/small', "<LOC _CANCEL>", 12)
    cancelButton.Top:Set(function() return dialog.Top() + 112 end)
    cancelButton.Left:Set(function() return dialog.Left() + (((dialog.Width() / 4) * 1) - (cancelButton.Width() / 2)) end)
    cancelButton.OnClick = function(self, modifiers)
        dialog:Destroy()
        dialog = false
    end

    --TODO this should be in layout
    local nameEdit = Edit(dialog)
    LayoutHelpers.AtLeftTopIn(nameEdit, dialog, 35, 60)
    nameEdit.Width:Set(283)
    nameEdit.Height:Set(nameEdit:GetFontHeight())
    nameEdit:ShowBackground(false)
    nameEdit:AcquireFocus()
    UIUtil.SetupEditStd(nameEdit, UIUtil.fontColor, nil, nil, nil, UIUtil.bodyFont, 16, 30)

    local firstTime = true

    nameEdit.OnEnterPressed = function(self, text)    
        local newName = nameEdit:GetText()
		local vals = Prefs.GetFromCurrentProfile("uimanager_sav")["layouts"]
		local exists = false
        if (newName != nil) and (newName != "") then
			for i, layoutData in defaultLayoutTable do
				if layoutData[2] == newName then
					UIUtil.QuickDialog(window, 'WARNING: Cannot save over a default layout',  "<LOC _OK>", function() end, nil, nil, nil, nil, true, {escapeButton = 1, enterButton = 1})
					return
				end
			end
			for i, v in vals do
				if v[2] == newName then
					exists = true
				end
			end
			if exists then
				layoutList.setValue(newName)
				UIUtil.QuickDialog(window, 'Are you sure you want to overwrite layout "' ..newName.. '"?',  "<LOC _OK>", function() Save() end, "<LOC _Cancel>", function() end, nil, nil, true, {escapeButton = 2, enterButton = 1})
			else
				table.insert(uimanager.uimanager_sav.layouts, {skinChooser.currentValue, newName, true, AskinChooser.currentValue, CskinChooser.currentValue, UskinChooser.currentValue})
				table.insert(uimanager.uimanager_sav.basicLayouts, {skinChooser.currentValue, newName, true, AskinChooser.currentValue, CskinChooser.currentValue, UskinChooser.currentValue})
				uimanager.uimanager_sav.currentscaLayout = {skinChooser.currentValue, newName}
				uimanager.uimanager_sav.defaultLayout = false
				layoutList.insertElement(newName)
				managerBox.inserElement({skinChooser.currentValue, newName})
				Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
				print(newName)
				uimod.saveModValues(newName)
				layoutList.setValue(uimanager.uimanager_sav.currentscaLayout)
			end
		end
        dialog:Destroy()
        dialog = false
		return true
	end

    dialog:SetNeedsFrameUpdate(true)
    dialog.OnFrame = function(self, elapsedTime)
        -- this works around the fact that wxWindows processes keys and then generates a wmChar message
        -- so if you don't set the text you'll see the hotkey that made this dialog
        if firstTime then
            nameEdit:SetText("mylayout")
            firstTime = false
        end
    end


    local okButton = UIUtil.CreateButtonStd(dialog, '/widgets02/small', "OK", 12)
    okButton.Top:Set(function() return dialog.Top() + 112 end)
    okButton.Left:Set(function() return dialog.Left() + (((dialog.Width() / 4) * 3) - (okButton.Width() / 2)) end)
    okButton.OnClick = function(self, modifiers)
        local newName = nameEdit:GetText()
		local vals = Prefs.GetFromCurrentProfile("uimanager_sav")["layouts"]
		local exists = false
        if (newName != nil) and (newName != "") then
			for i, layoutData in defaultLayoutTable do
				if layoutData[2] == newName then
					UIUtil.QuickDialog(window, 'WARNING: Cannot save over a default layout',  "<LOC _OK>", function() end, nil, nil, nil, nil, true, {escapeButton = 1, enterButton = 1})
					return
				end
			end
			for i, v in vals do
				if v[2] == newName then
					exists = true
				end
			end
			if exists then
				layoutList.setValue(newName)
				UIUtil.QuickDialog(window, 'Are you sure you want to overwrite layout "' ..newName.. '"?',  "<LOC _OK>", function() Save() end, "<LOC _Cancel>", function() end, nil, nil, true, {escapeButton = 2, enterButton = 1})
			else
				table.insert(uimanager.uimanager_sav.layouts, {skinChooser.currentValue, newName, true, AskinChooser.currentValue, CskinChooser.currentValue, UskinChooser.currentValue})
				table.insert(uimanager.uimanager_sav.basicLayouts, {skinChooser.currentValue, newName, true, AskinChooser.currentValue, CskinChooser.currentValue, UskinChooser.currentValue})
				uimanager.uimanager_sav.currentscaLayout = {skinChooser.currentValue, newName}
				uimanager.uimanager_sav.defaultLayout = false
				layoutList.insertElement(newName)
				Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
				print(newName)
				uimod.saveModValues(newName)
				layoutList.setValue(uimanager.uimanager_sav.currentscaLayout)
			end
		end
        dialog:Destroy()
        dialog = false
    end

end

function Delete()
	for i, v in uimanager.uimanager_sav.layouts do
		if v[2] == layoutList.currentValue then
			table.remove(uimanager.uimanager_sav.layouts, i)
			for i2, layouts in uimanager.uimanager_sav.basicLayouts do
				if layouts[2] == layoutList.currentValue then
					table.remove(uimanager.uimanager_sav.basicLayouts, i2)
				end
			end
			layoutList.removeItem(v[2])
			Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
			Prefs.SetToCurrentProfile("uimod_SCA_" .. v[2], nil)
		end
	end
end


function ApplySettings()
	uimanager.uimanager_sav.AEONskin = AskinChooser.currentValue
	uimanager.uimanager_sav.CYBRANskin = CskinChooser.currentValue
	uimanager.uimanager_sav.UEFskin = UskinChooser.currentValue
	Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
	
	selectLayout(skinChooser.currentValue, layoutList.currentValue, true)
end

function CreateLayoutSelectButton(skinData)
	local layoutSelect = Bitmap(basicwindow)
	local image = pathuiimg..'/layoutpreview_user.dds'
	local Askin = skinData[4]
	local Cskin = skinData[5]
	local Uskin = skinData[6]
	local skintitle = skinData[1]
	local overGlow = Bitmap(layoutSelect)
	if not skinData[3] then
		if skinfile[skinData[1]].layouts[skinData[2]].preview then
			image = skinfile[skinData[1]].path .. skinfile[skinData[1]].layouts[skinData[2]].preview
		elseif skinfile[skinData[1]].layouts.UseDefaults.preview then
			image = skinfile[skinData[1]].path .. skinfile[skinData[1]].layouts.UseDefaults.preview
		end
		if skinfile[skinData[1]].layouts[skinData[2]].factionskins then
			Askin = skinfile[skinData[1]].layouts[skinData[2]].factionskins[1]
			Cskin = skinfile[skinData[1]].layouts[skinData[2]].factionskins[2]
			Uskin = skinfile[skinData[1]].layouts[skinData[2]].factionskins[3]
			skintitle = 'Auto Faction'
		end
	end

	layoutSelect:SetTexture(image)
	layoutSelect.Width:Set(202) layoutSelect.Height:Set(154)
	overGlow:SetTexture(pathuiimg.. '/blank.dds')
	LayoutHelpers.FillParent(overGlow, layoutSelect)
	layoutSelect.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			if skintitle == 'Auto Faction' then
				uimanager.uimanager_sav = Prefs.GetFromCurrentProfile("uimanager_sav")
				uimanager.uimanager_sav.AEONskin = Askin
				uimanager.uimanager_sav.CYBRANskin = Cskin
				uimanager.uimanager_sav.UEFskin = Uskin
				Prefs.SetToCurrentProfile("uimanager_sav", uimanager.uimanager_sav)
			end
			selectLayout(skinData[1], skinData[2])
		elseif event.Type == 'MouseEnter' then
			overGlow:SetTexture(pathuiimg.. '/layoutmanager/basic-select-glow.dds')
			import('/lua/maui/effecthelpers.lua').Pulse(overGlow,2,0.4,1)
		elseif event.Type == 'MouseExit' then
			overGlow:SetTexture(pathuiimg.. '/blank.dds')
		end
	end
	
	local title = UIUtil.CreateText(layoutSelect, skintitle..': '..skinData[2], 12)
	title:SetColor('white')
	LayoutHelpers.AtLeftTopIn(title,  layoutSelect,2,2)
	title.Depth:Set(layoutSelect.Depth()+11)
	title:SetDropShadow(true)
	
	return layoutSelect
end

function CreateBasicWindow(parent)
	local location = {Top = GetFrame(0).Bottom()/2 - 200, Left = GetFrame(0).Right()/2  - 250, Bottom = GetFrame(0).Bottom()/2 + 173, Right = GetFrame(0).Right()/2 + 225}
	basicwindow = Window(parent, 'Layout Manager', nil, nil, false, true, false, 'layoutmanagerBasic', location)
	basicwindow.Depth:Set(1500)
	basicwindow.OnClose = function(self)
		basicwindow:Destroy()
		basicwindow = false
	end
	AdvancedBtn = UIUtil.CreateButtonStd( basicwindow, '/widgets/small', 'Advanced', 12, 0)
	AdvancedBtn.OnClick = function(self)
		basicwindow:Destroy()
		basicwindow = false
		init = true
		CreateAdvancedWindow(windowparent)
		init = false
	end
	LayoutHelpers.AtLeftTopIn(AdvancedBtn, basicwindow, 155, -3)
	AdvancedBtn.Depth:Set(basicwindow.Depth() + 10)
	layoutGrid = Grid(basicwindow, 210, 162)
	layoutGrid.Left:Set(function() return basicwindow.Left() + 12 end)
	layoutGrid.Top:Set(function() return basicwindow.Top() + 32 end)
	layoutGrid.Right:Set(function() return basicwindow.Right() - 43 end)
	layoutGrid.Bottom:Set(function() return basicwindow.Bottom() - 15 end)
	layoutGrid.Depth:Set(basicwindow.Depth() + 10)
	
	layoutGrid:DeleteAndDestroyAll(true)
	layoutGrid:AppendCols(2, true)
	local index = 1
	local basicLayouts = Prefs.GetFromCurrentProfile("uimanager_sav")["basicLayouts"]
	for i, skinData in basicLayouts do
		if math.mod(index, 2) == 1 then
			layoutGrid:AppendRows(1, true)
		end
		local button = CreateLayoutSelectButton(skinData)
		local row = math.ceil(index/2)
		local col = math.mod(index - 1, 2) + 1
		layoutGrid:SetItem(button, col, row, true)
		index = index + 1
	end
	
	layoutGrid:EndBatch()
	UIUtil.CreateVertScrollbarFor(layoutGrid)

	basicwindow.HandleEvent = function(self, event)
		if event.Type=="WheelRotation" then
			if event.WheelRotation > 0 then
				layoutGrid:ScrollLines("Vert", -1)
			else
				layoutGrid:ScrollLines("Vert", 1)
			end
		end
	end
end

function CreateAdvancedWindow(parent)
	window = Bitmap(parent, UIUtil.SkinnableFile(pathuiimg..'/layoutmanager/layoutmanager.dds'))
	window.Depth:Set(1500)
	LayoutHelpers.AtCenterIn(window, parent)
	local title= CreateText_sca(window, "layoutmanager_title" , 16, UIUtil.titleFont)
	LayoutHelpers.AtLeftTopIn(title, window, 70,20)
	title:SetColor(UIUtil.dialogColumnColor)

	okBtn = uimanager.Make_Button_generic(window)
	okBtn.OnQuit = function()
		ApplySettings()
		window:Destroy()
		window = false
	end
	
	cancelBtn = uimanager.Make_Button_generic(window, 'cancel', false, 147)
	cancelBtn.OnQuit = function()
		window:Destroy()
		window = false
	end
	
	applyBtn = uimanager.Make_Button_generic(window, 'apply', false, 252)
	applyBtn.OnQuit = function()
		ApplySettings()
	end
	
	deleteBtn = uimanager.Make_Button_generic(window, 'delete', false, 36, 76, modcontrols.trad("LM_delete"))
	deleteBtn.OnQuit = function()
		for i, layoutData in defaultLayoutTable do
			if layoutData[2] == layoutList.currentValue then
					UIUtil.QuickDialog(window, 'WARNING: Cannot delete a default layout',  "<LOC _OK>", function() end, nil, nil, nil, nil, true, {escapeButton = 1, enterButton = 1})
				return
			end
		end
		UIUtil.QuickDialog(window, 'Are you sure you want to delete layout "' ..layoutList.currentValue ..'"?',  "<LOC _OK>", function() Delete() end, "<LOC _Cancel>", function() end, nil, nil, true, {escapeButton = 2, enterButton = 1})
	end

	saveAsBtn = uimanager.Make_Button_generic(window, 'saveas', false, 36, 116, modcontrols.trad("LM_saveas"))
	saveAsBtn.OnQuit = function()
		SaveAs()
	end

	saveBtn = uimanager.Make_Button_generic(window, 'save', false, 36, 156, modcontrols.trad("LM_save"))
	saveBtn.OnQuit = function()
		for i, layoutData in defaultLayoutTable do
			if layoutData[2] == layoutList.currentValue then
				UIUtil.QuickDialog(window, 'WARNING: Cannot save over a default layout',  "<LOC _OK>", function() end, nil, nil, nil, nil, true, {escapeButton = 1, enterButton = 1})
				return
			end
		end
		UIUtil.QuickDialog(window, 'Are you sure you want to overwrite layout "' ..layoutList.currentValue ..'"?',  "<LOC _OK>", function() Save() end, "<LOC _Cancel>", function() end, nil, nil, true, {escapeButton = 2, enterButton = 1})
	end


	CreateSkinGroup(window, 40, 52)
	CreateLayoutChooser(window, 420, 52)
	CreateBasicManagerGroup(window, 40, 300)
	Make_CheckBox(window,32,455,"skinDebugWindow",function() if not uimanager.win then uimanager.Make_skin_info() end end,function() if uimanager.win then uimanager.win:Destroy() uimanager.win = false end end,nil,true,("skinDebugger"),false,false,false,false,false,("skinDebuggerHelp"))

	
	BasicBtn = UIUtil.CreateButtonStd( window, '/widgets/small', 'Basic', 12, 0)
	LayoutHelpers.AtLeftTopIn(BasicBtn, window, 86, 492)
	BasicBtn.OnClick = function(self)
		window:Destroy()
		window = false
		init = true
		CreateBasicWindow(windowparent)
		init = false
	end
end

function ToggleWindow(state)
	windowparent = import('/lua/ui/game/gamemain.lua').GetGameParent()
	if window then
		window:Destroy()
		window = false
	elseif basicwindow then
		basicwindow:Destroy()
		basicwindow = false
	else
		init = true
		Init()
		CreateBasicWindow(windowparent)
		init = false
	end
end