--*****************************************************************************
--* File: mods/uimod/sca_utils/uimanager.lua
--* Author: Saya and Cleopatre
--* Summary: New Controls for the UI mod
--* construct the config pannel
--*
--* Copyright  2007 Gas Powered Games, Inc.  All rights reserved. 
--*****************************************************************************

local debug_sca = false -- debugging mode, change the value here and in gamemain.lua
local pathui = '/mods/uimod/hook'
local pathuiimg='/mods/uimod/textures'
local sca_utils = '/mods/uimod/sca_utils/'
local uimod= import(sca_utils .. 'uimod.lua')
local gamemain = import('/lua/ui/game/gamemain.lua')
local remap = import(sca_utils .. 'remap.lua')
local UIManager = import(sca_utils .. 'uimanager.lua')

local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Group = import('/lua/maui/group.lua').Group
local Dragger = import('/lua/maui/dragger.lua').Dragger
local Slider = import('/lua/maui/slider.lua').Slider
local Tooltip = import('/lua/ui/game/tooltip.lua')
local Prefs = import('/lua/user/prefs.lua')
local Window = import('/lua/maui/window.lua').Window
local IntegerSlider = import('/lua/maui/slider.lua').IntegerSlider

ConfigCheckTable = {}
ConfigSliderTable = {}
TitleTable = {}

--makes a selection list given a table and a position
--parent is the list parent
--left and top are the list coordinates relatively to the parent
--paramTab is the table you want to make a list of
--length is the number of elements you can see at a time in the list 
--name is the name that will appear for the listBox
--file is a custom path to your itemSelector texture
function ListBox(parent,left,top,width,style_arrow,nbr_car,bline_number,paramTab,length,name,file,fct_on_over_enter,fct_on_over_exit, minimizable)
	--local tab used to remove or insert items in the list
	local itemTab = paramTab
	local top_reel=function() return top+parent.Top() end
	local left_reel=function() return left+parent.Left() end
	local slide=false
	local slideBG=false
	local slide_refresh=function() end
	local upArrow = false
	local downArrow = false
	local lineheight = 26
	local linecolour = '39414a'
	local altlinecolour = '333b45'
	local overcolour = 'c6c6c6'
	local selectedcolour = 'efefef'
	local titlecolour = '6b8294'
	local minimizedBtn = false
	local mintxt = false
	--list group
	local listBox = Group(parent)
	listBox.Depth:Set(function() return parent.Depth()+1 end)
	LayoutHelpers.AtLeftTopIn(listBox, parent, left, top)
	listBox.Width:Set(50)
	listBox.Height:Set(50)
	listBox.Depth:Set(parent.Depth()+20)
	listBox:DisableHitTest()
	
	--background table
	local bg = {}
	--text table
	local txt = {}
	--value table
	local values = {}
	
	local listHidden = true
	
	--p is the current position in the listBox
	listBox.p = 1
	--i is the current index in the listBox
	listBox.i = 1
	--r is the real index in the listBox
	listBox.r = 1
	
	--useful values
	listBox.endIndex = 0	
	listBox.currentValue = 0
	--item selector
	if not(file==false or file==nil) then 
		local itemSelector = Bitmap(listBox)
		itemSelector:SetTexture(UIUtil.UIFile(file))
		LayoutHelpers.AtLeftTopIn(itemSelector,listBox,left, top)
		itemSelector.Depth:Set(parent.Depth()+20) 
		if name and name!='' then
			local txt1= UIUtil.CreateText(itemSelector, name , 14, UIUtil.titleFont)
			LayoutHelpers.AtLeftTopIn(txt1, itemSelector, 8, 8)
			txt1:SetColor("white")
		end
		--item selector handleEvent
		itemSelector.HandleEvent = function(self, event)	
			if (event.Type == 'ButtonPress') then
				if not listHidden then
					listBox.Close()	
				else
					listBox.Open()	
				end
			elseif (file == pathuiimg..'/bouton_reset_layout.dds') then
				if (event.Type == 'MouseExit') then
					itemSelector:SetTexture(UIUtil.UIFile(pathuiimg..'/bouton_reset_layout.dds'))
				elseif (event.Type == 'MouseEnter') then
					itemSelector:SetTexture(UIUtil.UIFile(pathuiimg..'/bouton_reset_layout_actif.dds'))
				end
			end
			
		end	
	end
				


	if style_arrow=="normal" then 
		upArrow = Bitmap(listBox)
		downArrow = Bitmap(listBox)
		upArrow:SetTexture(UIUtil.UIFile(pathuiimg..'/fleche_haut.dds'))	
		upArrow.Top:Set(top_reel() + 1)
		upArrow.Depth:Set(parent.Depth()+20)
		upArrow.Width:Set(width)	
		downArrow:SetTexture(UIUtil.UIFile(pathuiimg..'/fleche_bas.dds'))
		downArrow.Depth:Set(parent.Depth()+20) 
		downArrow.Width:Set(width)
		upArrow.Left:Set(function() return left_reel()+width end)
		downArrow.Left:Set(function() return left_reel()+width end)
	elseif style_arrow=="noArrow" then
		--upArrow.Top:Set(top_reel()-upArrow.Height())
	elseif style_arrow=="Arrow" then
		upArrow = Bitmap(listBox)
		downArrow = Bitmap(listBox)	
		upArrow:SetTexture(UIUtil.UIFile(pathuiimg..'/fleche_haut.dds'))	
		upArrow.Top:Set(top_reel())
		upArrow.Depth:Set(parent.Depth()+20)
		upArrow.Width:Set(width)	
		downArrow:SetTexture(UIUtil.UIFile(pathuiimg..'/fleche_bas.dds'))
		downArrow.Depth:Set(parent.Depth()+20) 
		downArrow.Width:Set(width)
		upArrow.Left:Set(left_reel)
		downArrow.Left:Set(left_reel)
	end
	
	--refresh current selected item texture
	local updateCurrentIndex = function()
		if (listBox.endIndex >= 1) then
			if listBox.r >= listBox.p and listBox.r <= (listBox.p + length - 1) then
				bg[listBox.r-listBox.p+1]:SetSolidColor(selectedcolour)
				if txt[listBox.r-listBox.p+1] != nil then
					txt[listBox.r-listBox.p+1]:SetColor("black")
				end
			end
		end
	end

	--returns true if the item is a category, else returns false
	local category = function(i)
		local isCategory = false
		if	values[i] != nil and string.find(values[i],'<SCA_CATEGORY>') != nil then
			isCategory = true
		end
		return isCategory
	end
	
	--function to edit to determine specific on select actions
	listBox.OnSelect = function()
	end
	--creates the background of the item list
	local createBgTable = function()
		--create borders
		local bordertl = Bitmap(listBox)
		bordertl:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-tl.dds'))
		bordertl.Top:Set(top_reel)
		bordertl.Left:Set(left_reel)
		local bordertr = Bitmap(listBox)
		bordertr:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-tr.dds'))
		bordertr.Top:Set(top_reel)
		bordertr.Right:Set(left_reel() + width)
		local bordertop = Bitmap(listBox)
		bordertop:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-horiz.dds'))
		bordertop.Top:Set(top_reel)
		bordertop.Left:Set(bordertl.Right)
		bordertop.Right:Set(bordertr.Left)
		local borderbottom = Bitmap(listBox)
		borderbottom:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-horiz.dds'))
		borderbottom.Top:Set(top_reel() + bordertop.Height() + length*lineheight - 3)
		borderbottom.Left:Set(bordertl.Right)
		borderbottom.Right:Set(bordertr.Left)
		local borderbl = Bitmap(listBox)
		borderbl:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-bl.dds'))
		borderbl.Top:Set(borderbottom.Top)
		borderbl.Left:Set(left_reel)
		local borderbr = Bitmap(listBox)
		borderbr:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-br.dds'))
		borderbr.Top:Set(borderbottom.Top)
		borderbr.Right:Set(left_reel() + width)
		local borderleft = Bitmap(listBox)
		borderleft:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-vertical.dds'))
		borderleft.Top:Set(bordertl.Bottom)
		borderleft.Left:Set(left_reel)
		borderleft.Bottom:Set(borderbl.Top)
		local borderright = Bitmap(listBox)
		borderright:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-vertical.dds'))
		borderright.Top:Set(bordertl.Bottom)
		borderright.Right:Set(left_reel() + width)
		borderright.Bottom:Set(borderbl.Top)
		
		local k = 1;
		while k <= length do
			--save button index
			local index = k 
			--create item background
			bg[k] = Bitmap(listBox)	
			if k == 1 then
				bg[k].Top:Set(bordertop.Bottom() - 2)
			else
				bg[k].Top:Set(bg[k-1].Bottom)
			end
			bg[k].Height:Set(lineheight)
			bg[k].Left:Set(borderleft.Right() - 1)
			bg[k].Width:Set(borderright.Left() - borderleft.Right() + 3)
			bg[k].Depth:Set(listBox.Depth()) 		
					
			
			--create item text
			--if bline_number then buttonText = k .. ": " .. i end
			txt[k] = UIUtil.CreateText(bg[k], "" , 13, UIUtil.fixedFont)
			txt[k]:SetColor("white")
			LayoutHelpers.AtLeftTopIn(txt[k], bg[k], 13, 4)
			txt[k].Depth:Set(bg[k].Depth()+20)			
			local text = txt[index]:GetText()
			txt[k].bmp = Bitmap(listBox)
			txt[k].bmp.Top:Set(txt[k].Top())
			txt[k].bmp.Left:Set(txt[k].Left()+10)

			--make item handle event
			bg[k].HandleEvent = function(self, event)
			if values[listBox.p + index - 1] != nil then
				if (event.Type == 'ButtonPress') and category(listBox.p + index - 1) == false then
					--remove old selected marker
					local oldselected = listBox.r - listBox.p + 1
					if oldselected > 0 and oldselected < length + 1 then
						txt[oldselected]:SetColor("white")
						local showpos = listBox.p+(oldselected-1)
						if math.mod(showpos, 2) == 1 then
							bg[oldselected]:SetSolidColor(linecolour)
						else
							bg[oldselected]:SetSolidColor(altlinecolour)
						end
					end

					--change position of listBox
					listBox.i = index	
					listBox.r = listBox.p + listBox.i - 1
					listBox.currentValue  = values[listBox.p + listBox.i - 1]
					
					--set new textures for selected item
					bg[index]:SetSolidColor(selectedcolour)
					txt[index]:SetColor("black")
					
					--do specific action
					listBox.OnSelect()
					
					if minimizable then
						listBox:Hide()
						minimizedBtn:Show()
						mintxt:SetText(listBox.currentValue)
					end
				--mouse over	
				elseif (event.Type == 'MouseEnter') and values[listBox.p + index - 1] != nil and category(listBox.p + index - 1) == false then
					if fct_on_over_enter != nil then fct_on_over_enter(values[listBox.p + index - 1],index)  end
					if (listBox.p + index - 1) != (listBox.r) then
						bg[index]:SetSolidColor(overcolour)
						txt[index]:SetColor("black")
						
						if values[listBox.p + index - 1] != nil then
						--LOG("valeur over: " .. values[listBox.p + index - 1])
						end
						
					end
				elseif (event.Type == 'MouseExit') and category(listBox.p + index - 1) == false then
					if fct_on_over_exit != nil then fct_on_over_exit(text,index) end			
					if (listBox.p + index - 1) != (listBox.r) then
						txt[index]:SetColor("white")
						local showpos = listBox.p+(index-1)
						if math.mod(showpos, 2) == 1 then
							bg[index]:SetSolidColor(linecolour)
						else
							bg[index]:SetSolidColor(altlinecolour)
						end
					end
				end
			end
		end	
		
		k = k + 1 
		end
	end
	
	--function to show the items within the bounds
	local affiche = function()
		for k,v in bg do
			if bg[k] ~= nil then
				if listBox.endIndex <= length then
					listBox.p = 1
				end				
				
				local showpos = listBox.p+(k-1)
				if math.mod(showpos, 2) == 1 then
					bg[k]:SetSolidColor(linecolour)
				else
					bg[k]:SetSolidColor(altlinecolour)
				end

				if values != nil and txt[k] != nil then
				txt[k]:SetColor("white")
					if values[showpos] != nil then
						local text = string.gsub( values[showpos], "^<(.*)>(.*)$", "%2" )	
						txt[k]:SetText(string.sub(text,0,nbr_car))
						local bmp=string.gsub( values[showpos], "^<BMP_(.*)>(.*)$", "%1" )
						if bmp!=nil and  bmp!="" and bmp!=false and bmp!=values[showpos] then 
							--LOG("----->",k)
							txt[k].bmp:SetTexture(UIUtil.UIFile(pathuiimg..'/'..bmp))
							--txt[k].bmp.Height:Set(txt[k].Height())
							--LOG(txt[k].Height())
							--txt[k].bmp.Width:Set(50) --txt[k].Width())
							txt[k].bmp.Top:Set(txt[k].Top())
							txt[k].bmp.Left:Set(function() return left_reel() + 23 end)
							local wd=txt[k].bmp.Width()
							txt[k].Left:Set(function() return left_reel() +33 +wd end)
						else
							txt[k].bmp:SetTexture(UIUtil.UIFile(pathuiimg..'/empty_sca.dds'))
							txt[k].Left:Set(function() return left_reel() + 13 end)
						end
						
						--if the item is a category
						if category(showpos) == true then
							if text != "" then
								bg[k]:SetSolidColor(titlecolour)
							end
							txt[k]:SetColor("Yellow")
							txt[k]:SetText(text)
							LayoutHelpers.AtHorizontalCenterIn(txt[k], bg[k])
						end
					else
						txt[k]:SetText("")
						txt[k].bmp:SetTexture(UIUtil.UIFile(pathuiimg..'/empty_sca.dds'))
						txt[k].Left:Set(function() return left_reel() + 13 end)
					end	
				end
			end				
		end
		updateCurrentIndex()
	end
	
	

	
	--adds an element to the list
	listBox.addElement = function(v)
		listBox.endIndex = listBox.endIndex + 1
		table.insert(values,v)
	end
	
	--make background
	createBgTable()
	
	--init elements
	for k,v in itemTab do 
		--append item
		listBox.addElement(v)	
	end
		
	--insert element at the end of the list
	listBox.insertElement = function(e)
		--add element at end of list
		listBox.addElement(e)	
		--refresh
		listBox.setPosition(listBox.p+1)		
		slide_refresh()
	end
	
	listBox.slide_refresh= function(e)
		slide_refresh()
	end
	
	--remove element at a given index
	listBox.removeElement = function(k)
		if values[k] != nil then

			--LOG( "index/value: " .. k .. values[k] )
			table.remove(values,k)

			listBox.endIndex = listBox.endIndex - 1	 
			affiche()	
			slide_refresh()
			listBox.setPosition(1)
		end		
	end
	
	--remove a given element
	listBox.removeItem = function(item)
		for i, v in values do
			if v == item then
				--LOG( "index/value: " .. k .. values[k] )
				table.remove(values, i)

				listBox.endIndex = listBox.endIndex - 1	 
				listBox.r = math.min(i, listBox.endIndex)
				listBox.currentValue  = values[math.min(i, listBox.endIndex)]
				affiche()	
				slide_refresh()
				listBox.setPosition(1)
			end
		end		
	end
	
	--clear listBox, removes all values
	listBox.clear = function()
		values = {}
	    listBox.endIndex = 0
		slide_refresh()
		listBox.setPosition(1)
	end
	
	--look for an element in the list and return the index
	listBox.lookFor = function(e)
		local result = false
		local k = 0
		for i,v in values do
			if values[k] == e then
				 result = k
			end
			k = k + 1
		end
		if result == false then
			result = "element not found"
		end
		return result
	end
	

   if style_arrow=="normal" or style_arrow=="Arrow" then	
	--up arrow handleEvent
	upArrow.HandleEvent = function(self, event)	
		if (event.Type == 'ButtonPress') then
			if listBox.p > 1 then
				local newPosition = listBox.p - 1
				listBox.setPosition(newPosition)
			end
		end
	end	

	--down arrow handleEvent
	downArrow.HandleEvent = function(self, event)	
		if (event.Type == 'ButtonPress') then
			if listBox.p < listBox.endIndex - length + 1 then
				local newPosition = listBox.p + 1
				listBox.setPosition(newPosition)
			end
		end
	end	
   end
   
	--listbox handleEvent
	listBox.HandleEvent = function(self, event)	
		--mousewheel scrolling
		if event.Type=="WheelRotation" then
			if event.WheelRotation <= 0 then
				if listBox.p < listBox.endIndex - length + 1 then
					local newPosition = listBox.p + 1
					listBox.setPosition(newPosition)
				end
			else
				if listBox.p > 1 then
					local newPosition = listBox.p - 1
					listBox.setPosition(newPosition)
				end
			end
		end
	end
   
	--makes all components under the list box showable and shows the list box
	listBox.Open = function()
		listBox.Bottom:Set(function() return downArrow.Top() + downArrow.BitmapHeight() end)
		listHidden = false
	end
	
	--makes all components under the list box unshowable and hides the list box
	listBox.Close = function()
		listBox:Hide()
		listHidden = true
	end
	
	--makes all components under the list box showable
	listBox.initOpen = function()
		listBox.Open()
	end
	
	--sets the position of the list box, the position is the index of the first viewed item
	listBox.setPosition = function(position)
		if position < 1 then position = 1 end
		if position >= listBox.endIndex - length + 1 then position = listBox.endIndex - length + 1 end
		listBox.p = position		
		slide_refresh()
		affiche()		
	end
	
	listBox.setPosition2 = function(position)
		if position < 1 then position = 1 end
		if position >= listBox.endIndex - length + 1 then position = listBox.endIndex - length + 1 end
		listBox.p = position		
		--slide_refresh()
		--affiche()		
	end
	--returns the real index of the listBox, the real index is the position of the selected item
	listBox.getIndex = function()
		local listIndex = listBox.p+listBox.i-1
		return listIndex
	end
	
	--returns the value of the selected item
	listBox.getValue = function(k)
		if k ==  nil then
			return values[listBox.p+listBox.i-1]
		else
			return values[k]
		end
	end
	
	--return a table of values
	listBox.getValues = function()
		return values
	end
	
	--sets the listbox selection to a value
	listBox.setValue = function(value)
		for i, v in values do
			if value == v then
				listBox.r = i
				listBox.currentValue  = values[i]
				affiche()
				if minimizable then
					mintxt:SetText(listBox.currentValue)
				end
			end
		end
	end
	
	--return if the listBox is open or not (DOES NOT WORK, listHidden isnt set correctly)
	listBox.Opened = function() 
		return not listHidden
	end
	
	--refreshes the layout, if boolean is true it will show the list else it will hide the list
	listBox.Refresh = function(boolean)
		if boolean == true then
			listBox.setPosition(listBox.p)
			listBox:Show()
		else
			listBox:Hide()
		end
	end
	
	
	--makes a slider next to the listbox to control the listbox
	local Make_slide=function(parent)
		local x= left_reel() + width
		local ymin= upArrow.Bottom()
		local ymax=downArrow.Top()
		local width=50
		local height=30
		local size_el=1
		local size_slide=1
		
		local pos=listBox.p
		if pos==nil then pos=1 end
		local new_up=pos
		
		--LayoutHelpers.AtLeftTopIn(slide,parent,x,ymin)
		local slide = Group(parent)
		slide.Depth:Set(parent.Depth() + 10)
		slide.Left:Set(x)
		slide.Top:Set(ymin+pos*size_el)
		slide.Width:Set(26)
		
		local slide_t = Bitmap(slide)
		slide_t:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/scroll_slider_top.dds'))
		slide_t.Left:Set(slide.Left)
		slide_t.Top:Set(slide.Top)

		local slide_b = Bitmap(slide)
		slide_b:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/scroll_slider_bottom.dds'))
		slide_b.Left:Set(slide.Left)
		slide_b.Bottom:Set(slide.Bottom)
		
		local slide_m = Bitmap(slide)
		slide_m:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/scroll_slider.dds'))
		slide_m.Left:Set(slide.Left)
		slide_m.Top:Set(slide_t.Bottom)
		slide_m.Bottom:Set(slide_b.Top)
		
		local slideBG = Bitmap(parent)
		slideBG:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/scroll_slider_bg.dds'))
		slideBG.Depth:Set(parent.Depth() + 9)
		slideBG.Left:Set(x)
		slideBG.Top:Set(ymin)
		slideBG.Height:Set(ymax-ymin)
		slideBG.Width:Set(26)

		slideBG:SetAlpha(.7)
		
		listBox.slideGlow = Bitmap(slide)
		listBox.slideGlow:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/scroll_slider_glow.dds'))
		listBox.slideGlow.Left:Set(slide.Left)
		listBox.slideGlow.Top:Set(slide_t.Bottom)
		listBox.slideGlow.Bottom:Set(slide_b.Top)
		listBox.slideGlow:SetAlpha(0)

		--slider handle event
		slide.HandleEvent = function(self, event)
			if event.Type == 'ButtonPress' and size_el!=(ymax-ymin)  then
				local det_y=event.MouseY-slide.Top()
				local drag = Dragger()
				drag.OnMove = function(drag, x, y)
					if (y - det_y<ymin) then y=ymin + det_y end
					if (y - det_y>(ymax-slide.Height())) then y=ymax-slide.Height()+det_y end
					slide.Top:Set(y - det_y)
					
					if listBox.endIndex<=length  then new_up=1 else new_up=math.ceil(((y-det_y-ymin)*(listBox.endIndex-length))/(ymax-ymin-slide.Height()))+1 end
					listBox.setPosition(new_up)
					affiche()
				end
				dragging = false
				PostDragger(GetFrame(0),event.KeyCode,drag)
			else
				if size_el==(ymax-ymin) then 
					slide.Top:Set(function() return ymin end)
				end
			end

			--LOG("nbre max:",listBox.endIndex,"   size_el",size_el)
			if event.Type == 'MouseEnter' then
				listBox.slideGlow:SetAlpha(0.5)
			end
			if event.Type == 'MouseExit' then
				listBox.slideGlow:SetAlpha(0)
			end
		end
		
		slideBG.HandleEvent = function(self, event)
			if event.Type == 'ButtonPress' then
				if event.MouseY > ymin and event.MouseY < slide.Top() then
					if listBox.p > 1 then
						local newPosition = math.max(listBox.p - 5, 1)
						listBox.setPosition(newPosition)
					end
				elseif event.MouseY < ymax and event.MouseY > slide.Bottom() then
					if listBox.p < listBox.endIndex - length + 1 then
						local newPosition = math.min(listBox.p + 5, listBox.endIndex)
						listBox.setPosition(newPosition)
					end
				end
			end
		end
		
		listBox.Refreshslider=function()
		--init
		--set slide height
			if listBox.endIndex<=length then size_slide=ymax-ymin else size_slide=(ymax-ymin)*(length/listBox.endIndex) end
			local pos=listBox.p
			if pos==nil then pos=1 end
			if size_slide<10 then slide.Height:Set(10) else slide.Height:Set(size_slide) end
		--set slide position
			if listBox.endIndex<=length then size_el=ymax-ymin else size_el=(ymax-ymin - slide.Height())/(listBox.endIndex - length) end
			if size_el==(ymax-ymin) then
				slide.Top:Set(function() return ymin end)
			else 
				if ymin + (pos-1)*size_el>ymax-slide.Height() then
					slide.Top:Set(ymax-slide.Height()) 	
				else
					slide.Top:Set(function() return ymin + (pos-1)*size_el end)
				end
			end
		end
		listBox.Refreshslider()
		
		return slide,slideBG
	end
	
	if style_arrow=="normal" then 
		--set the down arrow position depending on the length

		slide_refresh = function()
			--if slide and slide!=nil then slide:Destroy() slideBG:Destroy() end
			downArrow.Bottom:Set(function() return top_reel() + (length)*lineheight + 6 end)
			if not slide then slide, slideBG=Make_slide(listBox) end
			--if slideBG then slideBG:Destroy() end
			--if slide then slide:Destroy() end
			
			listBox.Refreshslider()
			upArrow.Left:Set(slideBG.Left)
			downArrow.Left:Set(slideBG.Left)
			upArrow.Right:Set(slideBG.Right)
			downArrow.Right:Set(slideBG.Right)
		end
		slide_refresh()
		
	elseif style_arrow=="Arrow" then
		slide_refresh = function()
		end
		downArrow.Top:Set(function() return top_reel() + upArrow.Height() + (length)*lineheight end)
	elseif style_arrow=="noArrow" then
	end
	
	--function to select an item in the listbox
	--index is the index of the item in the list
	--if exe is true the OnSelect() function of the list will be called
	--if boolean is true the list will be shown/refreshed else it will not change its ishidden state
	listBox.select = function(index,exe,boolean)
		local showList = true
		if boolean != nil then
			showList = boolean
		end
		
		local run = true
		if exe != nil then
			run = exe
		end
		
		listBox.r = index
		listBox.i = 2
		listBox.currentValue  = values[listBox.r]
		
		if showList == true then
		listBox.setPosition(listBox.r-3)
		end
		
		if run == true then
			OnSelect()
		end
		
	end
	
	--init
	listBox.i = 1
	if category(1) == true then
		listBox.select(2,false)
	else
		listBox.select(1,false)
	end
	
	if minimizable then
		minimizedBtn = Bitmap(parent)
		
		local bordertl = Bitmap(minimizedBtn)
		bordertl:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-tl.dds'))
		bordertl.Top:Set(top_reel)
		bordertl.Left:Set(left_reel)
		local bordertr = Bitmap(minimizedBtn)
		bordertr:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-tr.dds'))
		bordertr.Top:Set(top_reel)
		bordertr.Right:Set(left_reel() + width)
		local bordertop = Bitmap(minimizedBtn)
		bordertop:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-horiz.dds'))
		bordertop.Top:Set(top_reel)
		bordertop.Left:Set(bordertl.Right)
		bordertop.Right:Set(bordertr.Left)
		local borderbottom = Bitmap(minimizedBtn)
		borderbottom:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-horiz.dds'))
		borderbottom.Top:Set(top_reel() + bordertop.Height() + lineheight - 7)
		borderbottom.Left:Set(bordertl.Right)
		borderbottom.Right:Set(bordertr.Left)
		local borderbl = Bitmap(minimizedBtn)
		borderbl:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-bl.dds'))
		borderbl.Top:Set(borderbottom.Top)
		borderbl.Left:Set(left_reel)
		local borderbr = Bitmap(minimizedBtn)
		borderbr:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-br.dds'))
		borderbr.Top:Set(borderbottom.Top)
		borderbr.Right:Set(left_reel() + width)
		local borderleft = Bitmap(minimizedBtn)
		borderleft:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-vertical.dds'))
		borderleft.Top:Set(bordertl.Bottom)
		borderleft.Left:Set(left_reel)
		borderleft.Bottom:Set(borderbl.Top)
		local borderright = Bitmap(minimizedBtn)
		borderright:SetTexture(UIUtil.UIFile(pathuiimg..'/listbox-border-vertical.dds'))
		borderright.Top:Set(bordertl.Bottom)
		borderright.Right:Set(left_reel() + width)
		borderright.Bottom:Set(borderbl.Top)

		minimizedBtn.Top:Set(bordertop.Bottom() - 2)
		minimizedBtn.Left:Set(borderleft.Right() - 1)
		minimizedBtn.Height:Set(borderbottom.Top() - bordertop.Bottom() + 3)
		minimizedBtn.Width:Set(borderright.Left() - borderleft.Right() + 3)
		minimizedBtn.Depth:Set(parent.Depth() + 10)
		minimizedBtn:SetSolidColor(linecolour)
		minimizedBtn.HandleEvent = function(self, event)
			if event.Type == 'ButtonPress' then
				minimizedBtn:Hide()
				listBox:Show()
			end
		end
		
		mintxt = UIUtil.CreateText(minimizedBtn, "" , 13, UIUtil.fixedFont)
		mintxt:SetColor("white")
		LayoutHelpers.AtLeftTopIn(mintxt, minimizedBtn, 8, 3)
		mintxt:SetText(listBox.currentValue)
		
		local arrow = Bitmap(minimizedBtn)
		arrow:SetTexture(UIUtil.UIFile(pathuiimg..'/fleche_bas.dds'))
		arrow.Left:Set(left_reel()+width)
		arrow.Top:Set(top_reel()+2)
		arrow.HandleEvent = minimizedBtn.HandleEvent
		listBox:Hide()
		listBox.minimizedBtn = minimizedBtn
	end
	
	return listBox
end


-- create a checkbox with his value  automaticly saved in import(sca_utils .. 'uimanager.lua').uimanager_sav
-- give your parent, his place
-- name_in_game_pref: his name in import(sca_utils .. 'uimanager.lua').uimanager_sav -->WARNING: you must creat the EXACT same entry in import(sca_utils .. 'uimanager.lua').uimanager_sav on the top of file, this will be also the reset value
-- brefresh= must we refresh the layout after the change ?
-- bmake_init= auto intialise ?
-- when the reset button is click will automaticly reset all the checkBox and slider create by these function/ coudl call the reset with reset()
-- for the labeling(only if label_txt used): if put nothing in size, style, and xrelatif/y if you want it by default
function Make_CheckBox(parent,x,y,name_in_game_pref,fct_ON,fct_OFF,brefresh,bmake_init,label_txt,xrelatif,yrelatif,size_txt,style,color,help_tips)
	if not(((parent==nil) or (parent==false)) or (Get_opt(name_in_game_pref)==nil)) then
		local check_bx
		local label
		check_bx = UIUtil.CreateCheckboxStd(parent, '/dialogs/check-box_btn/radio')
		LayoutHelpers.AtLeftTopIn(check_bx, parent, x,y)
		check_bx.OnCheck = function(self, checked)
			if not UIManager.constructing then
				if import(sca_utils .. 'uimanager.lua').construction_done then uimod.saveModValues("default") end
				if checked then
					if debug_sca then print(name_in_game_pref .. " - check True") end
					import(sca_utils .. 'uimanager.lua').uimanager_sav[name_in_game_pref] = 1
					if import(sca_utils .. 'uimanager.lua').construction_done then 
						Prefs.SetToCurrentProfile("uimanager_sav", import(sca_utils .. 'uimanager.lua').uimanager_sav)
						Prefs.SavePreferences()
					end
					if fct_ON != nil then fct_ON(checked) end
				else
					if debug_sca then print(name_in_game_pref .. " - check False") end
					import(sca_utils .. 'uimanager.lua').uimanager_sav[name_in_game_pref] = 0
					if import(sca_utils .. 'uimanager.lua').construction_done then 
						Prefs.SetToCurrentProfile("uimanager_sav", import(sca_utils .. 'uimanager.lua').uimanager_sav)
						Prefs.SavePreferences()
					end
					if fct_OFF != nil then fct_OFF(checked) end
				end

				if brefresh and import(sca_utils .. 'uimanager.lua').bRefresh_allow and import(sca_utils .. 'uimanager.lua').construction_done then uimod.RefreshLayout() end
			end
		end
		if bmake_init then
			init_checkBox(name_in_game_pref,check_bx)
		end
	
		if (label_txt!=nil) and (label_txt!=false) then
			if (style==false) or (style==nil) then style= UIUtil.titleFont end
			if (size_txt == false) or (size_txt ==nil) then size_txt=15 end
			if (xrelatif == false) or (xrelatif==nil) then xrelatif=35 end
			if (yrelatif == false) or (yrelatif==nil) then yrelatif=5 end
			label = UIUtil.CreateText(check_bx, trad(label_txt), size_txt, style)
			LayoutHelpers.AtLeftTopIn(label, check_bx, xrelatif,yrelatif)
			if (color!=nil) and (color!=false) then label:SetColor(color) else  label:SetColor(UIUtil.dialogCaptionColor) end
		end
	
		tooltips(check_bx,help_tips)

		--create_label(check_bx,label_txt, size_txt, style,xrelatif,yrelatif,color,help_tips)
		if label_txt!=nil and label!=nil then
			TitleTable[label_txt] = label
			label:SetText(trad(label_txt))
		end
		if help_tips!=nil then
			tooltips(check_bx,help_tips)
		end
	
		ConfigCheckTable[name_in_game_pref] = check_bx
	
		check_bx.OnDestroy = function()
			ConfigCheckTable[name_in_game_pref] = nil
			if label_txt != nil then
				TitleTable[label_txt] = nil
			end
		end
		
		check_bx.Depth:Set(parent.Depth() + 10)
		return check_bx,label
	else
		WARN("WARNING UIMOD: Make_checkbox call with incorrect parameter. (parent nil or false or pb in value in Gamepref="..repr(Get_opt(name_in_game_pref))..")")
	end	
end


-- create a slider with his value  automaticly saved in import(sca_utils .. 'uimanager.lua').uimanager_sav
-- almost same as Make_checkbox
-- when the reset button is click will automaticly reset all the checkBox and slider create by these function
-- Warning: you must creat EXACLY the same entry in import(sca_utils .. 'uimanager.lua').uimanager_sav on the top of file, this will be the reset value this entry MUST between 0 and 1 (in fact >debut and <fin)
function Make_Slider(parent,x,y,name_in_game_pref,debut,fin,fct_call,bmake_init,label_txt,xrelatif,yrelatif,size_txt,style,color,help_tips)
	--transparency slider
   if not((parent ==nil or parent ==false or debut<0 or debut >1 or fin<0 or fin>1 or fin<=debut or Get_opt(name_in_game_pref)==nil)) then
	local slider
	local label
	slider = Slider(parent, false, debut,fin, UIUtil.SkinnableFile('/slider02/slider_btn_up.dds'), UIUtil.SkinnableFile('/slider02/slider_btn_over.dds'), UIUtil.SkinnableFile('/slider02/slider_btn_down.dds'), UIUtil.SkinnableFile('/slider02/slider-back_bmp.dds'))  
	LayoutHelpers.AtLeftTopIn(slider, parent, x,y) --72, 410)
	slider:SetValue(Get_opt(name_in_game_pref))
	
	local tempo =1
	slider.OnValueSet = function(self, newValue)
		if not UIManager.constructing then
			if debug_sca then print(name_in_game_pref .. "= "..newValue.." - slider action(onValueSet)") end
			UIManager.uimanager_sav[name_in_game_pref] = newValue  
			if UIManager.import(sca_utils .. 'uimanager.lua').construction_done then
				Prefs.SetToCurrentProfile("uimanager_sav", UIManager.import(sca_utils .. 'uimanager.lua').uimanager_sav)
				Prefs.SavePreferences()
			end
			if (fct_call!=nil) and ((newValue!=nil) and ((newValue>=0) and (newValue<=1))) then fct_call(newValue) end
		end
	end
	slider.OnValueChanged = function(self, newValue)
		if debug_sca then print(name_in_game_pref .. "= "..newValue.." - slider action(onValueChanged)") end
		UIManager.uimanager_sav[name_in_game_pref] = newValue  
		if UIManager.import(sca_utils .. 'uimanager.lua').construction_done then
			Prefs.SetToCurrentProfile("uimanager_sav", UIManager.import(sca_utils .. 'uimanager.lua').uimanager_sav)
			Prefs.SavePreferences()
		end
		if (fct_call!=nil) and ((newValue!=nil) and ((newValue>=0) and (newValue<=1))) then fct_call(newValue) end
	end

	if bmake_init then
		init_slider(name_in_game_pref,slider)
	end
	
	if (label_txt!=nil) and (label_txt!=false) then
		if (style==false) or (style==nil) then style= UIUtil.titleFont end
		if (size_txt == false) or (size_txt ==nil) then size_txt=15 end
		if (xrelatif == false) or (xrelatif==nil) then xrelatif=40 end
		if (yrelatif == false) or (yrelatif==nil) then yrelatif=5 end
		label = UIUtil.CreateText(slider, trad(label_txt), size_txt, style)
		if ((xrelatif !=nil) and (xrelatif!=false)) and ((yrelatif !=nil) and (yrelatif!=false)) then 
			LayoutHelpers.AtLeftTopIn(label,slider, xrelatif,yrelatif)
		end
		if (color!=nil) and (color!=false) then label:SetColor(color) else  label:SetColor(UIUtil.dialogCaptionColor) end
	end

	tooltips(slider,help_tips)

	--create_label(check_bx,label_txt, size_txt, style,xrelatif,yrelatif,color,help_tips)
	local res2=refresh_lang
	if label_txt!=nil and label!=nil then 
		TitleTable[label_txt] = label
		label:SetText(trad(label_txt)) end 
	if help_tips!=nil then
		tooltips(slider,help_tips)
	end

	
	ConfigSliderTable[name_in_game_pref] = slider

	slider.OnDestroy = function()
		ConfigSliderTable[name_in_game_pref] = nil
		if label_txt != nil then
			TitleTable[label_txt] = nil
		end
	end
	
	return slider,label
  else
	WARN("WARNING UIMOD: Make_slider call with incorrect parameter.")
  end	
end


---------------  special functions

function Get_opt(name_in_game_pref)
	local tempo=Prefs.GetFromCurrentProfile("uimanager_sav")[name_in_game_pref]
	if tempo == nil then
		if not(debug_sca) then 
			WARN("WARNING UIMOD: "..name_in_game_pref.. " is a missing option in import(sca_utils .. 'uimanager.lua').uimanager_sav (call in Get_opt), please check, value put to 1.")
			tempo=1
		else
			WARN("WARNING UIMOD: "..name_in_game_pref.. " is a missing option in import(sca_utils .. 'uimanager.lua').uimanager_sav (call in Get_opt), please check, no value is given in debug mod.")
		end
	end
	return tempo
end

function init_checkBox(name,fct)
	if Get_opt(name) != nil then
		--LOG("init of "..name .. "= "..Get_opt(name)) 
		if (Get_opt(name)==1) then fct:SetCheck(true, false) else fct:SetCheck(false, false) end
	else -- is nil
		WARN("WARNING UIMOD: "..name.. " is a missing option in import(sca_utils .. 'uimanager.lua').uimanager_sav (call in init_checkBox)")
	end
end


function init_slider(name,fct)
	if Get_opt(name) != nil then
		if (Get_opt(name)>=0) and (Get_opt(name)<=1) then fct:SetValue(Get_opt(name)) end
	else -- is nil
		WARN("WARNING UIMOD: "..name.. " is a missing option in import(sca_utils .. 'uimanager.lua').uimanager_sav (call in init_slider)")
	end
end

function reset()
	local sav=false
	sav=UIManager.import(sca_utils .. 'uimanager.lua').construction_done
	UIManager.import(sca_utils .. 'uimanager.lua').construction_done=false
	--WARN("----------------- Reset ON")
	for key, box in ConfigCheckTable do
		init_checkBox(key, box)
	end
	for key, slider in ConfigSliderTable do
		init_slider(key, slider)
	end	
	UIManager.import(sca_utils .. 'uimanager.lua').construction_done=sav
	Prefs.SavePreferences()
	--WARN("----------------- Reset OFF")
end

function tooltips(parent,help_tips)
	if help_tips!=nil and help_tips!=false then
		local oldHandleEvent = parent.HandleEvent
		parent.HandleEvent = function(self, event)
			if event.Type == 'MouseEnter' then
				Tooltip.CreateMouseoverDisplay(self, trad(help_tips), .5) --, true)
			elseif event.Type == 'MouseExit' then
				Tooltip.DestroyMouseoverDisplay()
			end
			oldHandleEvent(self,event)
		end
	end
end	


function CreateText_sca(parent, traduction, size, style)
	local ret=UIUtil.CreateText(parent, trad(traduction), size, style)
	TitleTable[traduction] = ret
	ret.OnDestroy = function()
		TitleTable[traduction] = nil
	end
	return ret
end

function trad(id)
	return import(sca_utils .. 'dialog_txt.lua').trad(id)
end

function refresh_lang()
	for i, v in TitleTable do
	v:SetText(trad(i))
	end
end

function CreateConfigWindow()
    local win = Window(GetFrame(0), 'Configure Chat', nil, nil, nil, true, nil, 'chat_config')
    win.Depth:Set(GetFrame(0):GetTopmostDepth() + 1)
    Tooltip.AddButtonTooltip(win._closeBtn, 'chat_close')
    local location = Prefs.GetFromCurrentProfile("chat_config")
    if not location then
        win.Top:Set(function() return GetFrame(0).Bottom() - 700 end)
        win.Left:Set(function() return GetFrame(0).Left() + 200 end)
        win.Right:Set(function() return win.Left() + 400 end)   
    end
    
    win.OnClose = function(self)
        win:Destroy()
        win = false
    end
    
    local options = {
        filters = {}, --{type = 'filter', name = '<LOC _Links>Links', key = 'links', tooltip = 'chat_filter'}},
        winOptions = {
            --[[    {type = 'color', name = '<LOC _All>', key = 'all_color', tooltip = 'chat_color'},
                {type = 'color', name = '<LOC _Allies>', key = 'allies_color', tooltip = 'chat_color'},
                {type = 'color', name = '<LOC _Private>', key = 'priv_color', tooltip = 'chat_color'},
                {type = 'color', name = '<LOC _Links>', key = 'link_color', tooltip = 'chat_color'},
                {type = 'slider', name = '<LOC chat_0011>Font Size', key = 'font_size', tooltip = 'chat_fontsize', min = 12, max = 24, inc = 2},
                {type = 'slider', name = '<LOC chat_0012>Fade Time', key = 'fade_time', tooltip = 'chat_fadetime', min = 5, max = 30, inc = 1}, ]]
                {type = 'slider', name = '<LOC _Alpha>Alpha', key = 'win_alpha', tooltip = 'chat_alpha', min = 20, max = 100, inc = 1},
        },
    }
        
    local optionGroup = Group(win:GetClientGroup())
    LayoutHelpers.FillParent(optionGroup, win:GetClientGroup())
    optionGroup.options = {}
    local tempOptions = {}
    
    local function UpdateOption(key, value)
        if key == 'win_alpha' then
            value = value / 100
        end
        tempOptions[key] = value
    end
    
    local function CreateEntry(data)
        local group = Group(optionGroup)
        if data.type == 'filter' then
            group.name = UIUtil.CreateText(group, data.name, 14, "Arial")
            group.check = UIUtil.CreateCheckboxStd(group, '/dialogs/check-box_btn/radio')
            LayoutHelpers.AtLeftTopIn(group.check, group)
            LayoutHelpers.RightOf(group.name, group.check)
            LayoutHelpers.AtVerticalCenterIn(group.name, group.check)
            group.check.key = data.key
            group.Height:Set(group.check.Height)
            group.check.OnCheck = function(self, checked)
                UpdateOption(self.key, checked)
            end
        elseif data.type == 'color' then
            group.name = UIUtil.CreateText(group, data.name, 14, "Arial")
            local defValue = ChatOptions[data.key] or 1
            group.color = BitmapCombo(group, chatColors, defValue, true, nil, "UI_Tab_Rollover_01", "UI_Tab_Click_01")
            LayoutHelpers.AtLeftTopIn(group.color, group)
            LayoutHelpers.RightOf(group.name, group.color, 5)
            LayoutHelpers.AtVerticalCenterIn(group.name, group.color)
            group.color.Width:Set(55)
            group.color.key = data.key
            group.Height:Set(group.color.Height)
            group.color.OnClick = function(self, index)
                UpdateOption(self.key, index)
            end
        elseif data.type == 'slider' then
            group.name = UIUtil.CreateText(group, data.name, 14, "Arial")
            LayoutHelpers.AtLeftTopIn(group.name, group)
            group.slider = IntegerSlider(group, false, 
                data.min, data.max, 
                data.inc, UIUtil.SkinnableFile('/slider02/slider_btn_up.dds'), 
                UIUtil.SkinnableFile('/slider02/slider_btn_over.dds'), UIUtil.SkinnableFile('/slider02/slider_btn_down.dds'), 
                UIUtil.SkinnableFile('/dialogs/options-02/slider-back_bmp.dds'))
            LayoutHelpers.Below(group.slider, group.name)
            group.slider.key = data.key
            group.Height:Set(function() return group.name.Height() + group.slider.Height() end)
            group.slider.OnValueSet = function(self, newValue)
                UpdateOption(self.key, newValue)
            end
            group.value = UIUtil.CreateText(group, '', 14, "Arial")
            LayoutHelpers.RightOf(group.value, group.slider)
            group.slider.OnValueChanged = function(self, newValue)
                group.value:SetText(newValue)
            end
            local defValue = 1
            if data.key == 'win_alpha' then
                defValue = defValue * 100
            end
            group.slider:SetValue(defValue)
        end
        Tooltip.AddControlTooltip(group, data.tooltip or 'chat_filter')
        return group
    end
    
    --[[local armyData = GetArmiesTable()
    for i, v in armyData.armiesTable do
        if v.human then
            table.insert(options.filters, {type = 'filter', name = v.nickname, key = i})
        end
    end
    
    local filterTitle = UIUtil.CreateText(optionGroup, '<LOC chat_0013>Show Chat From:', 14, "Arial Bold")
    LayoutHelpers.AtLeftTopIn(filterTitle, optionGroup, 5, 5)
    Tooltip.AddControlTooltip(filterTitle, 'chat_filter')
    local index = 1
    for i, v in options.filters do
        optionGroup.options[index] = CreateEntry(v)
        optionGroup.options[index].Left:Set(filterTitle.Left)
        optionGroup.options[index].Right:Set(optionGroup.Right)
        if index == 1 then
            LayoutHelpers.Below(optionGroup.options[index], filterTitle, 5)
        else
            LayoutHelpers.Below(optionGroup.options[index], optionGroup.options[index-1], -2)
        end
        index = index + 1
    end]]
	local index=1
    local splitIndex = index
    local splitter = Bitmap(optionGroup)
    splitter:SetSolidColor('ff000000')
    splitter.Left:Set(optionGroup.Left)
    splitter.Right:Set(optionGroup.Right)
    splitter.Height:Set(2)
    splitter.Top:Set(function() return optionGroup.options[splitIndex-1].Bottom() + 5 end)
    
    local WindowTitle = UIUtil.CreateText(optionGroup, '<LOC chat_0014>Window Settings', 14, "Arial Bold")
    LayoutHelpers.Below(WindowTitle, splitter, 5)
    WindowTitle.Left:Set(optionGroup.Left)
    Tooltip.AddControlTooltip(WindowTitle, 'chat_color')
    
    local firstOption = true
    local optionIndex = 1
    for i, v in options.winOptions do
        optionGroup.options[index] = CreateEntry(v)
        if firstOption then
            LayoutHelpers.Below(optionGroup.options[index], WindowTitle, 5)
            optionGroup.options[index].Right:Set(function() return filterTitle.Left() + (optionGroup.Width() / 2) end)
            firstOption = false
        elseif v.type == 'color' then
            optionGroup.options[index].Right:Set(function() return filterTitle.Left() + (optionGroup.Width() / 2) end)
            if math.mod(optionIndex, 2) == 1 then
                LayoutHelpers.Below(optionGroup.options[index], optionGroup.options[index-2], 2)
            else
                LayoutHelpers.RightOf(optionGroup.options[index], optionGroup.options[index-1])
            end
        else
            LayoutHelpers.Below(optionGroup.options[index], optionGroup.options[index-1], 4)
            optionGroup.options[index].Right:Set(optionGroup.Right)
            optionGroup.options[index].Left:Set(WindowTitle.Left)
        end
        optionIndex = optionIndex + 1
        index = index + 1
    end
    
    local okBtn = UIUtil.CreateButtonStd(optionGroup, '/widgets02/small', '<LOC _Ok>', 16)
    LayoutHelpers.Below(okBtn, optionGroup.options[index-1], 4)
    LayoutHelpers.AtHorizontalCenterIn(okBtn, optionGroup)
    okBtn.OnClick = function(self)
        --ChatOptions = table.merged(ChatOptions, tempOptions)
      --  Prefs.SetToCurrentProfile("chatoptions", ChatOptions)
        --GUI.bg:OnOptionsSet()
        win:Destroy()
        win = false
    end
    
    win.Bottom:Set(function() return okBtn.Bottom() + 5 end)
	return win
end

function PlayErrorSound()
    local sound = Sound({Cue = 'UI_Menu_Error_01', Bank = 'Interface',})
        PlaySound(sound)
end