--*****************************************************************************
--* File: mods/uimod/sca_utils/modfunctions.lua
--* Author: SCA (Cleopatre, Goom and Saya)
--* Summary: Contains generic functionnalities used by the mod
--* construct the config pannel
--*
--* Copyright  2007 Gas Powered Games, Inc.  All rights reserved. 
--*****************************************************************************


--#######################################
--##Imports                                                                                            ##
--#######################################
local pathui = '/mods/uimod/hook'
local pathuiimg ='/mods/uimod/textures'
local sca_utils = '/mods/uimod/sca_utils/'
local uimod = import(sca_utils .. 'uimod.lua')
local uimanager = import(sca_utils .. 'uimanager.lua')
local gamemain = import('/lua/ui/game/gamemain.lua')
local UIUtil = import('/lua/ui/uiutil.lua')
local Prefs = import('/lua/user/prefs.lua')

local Slider = import('/lua/maui/slider.lua').Slider
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Group = import('/lua/maui/group.lua').Group
local Tooltip = import('/lua/ui/game/tooltip.lua')

--#######################################
--##Variables                                                                                         ##
--#######################################
local configwindow = {}


--#######################################
--## Basic Scripting function                                                             ##
--#######################################

--copies the values of a table into another table
--used because table2 = table1 does not copy the values but the references
function table.copy( self, deep, ud_copy_fn )
    ud_copy_fn = ud_copy_fn or function ( ud ) return ud end
    local new_table = {}
    for key, value in self do
        local new_key
        if ( deep and type( key ) == 'table' ) then
            new_key = table.copy( key, deep, ud_copy_fn )
        elseif ( type( key ) == 'userdata' ) then
            new_key = ud_copy_fn( key )
        else
            new_key = key
        end

        local new_value
        if ( deep and type( value ) == 'table' ) then
            new_value = table.copy( value, deep, ud_copy_fn )
        elseif ( type( value ) == 'userdata' ) then
            new_value = ud_copy_fn( value )
        else
            new_value = value
        end
        new_table[ new_key ] = new_value
    end
     return new_table
end

--returns the size of a table
--used because table.getn() does not work in some cases
function table.length(self)
	local j = 0
	for key,value in self do
		j = j + 1
	end
	return j	
end

--#######################################
--## Useful UI functions used often and other small functions       ##
--#######################################

--returns a text in the current language depending on the id (see 'mods/uimod/sca_utils/dialog_txt.lua')
function trad(id)
	return import(sca_utils .. 'dialog_txt.lua').trad(id)
end

--creates a text that uses the translations made in dialog_txt.lua and refreshes when the language is changed
function CreateText_sca(parent, traduction, size, style)
	local ret=UIUtil.CreateText(parent, trad(traduction), size, style)
	local res2=refresh_lang
	refresh_lang=function() if ret != nil then ret:SetText(trad(traduction)) end res2() end

	return ret
end

--function to lock a control by removing its handleevent
lock = function(control, boolean)
	if boolean == true then
		control.oldHandleEvent = control.HandleEvent
		control.HandleEvent = function(self, event)
			--do nothing, control is locked
		end
	else
		if control.oldHandleEvent != nil then
			control.HandleEvent = control.oldHandleEvent
		end
	end
end

--function to lock a slider
lockSlider = function(slider, boolean)
	lock(slider._thumb,boolean)
	lock(slider._background,boolean)
end

--#######################################
--## Fundamental UI functions used to make an interactive U I      ##
--#######################################


--Generic function to make a panel minimizable.
minimizableOnUI = function(control)
  if (control != false) and (control!=nil) then
	control.minimized = false 
	local minimizer = false
	local maximizer = false
	local minimizedLeft = -400
	local minimizedTop = 10
	
	minimizer = UIUtil.CreateButton(control, pathuiimg..'/reduc.dds', pathuiimg..'reduc_active.dds', pathuiimg..'reduc_active.dds', pathuiimg..'reduc.dds', nil, nil, nil, nil, 'UI_Arrow_Click')
	
	if (control.mPath ~= nil) then 
		maximizer = UIUtil.CreateButton(control:GetParent(), pathuiimg..control.mPath .. '.dds', pathuiimg..control.mPath .. '_active.dds' , pathuiimg..control.mPath .. '_active.dds' , pathuiimg..control.mPath .. '.dds' , nil, nil, nil, nil, 'UI_Arrow_Click')
	else 
		maximizer = UIUtil.CreateButton(control:GetParent(), pathuiimg..'reduit_normal.dds', pathuiimg..'reduit_active.dds', pathuiimg..'reduit_active.dds' , pathuiimg..'reduit_normal.dds'  , nil, nil, nil, nil, 'UI_Arrow_Click')
	end
		
	LayoutHelpers.CenteredRightOf(minimizer,control,control.mOffset)
	maximizer.Left:Set(0)
	maximizer.Right:Set(12)
	maximizer.Top:Set(minimizedTop + 50*control.mPosition + 40)
	maximizer:Hide()
	minimizer.Depth:Set(function() return control:Depth() + 5 end )
	maximizer.Depth:Set(function() return control:Depth() + 5 end )
	minimizer.OnClick = function()
		if( not control.minimized ) and (UIconfigPanel:IsHidden() or control.mminimizable_when_panel_open) then
			maximizer.Top:Set(function() return minimizedTop + 60*control.mPosition end)
			control.oldLeft = control.Left()
			control.oldTop = control.Top()
			control.oldHeight = control.Height()
			control.oldWidth = control.Width()
			control.oldAlpha = control:GetAlpha()
			if debug_sca then print("minimizing") end
			control.minimized = true
			minimizer:Hide()
			--function MoveAndSizeTo(control, newWidth, newHeight, xVal, yVal, time, mode, origin, frameFunction, initialAlpha, finalAlpha, controlparent)
			if control.Resizer then
				MyEffects.MoveAndSizeTo(control.Resizer, control.oldWidth, control.oldHeight, minimizedLeft, control.oldTop, 0.4, 0, 0,nil, control.oldAlpha, 0, control)--test
			else
				MyEffects.MoveAndSizeTo(control, control.oldWidth, control.oldHeight, minimizedLeft, control.oldTop, 0.4, 0, 0,nil, control.oldAlpha, 0)--test
			end
			maximizer:SetAlpha(1,true)
			maximizer:Show()
		else
			if( not control.minimized ) then print("UIMOD: Can't minimize this window when config panel is open")
			else LOG("INFO UIMOD: minimize trying to create an already existing minimize button (in minimizableOnUI).") end
		end
	end

	maximizer.OnClick = function()
		if( control.minimized ) then
		if debug_sca then print("maximizing") end
			control.minimized = false
			maximizer:Hide()
			minimizer:Show()
			if control.Resizer then
				MyEffects.MoveAndSizeTo(control.Resizer, control.oldWidth, control.oldHeight, control.oldLeft, control.oldTop, 0.4, 0, 0,function() if control.mRefresh == 1 then
					import('/lua/ui/game/economy.lua').controls.massWarningBG:SetAlpha(0,true)
					import('/lua/ui/game/economy.lua').controls.energyWarningBG:SetAlpha(0,true)
				end end, 0, control.oldAlpha, control)--test
				control.Resizer:Hide()
			else
				MyEffects.MoveAndSizeTo(control, control.oldWidth, control.oldHeight, control.oldLeft, control.oldTop, 0.4, 0, 0,function() if control.mRefresh == 1 then
					import('/lua/ui/game/economy.lua').controls.massWarningBG:SetAlpha(0,true)
					import('/lua/ui/game/economy.lua').controls.energyWarningBG:SetAlpha(0,true)
				end end, 0, control.oldAlpha)--test
			end
--			import('/lua/maui/effecthelpers.lua').MoveAndSizeTo(control, control.oldWidth, control.oldHeight, control.oldLeft, control.oldTop, 0, 0,1,function() if control.mRefresh == 1 then
--				import('/lua/ui/game/economy.lua').controls.massWarningBG:SetAlpha(0,true)
--				import('/lua/ui/game/economy.lua').controls.energyWarningBG:SetAlpha(0,true)
--				LOG('changing alpha')
--			end end)--test

			--if debug_sca then LOG("INFO UIMOD: Maximization done successfully.") end
		else
		--if debug_sca then LOG("INFO UIMOD: maximize is call with a control nil or false (in minimizeonUI).") end
		end
	end
	a=false
	return maximizer.OnClick
	--overviewOnUI(maximizer,control) -- TEST
   else
	WARN("WARNING UIMOD: minimizableOnUI is call with a control nil of false.")
   end
end

--minimize control from UI control panel
minimize = function(control)
	if (control.minimized == nil) or (control.minimized == false) then 
		control.minimized = false 
		WARN("Warning UIMOD: minimize is call with a control nil or false.")
	end
	if not control.minimized then
		control.oldAlpha = control:GetAlpha()
		if debug_sca then print("Minimizing") end
		Effects.FadeOut(control, 0.5, control.oldAlpha, 0 )--test		
		control.minimized = true
	end
end

--maximize control from UI control panel
maximize = function(control)
	if control.minimized then
		if debug_sca then print("Maximizing") end
		Effects.FadeOut(control, 0.5, 0, control.oldAlpha )--test		
		control.minimized = false
	else
		WARN("Warning UIMOD: maximize is call with a control nil or false.")
	end
end


--easy way to minimize until minimize(control) works better
flemmeMinimize = function(control)
	if control.Resizer.Left != nil and control.Resizer.Right != nil then
		control.Resizer.OldLeft = control.Resizer.Left()
		control.Resizer.OldRight = control.Resizer.Right()
		control.Resizer.Left:Set(4000)
		control.Resizer.Right:Set(4100)
	else
		LOG("INFO UIMOD: flemmeMinimize is call to a nil function")
	end
end

--easy way to maximize until maximize(control) works better
flemmeMaximize = function(control)
	if construction_done then
		if control.Resizer.OldLeft != nil  and control.Resizer.OldRight != nil then 
			control.Resizer.Left:Set(control.Resizer.OldLeft)
			control.Resizer.Right:Set(control.Resizer.OldRight)
		else
			LOG("INFO UIMOD: flemmeMaximize is call to a nil function")
		end
	end
end

function table.copy( self, deep, ud_copy_fn )
    ud_copy_fn = ud_copy_fn or function ( ud ) return ud end
    local new_table = {}
    for key, value in self do
        local new_key
        if ( deep and type( key ) == 'table' ) then
            new_key = table.copy( key, deep, ud_copy_fn )
        elseif ( type( key ) == 'userdata' ) then
            new_key = ud_copy_fn( key )
        else
            new_key = key
        end

        local new_value
        if ( deep and type( value ) == 'table' ) then
            new_value = table.copy( value, deep, ud_copy_fn )
        elseif ( type( value ) == 'userdata' ) then
            new_value = ud_copy_fn( value )
        else
            new_value = value
        end
        new_table[ new_key ] = new_value
    end
     return new_table
end

local snap=15



--new draggable/resizable function using an extra resize group.
--ResizeGrp is based on Window, usage is self = ResizeGrp(parent, title, icon, pin, config, lockSize, lockPosition, xmin, ymin, limitmovement,name)
--it's set up in another file because it's much longer than the original dragonui function
--xmin is minimum horizontal size, ymin is minimum vertical size defined as control.xmin in the relevant layout file, limitmove stops controls being moved offscreen
--name is important for border snapping
dragOnUI = function(control, bresize, bLimitMove, name)
	if (control!=nil and control!=false) then
		if not control.Resizer then
			--create the resize panel
			GameParent=import('/lua/ui/game/gamemain.lua').GetGameParent()
			
			if uimod.sca[name].configwindow then
				config = true
			else
				config = false
			end
			
			if trad(uimod.sca[name].name .. "_title")!="(no text)" then
				title = uimod.sca[name].name .. "_title"
			else
				title="nothing"
			end
			
			local ResizeGrp = import('/mods/uimod/sca_utils/resizegrp.lua').ResizeGrp
			if name then
				if bresize then
					control.Resizer = ResizeGrp(GameParent, title, nil, nil, config, false, false, control.xmin, control.ymin, bLimitMove, name)
				else
					control.Resizer = ResizeGrp(GameParent, title, nil, nil, config, true, false, nil, nil, bLimitMove, name)
				end
			else
				control.Resizer = ResizeGrp(GameParent, 'Unknown', nil, nil, config, true, false, nil, nil, bLimitMove)
			end
			
			--get the current position of each control from prefs and place the resizer in that position, then fill the control to the resizer so they adjust together
			
			local nom_sav = "sca"
			local vTop = 0
			local vLeft = 0
			local vBottom = 200
			local vRight = 200

			if name then
				if (Prefs.GetFromCurrentProfile("uimod_SCA_" .. nom_sav)[name]["Left"] != nil ) then
					vLeft = Prefs.GetFromCurrentProfile("uimod_SCA_" .. nom_sav)[name]["Left"]
				else
					vLeft= 0
					LOG("INFO UIMOD: using default setting for ".. name .."Left="..vLeft.." in dragOnUI")
				end

				if (Prefs.GetFromCurrentProfile("uimod_SCA_" .. nom_sav)[name]["Top"] != nil ) then 
					vTop = Prefs.GetFromCurrentProfile("uimod_SCA_".. nom_sav)[name]["Top"] 
				else
					vTop = 0
					LOG("INFO UIMOD: using default setting for ".. name .."Top="..vTop.." in dragOnUI")
				end

				if (Prefs.GetFromCurrentProfile("uimod_SCA_" .. nom_sav)[name]["Right"] != nil ) then
					vRight = Prefs.GetFromCurrentProfile("uimod_SCA_".. nom_sav)[name]["Right"] 
				else
					vRight = 200
					LOG("INFO UIMOD: using default setting for ".. name .."Right="..vRight.." in dragOnUI")
				end

				if (Prefs.GetFromCurrentProfile("uimod_SCA_" .. nom_sav)[name]["Bottom"] != nil ) then
					vBottom = Prefs.GetFromCurrentProfile("uimod_SCA_".. nom_sav)[name]["Bottom"] 
				else
					vBottom = 200
					LOG("INFO UIMOD: using default setting for ".. name .."Bottom="..vBottom.." in dragOnUI")
				end
			end

			control.Resizer.Left:Set(vLeft)
			control.Resizer.Right:Set(vRight)
			control.Resizer.Top:Set(vTop)
			control.Resizer.Bottom:Set(vBottom)
			if name == 'borders' then
				control.Resizer.Depth:Set(100)
			else
				control.Resizer.Depth:Set(900)
			end
			control.Resizer:Hide()

			if config then
				control.Resizer.OnConfigClick = function(self)
					--TODO: do I really want to save prefs every time here?
					uimod.saveModValues("default")
					configwindow[name](control.Resizer)
				end
			end
			
			if debug_sca and name then print('control resizer: '..name..' set to L: '..vLeft..' R: '..vRight..' T: '..vTop..' B: '..vBottom) end
			
			LayoutHelpers.FillParent(control, control.Resizer)
			
			--make a table of all active resizers for showing/hiding
			
			ResizeTable[name] = control.Resizer
		
		end
	end
end

--Resizer config windows, there's a better way of doing this than a big list of functions here but I can't think of it yet.
configwindow["energyBG"] = function(parent)
    if parent.configwindow then return end
    parent.configwindow = Window(GetFrame(0), 'Energy Config', nil, nil, nil, true)
    parent.configwindow.Left:Set(function() return parent.Left() + 10 end)
    parent.configwindow.Top:Set(function() return parent.Top() + 30 end)
    parent.configwindow.Right:Set(function() return parent.Left() + 180 end)
    parent.configwindow.Bottom:Set(function() return parent.Top() + 110 end)
    parent.configwindow.Depth:Set(GetFrame(0):GetTopmostDepth() + 1)
    
    local client = parent.configwindow:GetClientGroup()
	local prefsaver = Prefs.GetFromCurrentProfile("uimod_SCA_sca")
	
    local inverseenergylabel = UIUtil.CreateText(client, string.format("Inverse: "), 14)
	LayoutHelpers.AtLeftTopIn(inverseenergylabel, client, 5, 5)
	local inverseenergy = UIUtil.CreateCheckboxStd(client, '/dialogs/check-box_btn/radio')
	LayoutHelpers.CenteredRightOf(inverseenergy, inverseenergylabel)
	
	if Get_opt("inverse_barre") == 1 then
		inverseenergy:SetCheck(true)
	end
	

	inverseenergy.OnCheck = function(self, checked)
		oldLeft = prefsaver["energyBG"]["Left"]
		oldTop = prefsaver["energyBG"]["Top"]
		if checked then
			uimanager_sav["inverse_barre"] = 1
			if Get_opt("verticalenergy") == 1 then
				prefsaver["energyBG"]["Right"] = oldLeft + 104
				prefsaver["energyBG"]["Bottom"] = oldTop + 212
			else
				prefsaver["energyBG"]["Right"] = oldLeft + 372
				prefsaver["energyBG"]["Bottom"] = oldTop + 44
			end
		else
			uimanager_sav["inverse_barre"] = 0
			if Get_opt("verticalenergy") == 1 then
				prefsaver["energyBG"]["Right"] = oldLeft + 104
				prefsaver["energyBG"]["Bottom"] = oldTop + 212
			else
				prefsaver["energyBG"]["Right"] = oldLeft + 372
				prefsaver["energyBG"]["Bottom"] = oldTop + 44
			end
		end
		
		Prefs.SetToCurrentProfile("uimanager_sav", uimanager_sav)
		Prefs.SetToCurrentProfile("uimod_SCA_sca", prefsaver)
		Prefs.SavePreferences()
		uimod.RefreshLayout()
        parent.configwindow:Destroy()
        parent.configwindow = false
	end
	
    local verticalenergylabel = UIUtil.CreateText(client, string.format("Vertical: "), 14)
	LayoutHelpers.Below(verticalenergylabel, inverseenergylabel)
	local verticalenergy = UIUtil.CreateCheckboxStd(client, '/dialogs/check-box_btn/radio')
	LayoutHelpers.CenteredRightOf(verticalenergy, verticalenergylabel)
	
	
	if Get_opt("verticalenergy") == 1 then
		verticalenergy:SetCheck(true)
	end
	

	verticalenergy.OnCheck = function(self, checked)
		oldLeft = prefsaver["energyBG"]["Left"]
		oldTop = prefsaver["energyBG"]["Top"]
		if checked then
			uimanager_sav["verticalenergy"] = 1
			prefsaver["energyBG"]["Right"] = oldLeft + 104
			prefsaver["energyBG"]["Bottom"] = oldTop + 212
		else
			uimanager_sav["verticalenergy"] = 0
			if Get_opt("inverse_barre") == 1 then
				prefsaver["energyBG"]["Right"] = oldLeft + 372
				prefsaver["energyBG"]["Bottom"] = oldTop + 44
			else
				prefsaver["energyBG"]["Right"] = oldLeft + 372
				prefsaver["energyBG"]["Bottom"] = oldTop + 44
			end
		end
		
		Prefs.SetToCurrentProfile("uimanager_sav", uimanager_sav)
		Prefs.SetToCurrentProfile("uimod_SCA_sca", prefsaver)
		Prefs.SavePreferences()
		uimod.RefreshLayout()
        parent.configwindow:Destroy()
        parent.configwindow = false
	end
			
	parent.configwindow.OnClose = function(self)
        parent.configwindow:Destroy()
        parent.configwindow = false
    end	
end

configwindow["massBG"] = function(parent)
    if parent.configwindow then return end
    parent.configwindow = Window(GetFrame(0), 'Mass Config', nil, nil, nil, true)
    parent.configwindow.Left:Set(function() return parent.Left() + 10 end)
    parent.configwindow.Top:Set(function() return parent.Top() + 30 end)
    parent.configwindow.Right:Set(function() return parent.Left() + 180 end)
    parent.configwindow.Bottom:Set(function() return parent.Top() + 100 end)
    parent.configwindow.Depth:Set(GetFrame(0):GetTopmostDepth() + 1)
    
    local client = parent.configwindow:GetClientGroup()
	local prefsaver = Prefs.GetFromCurrentProfile("uimod_SCA_sca")
	
	
    local verticalmasslabel = UIUtil.CreateText(client, string.format("Vertical: "), 14)
	LayoutHelpers.AtLeftTopIn(verticalmasslabel, client, 5, 5)
	local verticalmass = UIUtil.CreateCheckboxStd(client, '/dialogs/check-box_btn/radio')
	LayoutHelpers.CenteredRightOf(verticalmass, verticalmasslabel)

	
	if Get_opt("verticalmass") == 1 then
		verticalmass:SetCheck(true)
	end
	

	verticalmass.OnCheck = function(self, checked)
		oldLeft = prefsaver["massBG"]["Left"]
		oldTop = prefsaver["massBG"]["Top"]
		if checked then
			uimanager_sav["verticalmass"] = 1
			prefsaver["massBG"]["Right"] = oldLeft + 104
			prefsaver["massBG"]["Bottom"] = oldTop + 212
		else
			uimanager_sav["verticalmass"] = 0
			prefsaver["massBG"]["Right"] = oldLeft + 372
			prefsaver["massBG"]["Bottom"] = oldTop + 44
		end
		
		Prefs.SetToCurrentProfile("uimanager_sav", uimanager_sav)
		Prefs.SetToCurrentProfile("uimod_SCA_sca", prefsaver)
		Prefs.SavePreferences()
		uimod.RefreshLayout()
        parent.configwindow:Destroy()
        parent.configwindow = false
	end
			
	parent.configwindow.OnClose = function(self)
        parent.configwindow:Destroy()
        parent.configwindow = false
    end	
end

configwindow["options"] = function(parent)
    if parent.configwindow then return end
    parent.configwindow = Window(GetFrame(0), 'Options Config', nil, nil, nil, true)
    parent.configwindow.Left:Set(function() return parent.Left() + 10 end)
    parent.configwindow.Top:Set(function() return parent.Top() + 30 end)
    parent.configwindow.Right:Set(function() return parent.Left() + 180 end)
    parent.configwindow.Bottom:Set(function() return parent.Top() + 100 end)
    parent.configwindow.Depth:Set(GetFrame(0):GetTopmostDepth() + 1)
    
    local client = parent.configwindow:GetClientGroup()
	local prefsaver = Prefs.GetFromCurrentProfile("uimod_SCA_sca")
	
	
    local verticaloptionslabel = UIUtil.CreateText(client, string.format("Vertical: "), 14)
	LayoutHelpers.AtLeftTopIn(verticaloptionslabel, client, 5, 5)
	local verticaloptions = UIUtil.CreateCheckboxStd(client, '/dialogs/check-box_btn/radio')
	LayoutHelpers.CenteredRightOf(verticaloptions, verticaloptionslabel)

	
	if Get_opt("verticaloptions") == 1 then
		verticaloptions:SetCheck(true)
	end
	

	verticaloptions.OnCheck = function(self, checked)
		oldLeft = prefsaver["options"]["Left"]
		oldTop = prefsaver["options"]["Top"]
		if checked then
			uimanager_sav["verticaloptions"] = 1
			prefsaver["options"]["Right"] = oldLeft + 64
			prefsaver["options"]["Bottom"] = oldTop + 212
		else
			uimanager_sav["verticaloptions"] = 0
			prefsaver["options"]["Right"] = oldLeft + 276
			prefsaver["options"]["Bottom"] = oldTop + 48
		end
		
		Prefs.SetToCurrentProfile("uimanager_sav", uimanager_sav)
		Prefs.SetToCurrentProfile("uimod_SCA_sca", prefsaver)
		Prefs.SavePreferences()
		uimod.RefreshLayout()
        parent.configwindow:Destroy()
        parent.configwindow = false
	end
			
	parent.configwindow.OnClose = function(self)
        parent.configwindow:Destroy()
        parent.configwindow = false
    end	
end

configwindow["MFD"] = function(parent)
    if parent.configwindow then return end
    parent.configwindow = Window(GetFrame(0), 'MFD Config', nil, nil, nil, true)
    parent.configwindow.Left:Set(function() return parent.Left() + 10 end)
    parent.configwindow.Top:Set(function() return parent.Top() + 30 end)
    parent.configwindow.Right:Set(function() return parent.Left() + 180 end)
    parent.configwindow.Bottom:Set(function() return parent.Top() + 100 end)
    parent.configwindow.Depth:Set(GetFrame(0):GetTopmostDepth() + 1)
    
    local client = parent.configwindow:GetClientGroup()
	local prefsaver = Prefs.GetFromCurrentProfile("uimod_SCA_sca")
	
	
    local verticalMFDlabel = UIUtil.CreateText(client, string.format("Vertical: "), 14)
	LayoutHelpers.AtLeftTopIn(verticalMFDlabel, client, 5, 5)
	local verticalMFD = UIUtil.CreateCheckboxStd(client, '/dialogs/check-box_btn/radio')
	LayoutHelpers.CenteredRightOf(verticalMFD, verticalMFDlabel)

	if Get_opt("verticalMFD") == 1 then
		verticalMFD:SetCheck(true)
	end
	

	verticalMFD.OnCheck = function(self, checked)
		oldLeft = prefsaver["MFD"]["Left"]
		oldTop = prefsaver["MFD"]["Top"]
		if checked then
			uimanager_sav["verticalMFD"] = 1
			prefsaver["MFD"]["Right"] = oldLeft + 64
			prefsaver["MFD"]["Bottom"] = oldTop + 212
		else
			uimanager_sav["verticalMFD"] = 0
			prefsaver["MFD"]["Right"] = oldLeft + 240
			prefsaver["MFD"]["Bottom"] = oldTop + 52
		end
		
		Prefs.SetToCurrentProfile("uimanager_sav", uimanager_sav)
		Prefs.SetToCurrentProfile("uimod_SCA_sca", prefsaver)
		Prefs.SavePreferences()
		uimod.RefreshLayout()
        parent.configwindow:Destroy()
        parent.configwindow = false
	end
			
	parent.configwindow.OnClose = function(self)
        parent.configwindow:Destroy()
        parent.configwindow = false
    end	
end

--#######################################
--## Functions To Create Generic Controls that use Save and Load ##
--#######################################


function Get_opt(name_in_game_pref)
	local tempo=Prefs.GetFromCurrentProfile("uimanager_sav")[name_in_game_pref]
	if tempo == nil then
		if not(debug_sca) then 
			WARN("WARNING UIMOD: "..name_in_game_pref.. " is a missing option in uimanager_sav (call in Get_opt), please check, value put to 1.")
			tempo=1
		else
			WARN("WARNING UIMOD: "..name_in_game_pref.. " is a missing option in uimanager_sav (call in Get_opt), please check, no value is given in debug mod.")
		end
	end
	return tempo
end

function init_checkBox(name,fct)
	if Get_opt(name) != nil then
		--LOG("init of "..name .. "= "..Get_opt(name)) 
		if (Get_opt(name)==1) then fct:SetCheck(true, false) else fct:SetCheck(false, false) end
	else -- is nil
		WARN("WARNING UIMOD: "..name.. " is a missing option in uimanager_sav (call in init_checkBox)")
	end
end

function tooltips(parent,help_tips)
	if help_tips!=nil and help_tips!=false then
		local oldHandleEvent = parent.HandleEvent
		parent.HandleEvent = function(self, event)
			if event.Type == 'MouseEnter' then
				Tooltip.CreateMouseoverDisplay(self, trad(help_tips), .5) --, true)
			elseif event.Type == 'MouseExit' then
				Tooltip.DestroyMouseoverDisplay()
			end
			oldHandleEvent(self,event)
		end
	end
end	

-- create a checkbox with his value  automaticly saved in uimanager_sav
-- give your parent, his place
-- name_in_game_pref: his name in uimanager_sav -->WARNING: you must creat the EXACT same entry in uimanager_sav on the top of file, this will be also the reset value
-- brefresh= must we refresh the layout after the change ?
-- bmake_init= auto intialise ?
-- when the reset button is click will automaticly reset all the checkBox and slider create by these function/ coudl call the reset with reset()
-- for the labeling(only if label_txt used): if put nothing in size, style, and xrelatif/y if you want it by default
function Make_CheckBox(parent,x,y,name_in_game_pref,fct_ON,fct_OFF,brefresh,bmake_init,label_txt,xrelatif,yrelatif,size_txt,style,color,help_tips)
   if not(((parent==nil) or (parent==false)) or (Get_opt(name_in_game_pref)==nil)) then
	local check_bx
	local label
	check_bx = UIUtil.CreateCheckboxStd(parent, '/dialogs/check-box_btn/radio')
	LayoutHelpers.AtLeftTopIn(check_bx, parent, x,y)
	check_bx.OnCheck = function(self, checked)
		if checked then
			if debug_sca then print(name_in_game_pref .. " - check True") end
			uimanager_sav[name_in_game_pref] = 1
			if construction_done then 
				Prefs.SetToCurrentProfile("uimanager_sav", uimanager_sav)
				Prefs.SavePreferences()
			end
			if fct_ON != nil then fct_ON(checked) end
		else
			if debug_sca then print(name_in_game_pref .. " - check False") end
			uimanager_sav[name_in_game_pref] = 0
			if construction_done then 
				Prefs.SetToCurrentProfile("uimanager_sav", uimanager_sav)
				Prefs.SavePreferences()
			end
			if fct_OFF != nil then fct_OFF(checked) end
		end

		if brefresh then uimod.RefreshLayout() end
	end
	if bmake_init then
		init_checkBox(name_in_game_pref,check_bx)
	end
	
	if (label_txt!=nil) and (label_txt!=false) then
		if (style==false) or (style==nil) then style= UIUtil.titleFont end
		if (size_txt == false) or (size_txt ==nil) then size_txt=15 end
		if (xrelatif == false) or (xrelatif==nil) then xrelatif=35 end
		if (yrelatif == false) or (yrelatif==nil) then yrelatif=5 end
		label = UIUtil.CreateText(check_bx, trad(label_txt), size_txt, style)
		if ((xrelatif !=nil) and (xrelatif!=false)) and ((yrelatif !=nil) and (yrelatif!=false)) then 
			LayoutHelpers.AtLeftTopIn(label, check_bx, xrelatif,yrelatif)
		end
		if (color!=nil) and (color!=false) then label:SetColor(color) else  label:SetColor(UIUtil.dialogCaptionColor) end
	end
	
	tooltips(check_bx,help_tips)

	--create_label(check_bx,label_txt, size_txt, style,xrelatif,yrelatif,color,help_tips)
	local res2=refresh_lang
	refresh_lang=function() if label_txt!=nil and label!=nil then label:SetText(trad(label_txt)) end if help_tips!=nil then tooltips(check_bx,help_tips) end res2() end
	
	ConfigCheckTable[name_in_game_pref] = check_bx
	
	return check_bx,label
   else
	WARN("WARNING UIMOD: Make_checkbox call with incorrect parameter. (parent nil or false or pb in value in Gamepref="..repr(Get_opt(name_in_game_pref))..")")
   end	
end

function init_slider(name,fct)
	if Get_opt(name) != nil then
		if (Get_opt(name)>=0) and (Get_opt(name)<=1) then fct:SetValue(Get_opt(name)) end
	else -- is nil
		WARN("WARNING UIMOD: "..name.. " is a missing option in uimanager_sav (call in init_slider)")
	end
end

-- create a slider with his value  automaticly saved in uimanager_sav
-- almost same as Make_checkbox

-- when the reset button is click will automaticly reset all the checkBox and slider create by these function
-- Warning: you must creat EXACLY the same entry in uimanager_sav on the top of file, this will be the reset value this entry MUST between 0 and 1 (in fact >debut and <fin)

function Make_Slider(parent,x,y,name_in_game_pref,debut,fin,fct_call,bmake_init,label_txt,xrelatif,yrelatif,size_txt,style,color,help_tips)
	--transparency slider
   if not((parent ==nil or parent ==false or debut<0 or debut >1 or fin<0 or fin>1 or fin<=debut or Get_opt(name_in_game_pref)==nil)) then
	local slider
	local label
	slider = Slider(parent, false, debut,fin, UIUtil.SkinnableFile('/slider02/slider_btn_up.dds'), UIUtil.SkinnableFile('/slider02/slider_btn_over.dds'), UIUtil.SkinnableFile('/slider02/slider_btn_down.dds'), UIUtil.SkinnableFile('/slider02/slider-back_bmp.dds'))  
	LayoutHelpers.AtLeftTopIn(slider, parent, x,y) --72, 410)
	slider:SetValue(Get_opt(name_in_game_pref))
	
	local tempo =1
	slider.OnValueSet = function(self, newValue)
		if debug_sca then print(name_in_game_pref .. "= "..newValue.." - slider action(onValueSet)") end
		uimanager_sav[name_in_game_pref] = newValue  
		if construction_done then
			Prefs.SetToCurrentProfile("uimanager_sav", uimanager_sav)
			Prefs.SavePreferences()
		end
		if (fct_call!=nil) and ((newValue!=nil) and ((newValue>=0) and (newValue<=1))) then fct_call(newValue) end
	end
	slider.OnValueChanged = function(self, newValue)
		if debug_sca then print(name_in_game_pref .. "= "..newValue.." - slider action(onValueChanged)") end
		uimanager_sav[name_in_game_pref] = newValue  
		if construction_done then
			Prefs.SetToCurrentProfile("uimanager_sav", uimanager_sav)
			Prefs.SavePreferences()
		end
		if (fct_call!=nil) and ((newValue!=nil) and ((newValue>=0) and (newValue<=1))) then fct_call(newValue) end
	end

	if bmake_init then
		init_slider(name_in_game_pref,slider)
	end
	
	if (label_txt!=nil) and (label_txt!=false) then
		if (style==false) or (style==nil) then style= UIUtil.titleFont end
		if (size_txt == false) or (size_txt ==nil) then size_txt=15 end
		if (xrelatif == false) or (xrelatif==nil) then xrelatif=40 end
		if (yrelatif == false) or (yrelatif==nil) then yrelatif=5 end
		label = UIUtil.CreateText(slider, trad(label_txt), size_txt, style)
		if ((xrelatif !=nil) and (xrelatif!=false)) and ((yrelatif !=nil) and (yrelatif!=false)) then 
			LayoutHelpers.AtLeftTopIn(label,slider, xrelatif,yrelatif)
		end
		if (color!=nil) and (color!=false) then label:SetColor(color) else  label:SetColor(UIUtil.dialogCaptionColor) end
	end

	tooltips(slider,help_tips)

	--create_label(check_bx,label_txt, size_txt, style,xrelatif,yrelatif,color,help_tips)
	local res2=refresh_lang
	refresh_lang=function() 
		if label_txt!=nil and label!=nil then 
			label:SetText(trad(label_txt)) end 
		if help_tips!=nil then tooltips(slider,help_tips) end res2() 
	end

	
	ConfigSliderTable[name_in_game_pref] = slider

	return slider,label
   else
	WARN("WARNING UIMOD: Make_slider call with incorrect parameter.")
   end	
end



--#######################################
--## Miscallenious functions                                                               ##
--#######################################

--resizes the icons size
function resizeIcons(value)	

	--various imports
	local construction = import('/lua/ui/game/construction.lua')
	local construction_sca = import(pathui .. 'lua/ui/game/layouts/construction_sca.lua')
	local construction_sca_mini = import(pathui .. 'lua/ui/game/layouts/construction_sca_mini.lua')
	local queue = import('/lua/ui/game/queue.lua')
	local queue_sca = import(pathui .. 'lua/ui/game/layouts/queue_sca.lua')
	local queue_sca_mini = import(pathui .. 'lua/ui/game/layouts/queue_sca_mini.lua')
	local selectionmanager_sca = import(pathui .. 'lua/ui/game/layouts/selectionmanager_sca.lua')
	local selectionmanager_sca_mini = import(pathui .. 'lua/ui/game/layouts/selectionmanager_sca_mini.lua')
	local selectionmanager = import('/lua/ui/game/selectionmanager.lua')

	
	--set the size of the small preview icon
	if infoIcon != false then
		infoIcon.Width:Set(value)
		infoIcon.Height:Set(value)
	end
	
	--set the icon sizes
	construction.boutonH = value
	construction.boutonW = value
	queue.boutonH = value
	queue.boutonW = value
	selectionmanager.boutonH = value
	selectionmanager.boutonW = value
	
	--set the grid sizes
	construction_sca.gridX = construction.boutonW
	construction_sca.gridY = construction.boutonH	
	construction_sca_mini.gridX = construction.boutonW
	construction_sca_mini.gridY = construction.boutonH	
	queue_sca.gridX = queue.boutonH
	queue_sca.gridY = queue.boutonW
	queue_sca_mini.gridX = queue.boutonH
	queue_sca_mini.gridY = queue.boutonW
	selectionmanager_sca.gridX = queue.boutonH
	selectionmanager_sca.gridY = queue.boutonW
	selectionmanager_sca_mini.gridX = queue.boutonH
	selectionmanager_sca_mini.gridY = queue.boutonW

	
end

-- adds a strategic icon to parent
--used to add small strategic icons to the build icons
function addStrategicIconTo(oParent, bpUnit)
	if not oParent.bmStratIcon then
		local sTexture = false
		if aSpecificStratIcons[bpUnit.BlueprintId] then
			sTexture = pathuiimg.. '/icons_strategic/'..aSpecificStratIcons[bpUnit.BlueprintId]..'.dds'
		elseif aStratIconTranslation[bpUnit.StrategicIconName] then
	    	sTexture = pathuiimg.. '/icons_strategic/'..aStratIconTranslation[bpUnit.StrategicIconName]..'.dds'
	    end
	    if sTexture then
	    	bmStratIcon = Bitmap(oParent)
	    	bmStratIcon:SetTexture(sTexture)
	    	bmStratIcon.Bottom:Set(function() return oParent.Bottom() - 1 end)
			bmStratIcon.Right:Set(function() return oParent.Right() - 1 end)
			bmStratIcon.Height:Set(construction.boutonH/2)
			bmStratIcon.Width:Set(construction.boutonW/2)
			bmStratIcon:SetAlpha(0.8)
	    	bmStratIcon:DisableHitTest()
	    	oParent.bmStratIcon = bmStratIcon
	    end
    end
end

--#######################################
--## Functions in progress   (Do not work correctly)                         ##
--#######################################

--minimize control from UI control panel
minimize = function(control)
	if (control.minimized == nil) or (control.minimized == false) then 
		control.minimized = false 
		WARN("Warning UIMOD: minimize is call with a control nil or false.")
	end
	if not control.minimized then
		control.oldAlpha = control:GetAlpha()
		if debug_sca then print("Minimizing") end
		Effects.FadeOut(control, 0.5, control.oldAlpha, 0 )--test		
		control.minimized = true
	end
end

--maximize control from UI control panel
maximize = function(control)
	if control.minimized then
		if debug_sca then print("Maximizing") end
		Effects.FadeOut(control, 0.5, 0, control.oldAlpha )--test		
		control.minimized = false
	else
		WARN("Warning UIMOD: maximize is call with a control nil or false.")
	end
end

--generic function to generate overview
overviewOnUI = function(control,control2)
	local oldHandleEvent = control.HandleEvent
	control.HandleEvent = function(self, event)	
	
		if event.Type == 'MouseEnter' then
			print("entering")			
			control2:SetAlpha(1,true)
			control2:Show()
			print("done")
		elseif event.Type == 'MouseExit' then
			print("exiting")			
			control2:SetAlpha(0,true)
			control2:Hide()
			print("done")
		end
		
		oldHandleEvent(self, event)
		
	end
	
end



