--################################
--##          KEY      REMAPPING          SCA                      ##
--################################
--we have 4 tabs: keys, commands , command equivalents and result
--a result is an assiociation of a key and a command
--the results table is used to replace the defaultKeyMap
--key and command are the list of keys and commands in strings
--command equivalents associates a lua script to a command string

local sca_utils = '/mods/uimod/sca_utils/'
local pathui = '/mods/uimod/hook'
local pathuiimg='/mods/uimod/textures'
local UIManager = import(sca_utils .. 'uimanager.lua')
local UImod = import(sca_utils .. 'uimod.lua')
local Prefs = import('/lua/user/prefs.lua')--preferences
local UIUtil = import('/lua/ui/uiutil.lua')
local Group = import('/lua/maui/group.lua').Group
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Effect = import('/lua/maui/effecthelpers.lua')
local keymapfile = import('/mods/uimod/addons/sca.keymap')

local default = import('/lua/keymap/defaultKeyMap.lua').defaultKeyMap
local debug_sca = false
local cycle_map = false
local cycle_time=0
local cycle_on=false
local Glow = false

local button={}
local cmd_sav={}
local tech_sav={}
local factory_sav={}
local lvl_sav={}
local imax=0
local key_sav=false

boutonH = 48
boutonW = 48

--    *  IN_AddKeyMapTable --
--    IN_AddKeyMapTable(keyMapTable) - add a set of key mappings --
--    * IN_ClearKeyMap --
--    IN_ClearKeyMap() - clears all key mappings --
--    * IN_RemoveKeyMapTable --
--    IN_RemoveKeyMapTable(keyMapTable) - removes the keys from the key map 
--    LOG(repr(table)) LOG all keys and values  of a table, very useful for debugging

--table copy method
local copy = UIManager.table.copy
	
--save default  keymap table
local original =  table.copy(default)

--temporary table used for confirm/cancel/restore default operations
temp = default
	
--result key map
scaKeymap = {}
keymaps = {names = {}, data = {}}

--list of blueprint id's used for each category
buildingTab = {
	--buildings
	["LandFactory"] = {
		"uab0101",
		"ueb0101",
		"urb0101",},
	["AirFactory"] = {
		"uab0102",
		"ueb0102",
		"urb0102",},
	["NavalFactory"] = {
		"uab0103",
		"ueb0103",
		"urb0103",},
	["MassExtractor"] = {
		"uab1302",
		"uab1202",
		"uab1103",
		"ueb1302",
		"ueb1202",
		"ueb1103",
		"urb1302",
		"urb1202",
		"urb1103",},
	["MassFabricator"] = {
		"uab1303",
		"uab1104",
		"ueb1303",
		"ueb1104",
		"urb1303",
		"urb1104",},
	["MassStorage"] = {
		"uab1106",
		"ueb1106",
		"urb1106",},
	["PowerGen"] = {
		"uab1301",
		"uab1201",
		"uab1101",
		"ueb1301",
		"ueb1201",
		"ueb1101",
		"urb1301",
		"urb1201",
		"urb1101",},
	["Hydrocarbon"] = {
		"uab1102",
		"ueb1102",
		"urb1102",},
	["EnergyStorage"] = {
		"uab1105",
		"ueb1105",
		"urb1105",},
	["Wall"] = {
		"uab5101",
		"ueb5101",
		"urb5101"},
	["PointDefense"] = {
		"uab2301",
		"uab2101",
		"ueb2301",
		"ueb2101",
		"urb2301",
		"urb2101",},
	["AA"] = {
		"uab2304",
		"uab2204",
		"uab2104",
		"ueb2304",
		"ueb2204",
		"ueb2104",
		"urb2304",
		"urb2204",
		"urb2104",},
	["TorpLauncher"] = {
		"uab2205",
		"uab2109",
		"ueb2205",
		"ueb2109",
		"urb2205",
		"urb2109",},
	["Radar"] = {
		"uab3104",
		"uab3201",
		"uab3101",
		"ueb3104",
		"ueb3201",
		"ueb3101",
		"urb3104",
		"urb3201",
		"urb3101",},
	["Sonar"] = {
		"uas0305",
		"uab3202",
		"uab3102",
		"ues0305",
		"ueb3202",
		"ueb3102",
		"urs0305",
		"urb3202",
		"urb3102",},
	["T2Artillery"] = {
		"uab2303",
		"ueb2303",
		"urb2303",},
	["T3Artillery"] = {
		"uab2302",
		"ueb2302",
		"urb2302",},
	["TacticalMissile"] = {
		"uab2108",
		"ueb2108",
		"urb2108",},
	["TacticalDefense"] = {
		"uab4201",
		"ueb4201",
		"urb4201",},
	["StrategicMissile"] = {
		"uab2305",
		"ueb2305",
		"urb2305",},
	["StratDefense"] = {
		"uab4302",
		"ueb4302",
		"urb4302",},
	["ShieldGen"] = {
		"uab4301",
		"uab4202",
		"ueb4301",
		"ueb4202",
		"urb4202",},
	["StealthGen"] = {
		"uab4203",
		"ueb4203",
		"urb4203",},
	["AirRepair"] = {
		"uab5202",
		"ueb5202",
		"urb5202",},
	["QuantumGate"] = {
		"uab0304",
		"ueb0304",
		"urb0304",},
	["Experimental1"] = {
		"uaa0310",
		"uel0401",
		"url0402",},
	["Experimental2"] = {
		"uas0401",
		"ues0401",
		"ura0401",},
	["Experimental3"] = {
		"ual0401",
		"ueb2401",
		"url0401",},
	--units
	["Engineer"] = {
		"ual0301",	--subcom
		"ual0309",	--t3 engineer
		"ual0208",	--t2 engineer
		"ual0105",	--t1 engineer
		"uel0301",
		"uel0309",
		"uel0208",
		"uel0105",
		"url0301",
		"url0309",
		"url0208",
		"url0105",},
	["BotBomb"] = {
		"ual0303",
		"ual0106",
		"uaa0304",
		"uaa0103",
		"uel0303",
		"uel0106",
		"uea0304",
		"uea0103",
		"url0303",
		"url0106",
		"ura0304",
		"ura0103",},
	["Artillery"] = {
		"ual0304",	--t3 arty
		"ual0103",	--t1 arty
		"uas0302",	--battleship
		"uel0304",
		"uel0103",
		"ues0302",
		"url0304",
		"url0103",
		"urs0302",},
	["Tanks"] = {
		"ual0202",	--t2 tank
		"ual0201",	--t1 tank
		"uaa0203",	--gunship
		"uas0201",	--destroyer
		"uel0202",
		"uel0201",
		"uea0305",	--t3 gunship
		"uea0203",
		"ues0201",
		"url0202",
		"url0107",
		"ura0203",
		"urs0201",},
	["Missile"] = {
		"ual0111",	--mobile missile
		"uas0202",	--cruiser
		"uel0111",
		"ues0202",
		"url0111",
		"urs0202",},
	["TransAmph"] = {
		"uaa0104",	--t2 transport
		"uaa0107",	--t1 transport
		"uel0203",	--amphib tank
		"uea0104",
		"uea0107",
		"url0203",
		"ura0104",
		"ura0107",},
	["ShieldStealth"] = {
		"ual0307",
		"uel0307",
		"url0306",},
	["Scouts"] = {
		"ual0101",	--land scout
		"uaa0302",	--t3 air scout
		"uaa0101",	--t1 air scout
		"uas0103",	--frigate
		"uel0101",
		"uea0302",
		"uea0101",
		"ues0103",
		"url0101",
		"ura0302",
		"ura0101",
		"urs0103",},
	["AntiAir"] = {
		"ual0205",	--t2 aa
		"ual0104",	--t1 aa
		"uaa0303",	--t3 fighter
		"uaa0102",	--t1 fighter
		"uas0102",	--aa ship
		"uel0205",
		"uel0104",
		"uea0303",
		"uea0102",
		"url0205",
		"url0104",
		"ura0303",
		"ura0102",},
	["Torpedo"] = {
		"uaa0204",	--torp bomber
		"uas0203",	--sub
		"uea0204",
		"ues0203",
		"ura0204",
		"urs0203",},
	["Carrier"] = {
		"uas0303",
		"urs0303",},
--upgrades
	["Upgrade"] = {
		"uab0201",
		"uab0202",
		"uab0203",
		"uab0301",
		"uab0302",
		"uab0303",
		"uab1202",
		"uab1302",
		"uab3201",
		"uab3104",
		"uab3202",
		"uas0305",
		"ueb0201",
		"ueb0202",
		"ueb0203",
		"ueb0301",
		"ueb0302",
		"ueb0303",
		"ueb1202",
		"ueb1302",
		"ueb3201",
		"ueb3104",
		"ueb3202",
		"ues0305",
		"ueb4202",
		"urb0201",
		"urb0202",
		"urb0203",
		"urb0301",
		"urb0302",
		"urb0303",
		"urb1202",
		"urb1302",
		"urb3201",
		"urb3104",
		"urb3202",
		"urs0305",},
}

--List of all string-command equivalents
CommandEquivalents = {
    ['OPTIONS']            = {action = 'UI_Lua import("/lua/ui/uimain.lua").EscapeHandler()', stringkey = 'escape', category = '<LOC keymap_category_0000>UI', order = 1,},
    ['HIDEUI']    = {action = 'UI_ToggleGamePanels', stringkey = 'ssmode', category = '<LOC keymap_category_0000>UI', order = 2,}, # Temporary assignment, should find a better key
    ['PAUSE']          = {action = 'UI_Lua import("/lua/ui/game/options.lua").PauseGame()', stringkey = 'pause', category = '<LOC keymap_category_0000>UI', order = 3,},
    ['SCREENSHOT']         = {action = 'Dump_Frame', stringkey = 'cap_frame', category = '<LOC keymap_category_0000>UI', order = 4,},
    ['LIFEBARS']          = {action =  'UI_RenderUnitBars', stringkey = 'toggle_lifebars', category = '<LOC keymap_category_0000>UI', order = 1,},
    ['MILITARY OVERLAY']         = {action =  'UI_Lua import("/lua/ui/game/multifunction.lua").ToggleMilitary()', stringkey = 'tog_military', category = '<LOC keymap_category_0000>UI', order = 5,},
    ['DEFENSE OVERLAY']         = {action =  'UI_Lua import("/lua/ui/game/multifunction.lua").ToggleDefense()', stringkey = 'tog_defense', category = '<LOC keymap_category_0000>UI', order = 6,},
    ['ECONOMY OVERLAY']         = {action =  'UI_Lua import("/lua/ui/game/multifunction.lua").ToggleEconomy()', stringkey = 'tog_econ', category = '<LOC keymap_category_0000>UI', order = 7,},
    ['INTEL OVERLAY']         = {action =  'UI_Lua import("/lua/ui/game/multifunction.lua").ToggleIntel()', stringkey = 'tog_intel', category = '<LOC keymap_category_0000>UI', order = 8,},
    ['RENAME']         = {action =  'UI_ShowRenameDialog', stringkey = 'rename', category = '<LOC keymap_category_0000>UI', order = 10,},
    ['SPLIT SCREEN']           = {action =  'UI_Lua import("/lua/ui/game/borders.lua").SplitMapGroup(true)', stringkey = 'split_screen_enable', category = '<LOC keymap_category_0000>UI', order = 13,},
    ['UNSPLIT SCREEN']            = {action =  'UI_Lua import("/lua/ui/game/borders.lua").SplitMapGroup(false)', stringkey = 'split_screen_disable', category = '<LOC keymap_category_0000>UI', order = 14,},
    ['SWITCH LAYOUT UP']    = {action =  'UI_RotateLayout +', stringkey = 'switch_layout_up', category = '<LOC keymap_category_0000>UI', order = 17,},
    ['SWITCH LAYOUT DOWN']  = {action =  'UI_RotateLayout -', stringkey = 'switch_layout_down', category = '<LOC keymap_category_0000>UI', order = 18,},
    ['CONNECTIVITY']            = {action =  'UI_Lua import("/lua/ui/game/connectivity.lua").CreateUI()', stringkey = 'toggle_disconnect_screen', category = '<LOC keymap_category_0000>UI', order = 19,},

    ['GROUP 1']              = {action = 'UI_ApplySelectionSet 1', stringkey = 'group1', category = '<LOC keymap_category_0004>Selection', order = 1,},
    ['GROUP 2']              = {action =  'UI_ApplySelectionSet 2', stringkey = 'group2', category = '<LOC keymap_category_0004>Selection', order = 2,},
    ['GROUP 3']              = {action =  'UI_ApplySelectionSet 3', stringkey = 'group3', category = '<LOC keymap_category_0004>Selection', order = 3,},
    ['GROUP 4']              = {action =  'UI_ApplySelectionSet 4', stringkey = 'group4', category = '<LOC keymap_category_0004>Selection', order = 4,},
    ['GROUP 5']              = {action =  'UI_ApplySelectionSet 5', stringkey = 'group5', category = '<LOC keymap_category_0004>Selection', order = 5,},
    ['GROUP 6']              = {action =  'UI_ApplySelectionSet 6', stringkey = 'group6', category = '<LOC keymap_category_0004>Selection', order = 6,},
    ['GROUP 7']              = {action =  'UI_ApplySelectionSet 7', stringkey = 'group7', category = '<LOC keymap_category_0004>Selection', order = 7,},
    ['GROUP 8']              = {action =  'UI_ApplySelectionSet 8', stringkey = 'group8', category = '<LOC keymap_category_0004>Selection', order = 8,},
    ['GROUP 9']              = {action =  'UI_ApplySelectionSet 9', stringkey = 'group9', category = '<LOC keymap_category_0004>Selection', order = 9,},
    ['GROUP 0']              = {action =  'UI_ApplySelectionSet 0', stringkey = 'group0', category = '<LOC keymap_category_0004>Selection', order = 10,},
    ['SET GROUP 1']         = {action =  'UI_MakeSelectionSet 1', stringkey = 'set_group1', category = '<LOC keymap_category_0004>Selection', order = 11,},
    ['SET GROUP 2']         = {action =  'UI_MakeSelectionSet 2', stringkey = 'set_group2', category = '<LOC keymap_category_0004>Selection', order = 12,},
    ['SET GROUP 3']         = {action =  'UI_MakeSelectionSet 3', stringkey = 'set_group3', category = '<LOC keymap_category_0004>Selection', order = 13,},
    ['SET GROUP 4']         = {action =  'UI_MakeSelectionSet 4', stringkey = 'set_group4', category = '<LOC keymap_category_0004>Selection', order = 14,},
    ['SET GROUP 5']         = {action =  'UI_MakeSelectionSet 5', stringkey = 'set_group5', category = '<LOC keymap_category_0004>Selection', order = 15,},
    ['SET GROUP 6']         = {action =  'UI_MakeSelectionSet 6', stringkey = 'set_group6', category = '<LOC keymap_category_0004>Selection', order = 16,},
    ['SET GROUP 7']         = {action =  'UI_MakeSelectionSet 7', stringkey = 'set_group7', category = '<LOC keymap_category_0004>Selection', order = 17,},
    ['SET GROUP 8']         = {action =  'UI_MakeSelectionSet 8', stringkey = 'set_group8', category = '<LOC keymap_category_0004>Selection', order = 18,},
    ['SET GROUP 9']         = {action =  'UI_MakeSelectionSet 9', stringkey = 'set_group9', category = '<LOC keymap_category_0004>Selection', order = 19,},
    ['SET GROUP 0']         = {action =  'UI_MakeSelectionSet 0', stringkey = 'set_group0', category = '<LOC keymap_category_0004>Selection', order = 20,},
    ['SELECT AIR']         = {action =  'UI_SelectByCategory AIR MOBILE', stringkey = 'select_air', category = '<LOC keymap_category_0004>Selection', order = 23,},
    ['SELECT NAVAL']         = {action =  'UI_SelectByCategory NAVAL MOBILE', stringkey = 'select_naval', category = '<LOC keymap_category_0004>Selection', order = 24,},
    ['SELECT LAND']         = {action =  'UI_SelectByCategory +excludeengineers LAND MOBILE', stringkey = 'select_land', category = '<LOC keymap_category_0004>Selection', order = 25,},
    ['SELECT SAME TYPE']         = {action =  'UI_ExpandCurrentSelection', stringkey = 'select_all_units_of_same_type', category = '<LOC keymap_category_0004>Selection', order = 26,},
    ['SELECT ENGINEERS']         = {action =  'UI_SelectByCategory ENGINEER', stringkey = 'select_engineers', category = '<LOC keymap_category_0004>Selection', order = 27,},
    ['GOTO ENGINEER']   = {action =  'UI_SelectByCategory +nearest +idle +goto ENGINEER', stringkey = 'goto_engineer', category = '<LOC keymap_category_0004>Selection', order = 29,},
    ['SELECT IDLE ENGINEER']     = {action =  'UI_SelectByCategory +nearest +idle ENGINEER', stringkey = 'select_idle_engineer', category = '<LOC keymap_category_0004>Selection', order = 30,},
    ['CYCLE ENGINEERS']         = {action =  'UI_Lua import("/lua/ui/game/avatars.lua").GetEngineerGeneric()', stringkey = 'cycle_engineers', category = '<LOC keymap_category_0004>Selection', order = 31,},
    ['VIEW COMMANDER']          = {action =  'UI_SelectByCategory +nearest +goto COMMAND', stringkey = 'goto_commander', category = '<LOC keymap_category_0004>Selection', order = 32,},
    ['SELECT COMMANDER']      = {action =  'UI_SelectByCategory +nearest COMMAND', stringkey = 'select_commander', category = '<LOC keymap_category_0004>Selection', order = 33,},
    ['SELECT ALL']         = {action =  'UI_SelectByCategory ALLUNITS', stringkey = 'select_all', category = '<LOC keymap_category_0004>Selection', order = 34,},
    ['SELECT ALL ON SCREEN']         = {action =  'UI_SelectByCategory +inview ALLUNITS', stringkey = 'select_all_onscreen', category = '<LOC keymap_category_0004>Selection', order = 35,},
    ['SELECT ALL ENGINEERS ON SCREEN']    = {action =  'UI_SelectByCategory +inview ENGINEER', stringkey = 'select_all_eng_onscreen', category = '<LOC keymap_category_0004>Selection', order = 36,},
    ['SELECT ALL FACTORIES ON SCREEN']         = {action =  'UI_SelectByCategory +inview FACTORY', stringkey = 'select_all_factory_onscreen', category = '<LOC keymap_category_0004>Selection', order = 37,},
    ['SELECT NEAREST FACTORY']              = {action =  'UI_SelectByCategory +nearest FACTORY', stringkey = 'select_nearest_factory', category = '<LOC keymap_category_0004>Selection', order = 38,},
    ['SELECT NEAREST LAND FACTORY']   = {action =  'UI_SelectByCategory +nearest LAND FACTORY', stringkey = 'select_nearest_land_factory', category = '<LOC keymap_category_0004>Selection', order = 39,},
    ['SELECT NEAREST AIR FACTORY']   = {action =  'UI_SelectByCategory +nearest AIR FACTORY', stringkey = 'select_nearest_air_factory', category = '<LOC keymap_category_0004>Selection', order = 40,},
    ['SELECT NEAREST NAVAL FACTORY']   = {action =  'UI_SelectByCategory +nearest NAVAL FACTORY', stringkey = 'select_nearest_naval_factory', category = '<LOC keymap_category_0004>Selection', order = 41,},
    ['SELECT NEXT LAND FACTORY']   = {action =  'UI_Lua import("/mods/uimod/sca_utils/selectionfunctions.lua").GetNextLandFactory()' , stringkey = 'select_next_land_factory', category = '<LOC keymap_category_0004>Selection', order = 42,},
    ['SELECT NEXT AIR FACTORY']   = {action =  'UI_Lua import("/mods/uimod/sca_utils/selectionfunctions.lua").GetNextAirFactory()' , stringkey = 'select_next_air_factory', category = '<LOC keymap_category_0004>Selection', order = 43,},
    ['SELECT NEXT NAVAL FACTORY']   = {action =  'UI_Lua import("/mods/uimod/sca_utils/selectionfunctions.lua").GetNextNavalFactory()' , stringkey = 'select_next_naval_factory', category = '<LOC keymap_category_0004>Selection', order = 44,},
    ['SELECT SIMILAR UNITS']   = {action =  'UI_Lua import("/mods/uimod/sca_utils/selectionfunctions.lua").GetSimilarUnits()' , stringkey = 'select_similar', category = '<LOC keymap_category_0004>Selection', order = 45,},
	
    ['RECALL CAMERA POSITION']            = {action =  'UI_Lua import("/lua/ui/game/zoomslider.lua").RecallCameraPos()', stringkey = 'next_cam_position', category = '<LOC keymap_category_0025>Camera', order = 1,},
    ['SAVE CAMERA POSITION']      = {action =  'UI_Lua import("/lua/ui/game/zoomslider.lua").SaveCameraPos()', stringkey = 'add_cam_position', category = '<LOC keymap_category_0025>Camera', order = 2,},
    ['REMOVE CAMERA POSITION']       = {action =  'UI_Lua import("/lua/ui/game/zoomslider.lua").RemoveCamPos()', stringkey = 'rem_cam_position', category = '<LOC keymap_category_0025>Camera', order = 3,},
    ['ZOOM IN']              = {action =  'UI_Lua import("/lua/ui/game/zoomslider.lua").ZoomIn(.02)', stringkey = 'zoom_in', category = '<LOC keymap_category_0025>Camera', order = 4,},
    ['ZOOM OUT']              = {action =  'UI_Lua import("/lua/ui/game/zoomslider.lua").ZoomOut(.02)', stringkey = 'zoom_out', category = '<LOC keymap_category_0025>Camera', order = 5,},
    ['ZOOM IN FAST']        = {action =  'UI_Lua import("/lua/ui/game/zoomslider.lua").ZoomIn(.08)', stringkey = 'zoom_in_fast', category = '<LOC keymap_category_0025>Camera', order = 6,},
    ['ZOOM OUT FAST']        = {action =  'UI_Lua import("/lua/ui/game/zoomslider.lua").ZoomOut(.08)', stringkey = 'zoom_out_fast', category = '<LOC keymap_category_0025>Camera', order = 7,},
    ['TRACK UNIT']              = {action =  'UI_TrackUnit WorldCamera', stringkey = 'track_unit', category = '<LOC keymap_category_0025>Camera', order = 8,},
    ['TRACK UNIT ON MINIMAP']   = {action =  'UI_TrackUnit MiniMap', stringkey = 'track_unit_minimap', category = '<LOC keymap_category_0025>Camera', order = 9,},
    ['TRACK UNIT ON SECOND MONITOR']     = {action =  'UI_TrackUnit CameraHead2', stringkey = 'track_unit_second_mon', category = '<LOC keymap_category_0025>Camera', order = 10,},
    ['FREE CAMERA']         = {action =  'Cam_Free',},
    ['TOGGLE WIDE VIEW']              = {action =  'UI_Lua import("/lua/ui/game/zoomslider.lua").ToggleWideView()', stringkey = 'reset_camera', category = '<LOC keymap_category_0025>Camera', order = 11,},

    ['<SCA_SHIFT>REPAIR']              = {action =  'StartCommandMode order RULEUCC_Repair', stringkey = 'repair', category = '<LOC keymap_category_0036>Orders', order = 1,},
    ['<SCA_SHIFT>RECLAIM']              = {action =  'StartCommandMode order RULEUCC_Reclaim', stringkey = 'reclaim', category = '<LOC keymap_category_0036>Orders', order = 2,},
    ['<SCA_SHIFT>PATROL']              = {action =  'StartCommandMode order RULEUCC_Patrol', stringkey = 'patrol', category = '<LOC keymap_category_0036>Orders', order = 3,},
    ['<SCA_SHIFT>ATTACK']              = {action =  'StartCommandMode order RULEUCC_Attack', stringkey = 'attack', category = '<LOC keymap_category_0036>Orders', order = 4,},
    ['<SCA_SHIFT>CAPTURE']              = {action =  'StartCommandMode order RULEUCC_Capture', stringkey = 'capture', category = '<LOC keymap_category_0036>Orders', order = 5,},
    ['<SCA_SHIFT>STOP']              = {action =  'IssueCommand Stop', stringkey = 'stop', category = '<LOC keymap_category_0036>Orders', order = 6,},
    ['<SCA_SHIFT>DIVE']              = {action =  'IssueCommand Dive', stringkey = 'dive', category = '<LOC keymap_category_0036>Orders', order = 7,},
    ['<SCA_SHIFT>FERRY']              = {action =  'StartCommandMode order RULEUCC_Ferry', stringkey = 'ferry', category = '<LOC keymap_category_0036>Orders', order = 8,},
    ['<SCA_SHIFT>GUARD']              = {action =  'StartCommandMode order RULEUCC_Guard', stringkey = 'guard', category = '<LOC keymap_category_0036>Orders', order = 9,},
    ['<SCA_SHIFT>TRANSPORT']              = {action =  'StartCommandMode order RULEUCC_Transport', stringkey = 'transport', category = '<LOC keymap_category_0036>Orders', order = 10,},
    ['<SCA_SHIFT>LAUNCH TACTICAL']              = {action =  'StartCommandMode order RULEUCC_Tactical', stringkey = 'launch_tactical', category = '<LOC keymap_category_0036>Orders', order = 11,},
    ['<SCA_SHIFT>OVERCHARGE']              = {action =  'UI_Lua import("/lua/ui/game/orders.lua").EnterOverchargeMode()', stringkey = 'overcharge', category = '<LOC keymap_category_0036>Orders', order = 12,},
    ['<SCA_SHIFT>MOVE']              = {action =  'StartCommandMode order RULEUCC_Move', stringkey = 'move', category = '<LOC keymap_category_0036>Orders', order = 16,},
    ['<SCA_SHIFT>LAUNCH NUKE']              = {action =  'StartCommandMode order RULEUCC_Nuke', stringkey = 'nuke', category = '<LOC keymap_category_0036>Orders', order = 28,},

    ['COMMANDMODE REPAIR']        = {action =  'StartCommandMode order RULEUCC_Repair'},
    ['COMMANDMODE RECLAIM']        = {action =  'StartCommandMode order RULEUCC_Reclaim'},
    ['COMMANDMODE PATROL']        = {action =  'StartCommandMode order RULEUCC_Patrol'},
    ['COMMANDMODE ATTACK']        = {action =  'StartCommandMode order RULEUCC_Attack'},
    ['COMMANDMODE CAPTURE']        = {action =  'StartCommandMode order RULEUCC_Capture'},
    ['COMMANDMODE STOP']        = {action =  'IssueCommand Stop'},
    ['COMMANDMODE DIVE']        = {action =  'IssueCommand Dive'},
    ['COMMANDMODE FERRY']        = {action =  'StartCommandMode order RULEUCC_Ferry'},
    ['COMMANDMODE GUARD']        = {action =  'StartCommandMode order RULEUCC_Guard'},
    ['COMMANDMODE TRANSPORT']        = {action =  'StartCommandMode order RULEUCC_Transport'},
    ['COMMANDMODE LAUNCH TACTICAL']        = {action =  'StartCommandMode order RULEUCC_Tactical'},
    ['COMMANDMODE OVERCHARGE']        = {action =  'StartCommandMode order RULEUCC_Overcharge'},
    ['COMMANDMODE MOVE']        = {action =  'StartCommandMode order RULEUCC_Move'},
    ['COMMANDMODE NUKE']        = {action =  'StartCommandMode order RULEUCC_Nuke'},

    ['TOGGLE BUILD MODE']              = {action =  'UI_Lua import("/lua/ui/game/buildmode.lua").ToggleBuildMode()', stringkey = 'toggle_build_mode', category = '<LOC keymap_category_0036>Orders', order = 15,},

    ['PAUSE UNIT']              = {action =  'UI_Lua import("/lua/ui/game/queue.lua").ToggleUnitPause()', stringkey = 'pause_unit', category = '<LOC keymap_category_0036>Orders', order = 13,},

    ['CHANGE FIRE MOD']    = {action =  'UI_Lua import("/lua/ui/game/orders.lua").CycleRetaliateStateUp()', stringkey = 'mode', category = '<LOC keymap_category_0036>Orders', order = 17,}, #cut?

    ['SUICIDE']         = {action =  'KillSelectedUnits', stringkey = 'suicide', category = '<LOC keymap_category_0036>Orders', order = 14,},

    ['DECREASE GAME SPEED']       = {action =  'WLD_DecreaseSimRate', stringkey = 'decrease_game_speed', category = '<LOC keymap_category_0079>Game', order = 1,},
    ['INCREASE GAME SPEED']        = {action =  'WLD_IncreaseSimRate', stringkey = 'increase_game_speed', category = '<LOC keymap_category_0079>Game', order = 2,},
    ['RESET GAME SPEED']        = {action =  'WLD_ResetSimRate', stringkey = 'reset_game_speed', category = '<LOC keymap_category_0079>Game', order = 3,},

    ['CHAT PAGE UP']         = {action =  'UI_Lua import("/lua/ui/game/chat.lua").ChatPageUp(10)', stringkey = 'chat_page_up', category = '<LOC keymap_category_0087>Chat', order = 2,},
    ['CHAT PAGE DOWN']       = {action =  'UI_Lua import("/lua/ui/game/chat.lua").ChatPageDown(10)', stringkey = 'chat_page_down', category = '<LOC keymap_category_0087>Chat', order = 3,},
    ['CHAT LINE UP']   = {action =  'UI_Lua import("/lua/ui/game/chat.lua").ChatPageUp(1)', stringkey = 'chat_line_up', category = '<LOC keymap_category_0087>Chat', order = 4,},
    ['CHAT LINE DOWN'] = {action =  'UI_Lua import("/lua/ui/game/chat.lua").ChatPageDown(1)', stringkey = 'chat_line_down', category = '<LOC keymap_category_0087>Chat', order = 5,},


    ['SCORE']              = {action =  'UI_Lua import("/lua/ui/game/options.lua").ToggleScore()', stringkey = 'toggle_score_screen', category = '<LOC keymap_category_0000>UI', order = 1,},
    ['OBJECTIVES']              = {action =  'UI_Lua import("/lua/ui/game/options.lua").ToggleObjectives()', stringkey = 'toggle_objective_screen', category = '<LOC keymap_category_0000>UI', order = 2,},
    ['TRANSMISSIONS']              = {action =  'UI_Lua import("/lua/ui/game/options.lua").ToggleTransmissions()', stringkey = 'toggle_transmission_screen', category = '<LOC keymap_category_0000>UI', order = 3,},
    ['DIPLOMACY']              = {action =  'UI_Lua import("/lua/ui/game/options.lua").ToggleDiplomacy()', stringkey = 'toggle_diplomacy_screen', category = '<LOC keymap_category_0000>UI', order = 4,},
    ['KEY BINDINGS']             = {action = 'UI_Lua import("/lua/ui/game/keybindings.lua").CreateUI()', stringkey = 'toggle_key_bindings', category = '<LOC keymap_category_0000>UI', order = 6,},
    ['SHOW FPS']       = {action =  'UI_Lua import("/lua/debug/uidebug.lua").ShowFPS()'},
	
    ['SHOW NETWORK STATS']       = {action =  'ren_ShowNetworkStats'},

	--mod specific commands
	['POPUP TOGGLE']	= {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").toggleMainPopup()', stringkey = 'UI_mod1', category =  '<LOC keymap_category_0000>UI', order = 12,},
	['ZOOMPOP TOGGLE'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/zoompopper.lua").ToggleZoomPop()', stringkey = 'UI_mod2', category =  '<LOC keymap_category_0000>UI', order = 12,},
	['MOVE RESIZE TOGGLE'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").MoveResizeToggle()', stringkey = 'UI_mod3', category =  '<LOC keymap_category_0000>UI', order = 12,},
	['GENERAL OPTIONS WINDOW'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").toggle_config_panel()', stringkey = 'UI_mod4', category =  '<LOC keymap_category_0000>UI', order = 12,},
	['KEY REMAPPING WINDOW'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/myeffecthelpers.lua").FadeIn(import("/mods/uimod/sca_utils/uimanager.lua").keyPanel,import("/mods/uimod/sca_utils/uimanager.lua").time_effect)', stringkey = 'UI_mod5', category =  '<LOC keymap_category_0000>UI', order = 12,},
	['STATISTIC WINDOW']={action =  'UI_Lua import("/mods/uimod/sca_utils/myeffecthelpers.lua").FadeIn(import("/mods/uimod/sca_utils/uimanager.lua").info_panel,import("/mods/uimod/sca_utils/uimanager.lua").time_effect)', stringkey = 'UI_mod5', category =  '<LOC keymap_category_0000>UI', order = 12,},
	['SHOW LIFE BARS'] = {action = 'ui_ForceLifbarsOnEnemy', stringkey = 'forcelifebars'},
	['CONSTRUCTION LOOP TOGGLE'] = {action = 'UI_Lua import("/mods/uimod/hook/lua/ui/game/queue.lua").toggleLoop()', stringkey = 'looptoggle'},
	
	--build hotkeys
	['<SCA_SHIFT_BUILD>LAND FACTORY'] = {cmd="LandFactory",des='<SCA_SHIFT_BUILD>LAND FACTORY',stringkey='bh_1'},
	['<SCA_SHIFT_BUILD>AIR FACTORY'] = {cmd="AirFactory",des='<SCA_SHIFT_BUILD>AIR FACTORY',stringkey='bh_2'},
	['<SCA_SHIFT_BUILD>NAVAL FACTORY'] = {cmd="NavalFactory",des='<SCA_SHIFT_BUILD>NAVAL FACTORY',stringkey='bh_3'},
	['<SCA_SHIFT_BUILD>MASS EXTRACTOR'] = {cmd="MassExtractor",des='<SCA_SHIFT_BUILD>MASS EXTRACTOR',stringkey='bh_4'},
	['<SCA_SHIFT_BUILD>MASS FABRICATOR'] = {cmd="MassFabricator",des='<SCA_SHIFT_BUILD>MASS FABRICATOR',stringkey='bh_5'},
	['<SCA_SHIFT_BUILD>MASS STORAGE'] = {cmd="MassStorage",des='<SCA_SHIFT_BUILD>MASS STORAGE',stringkey='bh_6'},
	['<SCA_SHIFT_BUILD>POWER GENERATOR'] = {cmd="PowerGen",des='<SCA_SHIFT_BUILD>POWER GENERATOR',stringkey='bh_7'},
	['<SCA_SHIFT_BUILD>HYDROCARBON POWERPLANT'] = {cmd="Hydrocarbon",des='<SCA_SHIFT_BUILD>HYDROCARBON POWERPLANT',stringkey='bh_8'},
	['<SCA_SHIFT_BUILD>ENERGY STORAGE'] = {cmd="EnergyStorage",des='<SCA_SHIFT_BUILD>ENERGY STORAGE',stringkey='bh_9'},
	['<SCA_SHIFT_BUILD>WALL'] = {cmd="Wall",des='<SCA_SHIFT_BUILD>WALL',stringkey='bh_10'},
	['<SCA_SHIFT_BUILD>POINT DEFENSE'] = {cmd="PointDefense",des='<SCA_SHIFT_BUILD>POINT DEFENSE',stringkey='bh_11'},
	['<SCA_SHIFT_BUILD>ANTI AIR TURRET'] = {cmd="AA",des='<SCA_SHIFT_BUILD>ANTI AIR TURRET',stringkey='bh_12'},
	['<SCA_SHIFT_BUILD>TORPEDO LAUNCHER'] = {cmd="TorpLauncher",des='<SCA_SHIFT_BUILD>TORPEDO LAUNCHER',stringkey='bh_13'},
	['<SCA_SHIFT_BUILD>RADAR'] = {cmd="Radar",des='<SCA_SHIFT_BUILD>RADAR',stringkey='bh_14'},
	['<SCA_SHIFT_BUILD>SONAR'] = {cmd="Sonar",des='<SCA_SHIFT_BUILD>SONAR',stringkey='bh_15'},
	['<SCA_SHIFT_BUILD>ARTILLERY T2'] = {cmd="T2Artillery",des='<SCA_SHIFT_BUILD>ARTILLERY T2',stringkey='bh_16'},
	['<SCA_SHIFT_BUILD>ARTILLERY T3'] = {cmd="T3Artillery",des='<SCA_SHIFT_BUILD>ARTILLERY T3',stringkey='bh_17'},
	['<SCA_SHIFT_BUILD>TACTICAL MISSILE'] = {cmd="TacticalMissile",des='<SCA_SHIFT_BUILD>TACTICAL MISSILE',stringkey='bh_18'},
	['<SCA_SHIFT_BUILD>TACTICAL DEFENSE'] = {cmd="TacticalDefense",des='<SCA_SHIFT_BUILD>TACTICAL DEFENSE',stringkey='bh_19'},
	['<SCA_SHIFT_BUILD>MISSILE SILO'] = {cmd="StrategicMissile",des='<SCA_SHIFT_BUILD>MISSILE SILO',stringkey='bh_20'},
	['<SCA_SHIFT_BUILD>MISSILE DEFENSE'] = {cmd="StratDefense",des='<SCA_SHIFT_BUILD>MISSILE DEFENSE',stringkey='bh_21'},
	['<SCA_SHIFT_BUILD>SHIELD GENERATOR'] = {cmd="ShieldGen",des='<SCA_SHIFT_BUILD>SHIELD GENERATOR',stringkey='bh_22'},
	['<SCA_SHIFT_BUILD>STEALTH GENERATOR'] = {cmd="StealthGen",des='<SCA_SHIFT_BUILD>STEALTH GENERATOR',stringkey='bh_23'},
	['<SCA_SHIFT_BUILD>AIR STAGING FACILITY'] = {cmd="AirRepair",des='<SCA_SHIFT_BUILD>AIR STAGING FACILITY',stringkey='bh_24'},
	['<SCA_SHIFT_BUILD>QUANTUM GATE'] = {cmd="QuantumGate",des='<SCA_SHIFT_BUILD>QUANTUM GATE',stringkey='bh_25'},
	['<SCA_SHIFT_BUILD>EXPERIMENTAL 1'] = {cmd="Experimental1",des='<SCA_SHIFT_BUILD>EXPERIMENTAL 1',stringkey='bh_26'},
	['<SCA_SHIFT_BUILD>EXPERIMENTAL 2'] = {cmd="Experimental2",des='<SCA_SHIFT_BUILD>EXPERIMENTAL 2',stringkey='bh_27'},
	['<SCA_SHIFT_BUILD>EXPERIMENTAL 3'] = {cmd="Experimental3",des='<SCA_SHIFT_BUILD>EXPERIMENTAL 3',stringkey='bh_28'},
	['<SCA_SHIFT_UNIT>UPGRADE'] = {tech=-1,cmd="Upgrade",typ=1,des='<SCA_SHIFT_UNIT>UPGRADE',stringkey='bh_29'},

	--build units hotkeys
	['<SCA_SHIFT_UNIT>ENGINEER/SUB COMMANDER'] = {cmd="Engineer",des='<SCA_SHIFT_UNIT>ENGINEER/SUB COMMANDER',stringkey='bh_30'},
	['<SCA_SHIFT_UNIT>BOT/BOMBER'] = {cmd="BotBomb",des='<SCA_SHIFT_UNIT>BOT/BOMBER',stringkey='bh_31'},
	['<SCA_SHIFT_UNIT>MOBILE ARTILLERY/BATTLESHIP'] = {cmd="Artillery",des='<SCA_SHIFT_UNIT>MOBILE ARTILLERY/BATTLESHIP',stringkey='bh_32'},
	['<SCA_SHIFT_UNIT>TANK/GUNSHIP/DESTROYER'] = {cmd="Tanks",des='<SCA_SHIFT_UNIT>TANK/GUNSHIP/DESTROYER',stringkey='bh_33'},
	['<SCA_SHIFT_UNIT>MOBILE MISSILE LAUNCHER/CRUISER'] = {cmd="Missile",des='<SCA_SHIFT_UNIT>MOBILE MISSILE LAUNCHER/CRUISER',stringkey='bh_34'},
	['<SCA_SHIFT_UNIT>AMPHIBIOUS TANK/TRANSPORT'] = {cmd="TransAmph",des='<SCA_SHIFT_UNIT>AMPHIBIOUS TANK/TRANSPORT',stringkey='bh_35'},
	['<SCA_SHIFT_UNIT>MOBILE SHIELD/STEALTH GENERATOR'] = {cmd="ShieldStealth",des='<SCA_SHIFT_UNIT>MOBILE SHIELD/STEALTH GENERATOR',stringkey='bh_36'},
	['<SCA_SHIFT_UNIT>SCOUT/FRIGATE'] = {cmd="Scouts",des='<SCA_SHIFT_UNIT>SCOUT/FRIGATE',stringkey='bh_37'},
	['<SCA_SHIFT_UNIT>FIGHTER/MOBILE AA/AA SHIP'] = {cmd="AntiAir",des='<SCA_SHIFT_UNIT>FIGHTER/MOBILE AA/AA SHIP',stringkey='bh_38'},
	['<SCA_SHIFT_UNIT>TORPEDO BOMBER/SUB'] = {cmd="Torpedo",des='<SCA_SHIFT_UNIT>TORPEDO BOMBER/SUB',stringkey='bh_39'},
	['<SCA_SHIFT_UNIT>CARRIER'] = {cmd="Carrier",des='<SCA_SHIFT_UNIT>CARRIER',stringkey='bh_40'},
	
	--grid hotkeys
	['<SCA_SHIFT>SLOT 1'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(1)', stringkey = 'buildpos1'},
	['<SCA_SHIFT>SLOT 2'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(2)', stringkey = 'buildpos2'},
	['<SCA_SHIFT>SLOT 3'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(3)', stringkey = 'buildpos3'},
	['<SCA_SHIFT>SLOT 4'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(4)', stringkey = 'buildpos4'},
	['<SCA_SHIFT>SLOT 5'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(5)', stringkey = 'buildpos5'},
	['<SCA_SHIFT>SLOT 6'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(6)', stringkey = 'buildpos6'},
	['<SCA_SHIFT>SLOT 7'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(7)', stringkey = 'buildpos7'},
	['<SCA_SHIFT>SLOT 8'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(8)', stringkey = 'buildpos8'},
	['<SCA_SHIFT>SLOT 9'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(9)', stringkey = 'buildpos9'},
	['<SCA_SHIFT>SLOT 10'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(10)', stringkey = 'buildpos10'},
	['<SCA_SHIFT>SLOT 11'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(11)', stringkey = 'buildpos11'},
	['<SCA_SHIFT>SLOT 12'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(12)', stringkey = 'buildpos12'},
	['<SCA_SHIFT>SLOT 13'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(13)', stringkey = 'buildpos13'},
	['<SCA_SHIFT>SLOT 14'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(14)', stringkey = 'buildpos14'},
	['<SCA_SHIFT>SLOT 15'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(15)', stringkey = 'buildpos15'},
	['<SCA_SHIFT>SLOT 16'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(16)', stringkey = 'buildpos16'},
	['<SCA_SHIFT>SLOT 17'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(17)', stringkey = 'buildpos17'},
	['<SCA_SHIFT>SLOT 18'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(18)', stringkey = 'buildpos18'},
	['<SCA_SHIFT>SLOT 19'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(19)', stringkey = 'buildpos19'},
	['<SCA_SHIFT>SLOT 20'] = {action =  'UI_Lua import("/mods/uimod/sca_utils/uimanager.lua").buildme(20)', stringkey = 'buildpos20'},

}

--List of all Commands
BuildCommands = {		
	'<SCA_CATEGORY>BUILDINGS HOTKEYS',
	
	'<SCA_SHIFT_BUILD>LAND FACTORY',
	'<SCA_SHIFT_BUILD>AIR FACTORY',
	'<SCA_SHIFT_BUILD>NAVAL FACTORY',
	'<SCA_SHIFT_BUILD>MASS EXTRACTOR',
	'<SCA_SHIFT_BUILD>MASS FABRICATOR',
	'<SCA_SHIFT_BUILD>MASS STORAGE',
	'<SCA_SHIFT_BUILD>POWER GENERATOR',
	'<SCA_SHIFT_BUILD>HYDROCARBON POWERPLANT',
	'<SCA_SHIFT_BUILD>ENERGY STORAGE',
	'<SCA_SHIFT_BUILD>WALL',
	'<SCA_SHIFT_BUILD>POINT DEFENSE',
	'<SCA_SHIFT_BUILD>ANTI AIR TURRET',
	'<SCA_SHIFT_BUILD>TORPEDO LAUNCHER',
	'<SCA_SHIFT_BUILD>RADAR',
	'<SCA_SHIFT_BUILD>SONAR',
	'<SCA_SHIFT_BUILD>ARTILLERY T2',
	'<SCA_SHIFT_BUILD>ARTILLERY T3',
	'<SCA_SHIFT_BUILD>TACTICAL MISSILE',
	'<SCA_SHIFT_BUILD>TACTICAL DEFENSE',
	'<SCA_SHIFT_BUILD>MISSILE SILO',
	'<SCA_SHIFT_BUILD>MISSILE DEFENSE',
	'<SCA_SHIFT_BUILD>SHIELD GENERATOR',
	'<SCA_SHIFT_BUILD>STEALTH GENERATOR',
	'<SCA_SHIFT_BUILD>AIR STAGING FACILITY',
	'<SCA_SHIFT_BUILD>QUANTUM GATE',
	'<SCA_SHIFT_BUILD>EXPERIMENTAL 1',
	'<SCA_SHIFT_BUILD>EXPERIMENTAL 2',
	'<SCA_SHIFT_BUILD>EXPERIMENTAL 3',
	
	

	'<SCA_CATEGORY>',
	'<SCA_CATEGORY>UNITS HOTKEYS',
	
	'<SCA_SHIFT_UNIT>UPGRADE',
	'<SCA_SHIFT_UNIT>ENGINEER/SUB COMMANDER',
	'<SCA_SHIFT_UNIT>SCOUT/FRIGATE',
	'<SCA_SHIFT_UNIT>BOT/BOMBER',
	'<SCA_SHIFT_UNIT>TANK/GUNSHIP/DESTROYER',
	'<SCA_SHIFT_UNIT>MOBILE ARTILLERY/BATTLESHIP',
	'<SCA_SHIFT_UNIT>FIGHTER/MOBILE AA/AA SHIP',
	'<SCA_SHIFT_UNIT>MOBILE MISSILE LAUNCHER/CRUISER',
	'<SCA_SHIFT_UNIT>MOBILE SHIELD/STEALTH GENERATOR',
	'<SCA_SHIFT_UNIT>AMPHIBIOUS TANK/TRANSPORT',
	'<SCA_SHIFT_UNIT>TORPEDO BOMBER/SUB',
	'<SCA_SHIFT_UNIT>CARRIER',
	
	'<SCA_CATEGORY>',
	'<SCA_CATEGORY>CONSTRUCTION SLOTS',
	
	'<SCA_SHIFT>SLOT 1',
	'<SCA_SHIFT>SLOT 2',
	'<SCA_SHIFT>SLOT 3',
	'<SCA_SHIFT>SLOT 4',
	'<SCA_SHIFT>SLOT 5',
	'<SCA_SHIFT>SLOT 6',
	'<SCA_SHIFT>SLOT 7',
	'<SCA_SHIFT>SLOT 8',
	'<SCA_SHIFT>SLOT 9',
	'<SCA_SHIFT>SLOT 10',
	'<SCA_SHIFT>SLOT 11',
	'<SCA_SHIFT>SLOT 12',
	'<SCA_SHIFT>SLOT 13',
	'<SCA_SHIFT>SLOT 14',
	'<SCA_SHIFT>SLOT 15',
	'<SCA_SHIFT>SLOT 16',
	'<SCA_SHIFT>SLOT 17',
	'<SCA_SHIFT>SLOT 18',
	'<SCA_SHIFT>SLOT 19',
	'<SCA_SHIFT>SLOT 20',
}

GeneralCommands = {
	'<SCA_CATEGORY>UI MOD',
	--mod specific commands
	'POPUP TOGGLE',
	'ZOOMPOP TOGGLE',
	'MOVE RESIZE TOGGLE',
	'GENERAL OPTIONS WINDOW',
	'KEY REMAPPING WINDOW',
	'STATISTIC WINDOW',
	'SHOW LIFE BARS',
	'CONSTRUCTION LOOP TOGGLE',
	
	'<SCA_CATEGORY>',
	'<SCA_CATEGORY>ORDERS',
	
    '<SCA_SHIFT>ATTACK',	
    '<SCA_SHIFT>MOVE',
	'<SCA_SHIFT>REPAIR',
    '<SCA_SHIFT>RECLAIM',
    '<SCA_SHIFT>PATROL',
    '<SCA_SHIFT>CAPTURE',
    '<SCA_SHIFT>STOP',
    '<SCA_SHIFT>DIVE',
    '<SCA_SHIFT>FERRY',
    '<SCA_SHIFT>GUARD',
    '<SCA_SHIFT>TRANSPORT',
    '<SCA_SHIFT>LAUNCH TACTICAL',
    '<SCA_SHIFT>OVERCHARGE',
    '<SCA_SHIFT>LAUNCH NUKE',
	
	'<SCA_CATEGORY>',--skip line
	'<SCA_CATEGORY>OPTIONS',
	
	'TOGGLE BUILD MODE',
    'PAUSE UNIT',
    'CHANGE FIRE MOD',
    'SUICIDE',
	
	'SCORE',
    'OBJECTIVES',
    'TRANSMISSIONS',
    'DIPLOMACY',
    'SHOW FPS',
	'SHOW NETWORK STATS',
    'OPTIONS',
    'HIDEUI',
    'PAUSE',
    'SCREENSHOT',
    'LIFEBARS',
    'MILITARY OVERLAY',
    'DEFENSE OVERLAY',
    'ECONOMY OVERLAY',
    'INTEL OVERLAY',
    'RENAME',
    'SPLIT SCREEN',
    'UNSPLIT SCREEN',
    'SWITCH LAYOUT UP',
    'SWITCH LAYOUT DOWN',
    'CONNECTIVITY',
	
	'RECALL CAMERA POSITION',
    'SAVE CAMERA POSITION',
    'REMOVE CAMERA POSITION',
    'ZOOM IN',
    'ZOOM OUT',
    'ZOOM IN FAST',
    'ZOOM OUT FAST',
    'TRACK UNIT',
    'TRACK UNIT ON MINIMAP',
    'TRACK UNIT ON SECOND MONITOR',
    'FREE CAMERA',
    'TOGGLE WIDE VIEW',
	
    'DECREASE GAME SPEED',
    'INCREASE GAME SPEED',
    'RESET GAME SPEED',

    'CHAT PAGE UP',
    'CHAT PAGE DOWN',
    'CHAT LINE UP',
    'CHAT LINE DOWN',	
	
    'KEY BINDINGS',
}

SelectionCommands = {
	'<SCA_CATEGORY>SELECTION',
	
    'GROUP 1',
    'GROUP 2',
    'GROUP 3',
    'GROUP 4',
    'GROUP 5',
    'GROUP 6',
    'GROUP 7',
    'GROUP 8',
    'GROUP 9',
    'GROUP 0',
    'SET GROUP 1',
    'SET GROUP 2',
    'SET GROUP 3',
    'SET GROUP 4',
    'SET GROUP 5',
    'SET GROUP 6',
    'SET GROUP 7',
    'SET GROUP 8',
    'SET GROUP 9',
    'SET GROUP 0',
    'SELECT AIR',
    'SELECT NAVAL',
    'SELECT LAND',
    'SELECT SAME TYPE',
    'SELECT ENGINEERS',
    'GOTO ENGINEER',
    'SELECT IDLE ENGINEER',
    'CYCLE ENGINEERS',
    'VIEW COMMANDER',
    'SELECT COMMANDER',
    'SELECT ALL',
    'SELECT ALL ON SCREEN',
    'SELECT ALL ENGINEERS ON SCREEN',
    'SELECT ALL FACTORIES ON SCREEN',
    'SELECT NEAREST FACTORY',
    'SELECT NEAREST LAND FACTORY',
    'SELECT NEAREST AIR FACTORY',
    'SELECT NEAREST NAVAL FACTORY',
    'SELECT NEXT LAND FACTORY',
    'SELECT NEXT AIR FACTORY',
    'SELECT NEXT NAVAL FACTORY',
    'SELECT SIMILAR UNITS'
}

--List of all number-key equivalents
KeyEquivalents = {
 
    ['9'] = 'Tab',
    ['27'] = 'Esc',
	['32'] = 'Space',
	['39'] = 'Square',

    ['48'] = '0',
    ['49'] = '1',
    ['50'] = '2',
    ['51'] = '3',
    ['52'] = '4',
    ['53'] = '5',
    ['54'] = '6',
    ['55'] = '7',
    ['56'] = '8',
    ['57'] = '9',

    ['65'] = 'A',
    ['66'] = 'B',
    ['67'] = 'C',
    ['68'] = 'D',
    ['69'] = 'E',
    ['70'] = 'F',
    ['71'] = 'G',
    ['72'] = 'H',
    ['73'] = 'I',
    ['74'] = 'J',
    ['75'] = 'K',
    ['76'] = 'L',
    ['77'] = 'M',
    ['78'] = 'N',
    ['79'] = 'O',
    ['80'] = 'P',
    ['81'] = 'Q',
    ['82'] = 'R',
    ['83'] = 'S',
    ['84'] = 'T',
    ['85'] = 'U',
    ['86'] = 'V',
    ['87'] = 'W',
    ['88'] = 'X',
    ['89'] = 'Y',
    ['90'] = 'Z',	
	
    ['39'] = 'Quote',
	--['43'] = 'Plus',
    ['44'] = 'Comma',
    ['45'] = 'Minus',
    ['46'] = 'Period',
    ['47'] = 'Slash',
	['59'] = 'Semicolon',
    ['61'] = 'Equals',
	['91'] = 'LeftBracket',
	['92'] = 'Backslash',
    ['93'] = 'RightBracket',
    ['126'] = 'Tilde',
	['127'] = 'Del',
	
	['166'] = 'BrowserBack',
    ['167'] = 'BrowserForward',
    ['168'] = 'BrowserRefresh',
    ['169'] = 'BrowserStop',
    ['170'] = 'BrowserSearch',
    ['171'] = 'BrowserFavorites',
    ['172'] = 'BrowserHome',
    ['173'] = 'VolumeMute',
    ['174'] = 'VolumeDown',
    ['175'] = 'VolumeUp',
    ['176'] = 'MediaNextTrack',
    ['177'] = 'MediaPrevTrack',
    ['178'] = 'MediaStop',
    ['179'] = 'MediaPlayPause',
    ['180'] = 'LaunchMail',
    ['181'] = 'LaunchMediaSelect',
	--['182'] = 'Application1',
    --['183'] = 'Application2',
	
    ['342'] = 'F1',
    ['343'] = 'F2',
    ['344'] = 'F3',
    ['345'] = 'F4',
	['346'] = 'F5',
	['347'] = 'F6',
    ['348'] = 'F7',
    ['349'] = 'F8',
    ['350'] = 'F9',
	['351'] = 'F10',
	['352'] = 'F11',
    ['353'] = 'F12',
		
	['310'] = 'Pause',
    ['312'] = 'Pageup',
    ['313'] = 'Pagedown',	
    ['314'] = 'End',
	['315'] = 'Home',
	['324'] = 'Inser',
	
	['316'] = 'LeftArrow',
	['317'] = 'UpArrow',
	['318'] = 'RightArrow',
	['319'] = 'DownArrow',

	['326'] = 'Num0',
    ['327'] = 'Num1',
    ['328'] = 'Num2',
    ['329'] = 'Num3',
    ['330'] = 'Num4',
    ['331'] = 'Num5',
    ['332'] = 'Num6',
    ['333'] = 'Num7',
    ['334'] = 'Num8',
    ['335'] = 'Num9',
	
	['391'] = 'NumStar',
    ['392'] = 'NumPlus',
    ['394'] = 'NumMinus',
	['395'] = 'NumPeriod',
    ['396'] = 'NumSlash',
	
}


--List of all keys
Keys = {}

--List of all hotkey equivalents
HotkeyEquivalents = {}

--List of all results-resultList equivalents
returnIndex = {}

local function PlayErrorSound()
        local sound = Sound({Cue = 'UI_Menu_Error_01', Bank = 'Interface',})
        PlaySound(sound)
end

--init, builds key table using KeyEquivalents and remaps the saved keys
function init()
	loadKeymaps()
	--makes key table using key equivalents with all cases using modifiers
	for key, value in KeyEquivalents do
		if original[value] == nil then
			for kc,c in  {"","Ctrl-"} do
				for ks,s in {"","Shift-"} do
					for ka,a in {"","Alt-"} do
						local name = c..s..a..value
						table.insert(Keys, name)
						--IN_SetKeyName(key,value) 
					end
				end
			end
		end
	end
	
	load()	
	remapKey('Shift-Tab','POPUP TOGGLE')
	remapKey('Tab','ZOOMPOP TOGGLE')
	Confirm()
end

--replaces the default key map by a new keyMap
function replaceDefault(keyMapTable)
	IN_RemoveKeyMapTable(default) 
	IN_AddKeyMapTable(keyMapTable)
end

	
		
--remaps a key
function remapKey(key,command)
	--LOG("remapping: " .. command .. " / " .. key .. " ...")
	
	if string.find(command,'<SCA_SHIFT_')!= nil then
		temp[key] = makeHotkeyEquivalent(command,key)
	else
		temp[key] = CommandEquivalents[command]
	end
	
	--LOG(key)
	
	--categories
	-- if string.find(command,'SCA_SHIFT')!= nil and string.find(key,'Shift-') == nil then
		-- local categoryKey = "Shift-" .. key
		-- if string.find(command,'<SCA_SHIFT_BUILD>')!= nil then
			-- temp[categoryKey ] = makeHotkeyEquivalent(command)
		-- else
			-- temp[categoryKey ] = CommandEquivalents[command]
		-- end	
		-- LOG("<SCA_SHIFT> category detected, remapped SHIFT+KEY too: "..key.." => "..categoryKey)
	-- end
	
	--LOG("done...")

end		

--takes the last saved value for the key
function unmapKey(key)
	temp[key] = scaKeymap[key]
end

--resets temp
function Cancel()
	temp = copy(scaKeymap)
end

--loads the new table
function Confirm()	
	scaKeymap = copy(temp)
	replaceDefault(scaKeymap)
end

--remaps a key and loads the new table 
function remapAndConfirm(key,command)
	remapKey(key,command)
	Confirm()	
end


--restore default values
function restoreDefault()
	temp = copy(original)
	remapKey('Shift-Tab','POPUP TOGGLE')
	remapKey('Tab','ZOOMPOP TOGGLE')
end

--reset values
function reset()
	temp = copy(original)
	remapKey('Shift-Tab','POPUP TOGGLE')
	remapKey('Tab','ZOOMPOP TOGGLE')
	Confirm()
end

--save differences between results and original in prefs
function save()
	-- local savedTab = {}
	-- for key, value in scaKeymap do
		-- if scaKeymap[key] != original[key] then
			-- savedTab[key] = value
		-- end
	-- end
	-- Prefs.SetToCurrentProfile("sca_key_map" , savedTab)
	-- Prefs.SavePreferences()
	
	Prefs.SetToCurrentProfile("sca_key_map" , UIManager.results)
	Prefs.SavePreferences()
end

--load and remap the saved keymap
function load()
	remapResultTable(getSavedResults(),false)	
end






--
--        FUNCTIONS TO MANAGE THE KEY MAP PANEL LISTS
--


--refreshes a listbox to show the results in the right format
function refresh_result(listBox,tableau)
	local ret={}
	listBox.clear()
  if  tableau!=nil  and  table.getsize(tableau) > 0 and listBox!=nil then
	local i=0
	local ord={}
	for name,value in tableau do
		local key=value.key
		if (value.category=="<SCA_SHIFT_BUILD>" or value.category=="<SCA_SHIFT_UNIT>")  and string.find(value.key,'Shift-')!=nil then
			key=""
		end
		--if (value.category=="<SCA_SHIFT_BUILD" or value.category=="<SCA_SHIFT_UNIT>")  and string.find(value.key,'Shift-')==nil then
		--	key=key.." (+Shift)"
		--end
		if (value.category=="<SCA_SHIFT_BUILD>" )  and string.find(value.key,'Shift-')==nil then
			key=key.." Build"
		end
		if (value.category=="<SCA_SHIFT_UNIT>")  and string.find(value.key,'Shift-')==nil then
			key=key.." Unit"
		end		
		value["index"]=name
		if key!="" and key!=nil then
			if ord[key] == nil then ord[key] = {} end
			table.insert(ord[key],value) 
		end
		i=i+1
	end	
	--LOG(repr(ord))
	--table.sort (ord)
	--LOG("-------------------------------")
		--LOG(repr(ord))

	for name,value in ord do
		local i=1
	   if (table.getsize(value) > 1) then
		for n,v in value do
			if i==1 then 
				if v.category=="<SCA_SHIFT_BUILD>" then
					listBox.addElement("<SCA_CATEGORY>"..v.key.." + Shift-"..v.key)--name.." : "..v.command)
					listBox.setPosition2(listBox.p+1)
					table.insert(ret,0)
				elseif v.category=="<SCA_SHIFT_UNIT>" then
					listBox.addElement("<SCA_CATEGORY>"..v.key.." + Shift-"..v.key)--name.." : "..v.command)
					listBox.setPosition2(listBox.p+1)
					table.insert(ret,0)
				end
				
			end
				--local j=0
				--local space=""
				--[[while j<string.len(name) do
					space=space.."  "
					j=j+1
				end]]
				if v.category=="<SCA_SHIFT_UNIT>" then
					listBox.addElement("<BMP_buildqueue.dds>".."."..v.command)
					table.insert(ret,v.index)
				else
					listBox.addElement("<BMP_cycle.dds>"..i.."."..v.command)
					table.insert(ret,v.index)
				end
				listBox.setPosition2(listBox.p+1)
			i=i+1
		end
	   else
		if (value[1].category=="<SCA_SHIFT_BUILD>" or value[1].category=="<SCA_SHIFT_UNIT>") then
			listBox.addElement("<SCA_CATEGORY>"..name.." + Shift-"..name)
			table.insert(ret,0)
		else
			listBox.addElement("<SCA_CATEGORY>"..name)--name.." : "..v.command)
			table.insert(ret,0)
		end
		listBox.setPosition2(listBox.p+1)
		listBox.addElement(value[1].command)
		table.insert(ret,value[1].index)
		listBox.setPosition2(listBox.p+1)
	   end
	end
	listBox.Refresh(true)
 end
	returnIndex = ret
end

--make the command list
function makeCommandTab()
	local gentab = makeGeneralTab()
	local buildtab = makeBuildTab()
	local selectiontab = makeSelectionTab()
	return gentab, buildtab, selectiontab
end

function makeGeneralTab()
	local currentKeyMap = table.copy(scaKeymap)
	local sortedcommands = table.copy(GeneralCommands)
	local processedKeyMap = {}
	
	for key, value in currentKeyMap do
		for key2, value2  in CommandEquivalents do
			if value.stringkey ==  value2.stringkey then
				processedKeyMap[key] = key2
			end
		end
	end			
	
	for key, value in processedKeyMap do
		for key2, value2 in sortedcommands do
			if value2 == value then
				sortedcommands[key2]= "" .. value .. ": " .. key .. ""
			end
		end
	end
	return sortedcommands
end

function makeBuildTab()
	local currentKeyMap = table.copy(scaKeymap)
	local sortedcommands = table.copy(BuildCommands)
	local processedKeyMap = {}
	
	for key, value in currentKeyMap do
		for key2, value2  in CommandEquivalents do
			if value.stringkey ==  value2.stringkey then
				processedKeyMap[key] = key2
			end
		end
	end			
	
	for key, value in processedKeyMap do
		for key2, value2 in sortedcommands do
			if value2 == value then
				sortedcommands[key2]= "" .. value .. ": " .. key .. ""
			end
		end
	end
	return sortedcommands
end

function makeSelectionTab()
	local currentKeyMap = table.copy(scaKeymap)
	local sortedcommands = table.copy(SelectionCommands)
	local processedKeyMap = {}
	
	for key, value in currentKeyMap do
		for key2, value2  in CommandEquivalents do
			if value.stringkey ==  value2.stringkey then
				processedKeyMap[key] = key2
			end
		end
	end			
	
	for key, value in processedKeyMap do
		for key2, value2 in sortedcommands do
			if value2 == value then
				sortedcommands[key2]= "" .. value .. ": " .. key .. ""
			end
		end
	end
	return sortedcommands
end

--creates a table using the UIManager.resultList values
function interpretResults()
	local properList = table.copy(UIManager.resultList.getValues())	
	local rkey = {}
	local rvalue = {}
	local finalList = {}
	
	for key, value in properList do
		local mykey = string.gsub( value, "^(.*): (.*)$", "%2" )
		local myvalue = string.gsub( value, "^(.*): (.*)$", "%1" )
		table.insert(rkey ,  mykey)
		table.insert(rvalue , myvalue)
	end
	for key, value in rkey do
		finalList[rkey[key]] = rvalue[key]
	end
	return finalList,rkey,rvalue
end
	

--returns a table containing the saved keys formatted correctly
function getSavedResults() 
    local loadedKeys = {}
	--WARN("LOADEDKEYS")
	--WARN(repr(loadedKeys))
	if Prefs.GetFromCurrentProfile("sca_key_map") != nil then
		loadedKeys = table.copy(Prefs.GetFromCurrentProfile("sca_key_map"))
	end
	return loadedKeys
end

--remaps the keys using the tables contained in UIManager.results
function remapResultTable(results,saveKeys)
	--result: {command=String, key=String,category=String}
	local dosave = true
	if saveKeys != nil then
		dosave =saveKeys
	end
	
	local command = ""
	for i,v in results do
		if v.category and v.command != nil then
			command = v.category .. v.command
			remapKey(v.key,command)
			--LOG(v.key,command)
		end
	end
	Confirm()	
	
	if dosave == true then
		save()
	end
	
	--TEST
	HotkeyEquivalents = {}
end

--given a key and a command interpret key removes and adds elements to the result list
--ismodify tells us wether we are doing a modification of an existing key or adding a new result
function interpretKey(event,command,ismodify)
	local leaveChoice = false	
	local properKeyCode = "" .. event.KeyCode  .. ""		
	local keyName = ""	
	local modifiers = ""	
	local finalKeyCode = ""
	
	if event.Modifiers.Ctrl then
		modifiers = modifiers .. "Ctrl-"
	end
	if event.Modifiers.Shift then
		modifiers = modifiers .. "Shift-"
	end
	if event.Modifiers.Alt then
		modifiers = modifiers .. "Alt-"
	end
	
	finalKeyCode = modifiers

	
	if KeyEquivalents[properKeyCode] != nil then
		keyName = KeyEquivalents[properKeyCode]
	else
		if debug_sca == true then
		LOG("key undefined")
		end
	end
	
	finalKeyCode = modifiers .. keyName
	
	--if you pressed Esc it cancels the key choice
	if finalKeyCode == 'Esc' then
		--close keychoice panel
		leaveChoice = true
		
	--if you are pressing other keys than just modifiers (ctrl,shift or alt) it does the remapping		
	elseif finalKeyCode != modifiers and finalKeyCode != 'Shift-Tab' then 
		
		if ismodify == true then
		--modify selected result
		modifyResult(ismodify,finalKeyCode)		
		
		else
		--add result
		addResult(finalKeyCode,command)
		end
		
		leaveChoice = true	
		
	else
		if debug_sca == true then
		LOG("failed... key registered but not with modifiers")
		end
		leaveChoice = false
	end
	
	--if leaveChoice is false do not close the panel
	return leaveChoice
	

end
	
	
--add a result to the results 	
function addResult(key,command)
	local add = false
	local processedKey = ''
	--remove commands corresponding to the same key if necessary
	add = removeResult(key,command)
	--if these commands have been deleted
	if add == true then 
		--add the command to the list
		UIManager.resultList.addResult(key,command)
		--if the element belongs to the SHIFT category
		if string.find(command,'<SCA_SHIFT')!=nil then
			--if the key does not contain shift add result( shift-key,command)
			if string.find(key,'Shift-')!=nil then
				processedKey = 'Shift-'..key
				UIManager.resultList.addResult(processedKey,command)
			--if it contains shift- add result (key minus Shift-,command)
			else
				local left = string.gsub(key,'^(.*)Shift-(.*)$',"%1")
				local right = string.gsub(key,'^(.*)Shift-(.*)$',"%2")
				processedKey = left .. right
				UIManager.resultList.addResult('Shift-'..key,command)
			end
		end
	end	
	return add
end

--removes  none to several results from the result table depending on the key and the command	
function removeResult(key,command)
	local deleted = true
	--look at all the results table
	for i,v in UIManager.resultList.results() do
		--if the key is equal to the table value key 
		if v.key == key then		
			--if no category simply delete the key
			if v.category == '' then
				UIManager.resultList.removeResult(i)
			--if the category is <SCA_SHIFT> delete both key and Shift-key
			elseif v.category == '<SCA_SHIFT>' then
				--delete key
				UIManager.resultList.removeResult(i)
				--delete shift-key
				for i2,v2 in UIManager.resultList.results() do
					if v2.key == 'Shift-'..key then
						UIManager.resultList.removeResult(i2)
					end
				end	
			--if the new command is not a build hotkey
			elseif string.find(command,'<SCA_SHIFT_')==nil then
				--if the category is <SCA_SHIFT_UNIT>or <SCA_SHIFT_BUILD>delete all entries
				if v.category == '<SCA_SHIFT_BUILD>' or v.category == '<SCA_SHIFT_UNIT>' then
					--show a warning window and ask if the key entries should be deleted
					deleted = false
					UIManager.resultList.overwrite(key,command)
				end
			end
		end
	end
	--return true if the elements have been removed
	return deleted
end

---if the key entries have to be deleted delete all entries containing the right category
function emptyKeyEntries(key)
	local removelist = {}
	for i,v in UIManager.resultList.results() do
		if v.key == key and (v.category != '<SCA_SHIFT_BUILD>' or v.category != '<SCA_SHIFT_UNIT>') then
			table.insert(removelist, i)
		end
	end
	if table.getn(removelist) > 0 then
		for i = table.getn(removelist), 1, -1 do
			UIManager.resultList.removeResult(removelist[i])
		end
	end
end

--modify a result
function modifyResult(ismodify,key,command)
	local equIndex = returnIndex[UIManager.resultList.r]
	UIManager.resultList.results()[equIndex].key = key
	if string.find(UIManager.resultList.results()[equIndex].category,'SCA_SHIFT')!=nil then
		UIManager.resultList.results()[equIndex+1].key = 'Shift-'..key
	end
end


--
--       BUILD HOTKEYS FUNCTIONS
--

--builds a hotkey equivalent given a command using the command equivalents: USE ON HOTKEYS ONLY
function makeHotkeyEquivalent(command,key)
	
	--a hotkey is a special table containing a tech level a cmd, a unit type (unit or building), a description and an id(stringkey).
	local hotkey = CommandEquivalents[command]
	
	--old params contains the hotkeys already assigned to the key
	local oldparams = ""
	if HotkeyEquivalents[key] != nil then
		oldparams = HotkeyEquivalents[key]
	end	
	if hotkey==nil then return end
	--parameters is the good syntax of the table inside the table parameter for buildhotkey
	local parameters = "{cmd='"..hotkey.cmd.."',des='"..hotkey.des.. "',key='"..key.. "'},"
	
	--new param is the concatanation of the current hotkey and the old hotkeys
	local newparams = ""
	if string.find(oldparams,hotkey.des)!=nil then
		newparams = oldparams.. parameters
	else
		newparams = oldparams .. parameters
	end	
	
	--buildAction creates the right syntax for a key action using the newparams 
	local buildAction = 'UI_Lua import("/mods/uimod/sca_utils/remap.lua").buildhotkey('
	buildAction = buildAction .. '{'.. newparams .. '}' .. ')'
	
	--LOG("params: ".. parameters)
	--LOG("oldparams: " .. oldparams)
	--LOG("newparams: ".. newparams)
	--LOG("buildAction: ".. buildAction)
	
	--hotkey equivalent is the final table compatible with the defaultKeymap
	local hotkeyEquivalent = { action = buildAction ,stringkey = hotkey.stringkey , category =  '<LOC keymap_category_0000>UI', order = 12,}
	
	--save new params so we can access oldparams later on
	HotkeyEquivalents[key] = newparams
	
	--LOG(repr(HotkeyEquivalents))
	
	--return the hotkeyEquivalent
	return hotkeyEquivalent
end		


function  apply_exceptions(list_cmd)
		local cmd=list_cmd.cmd
		local selection=GetSelectedUnits()
		local availableOrders,  availableToggles, buildableCategories = GetUnitCommandData(selection)
		local buildable = EntityCategoryGetUnitList(buildableCategories)
		
		--check if the current selection can actually build the cmd
		if cmd != nil and cmd != false then
			if cmd == "Upgrade" then
				cmd = buildingTab[cmd]
				for i, id in cmd do
					for i2, id2 in buildable do
						if id == id2 then
							return id, true
						end
					end
				end
				return nil
			else
				cmd = buildingTab[cmd]
				for i, id in cmd do
					for i2, id2 in buildable do
						if id == id2 then
							return id
						end
					end
				end
				return nil
			end
		end
end


function cycle_engineer(list)
	local i=0

	cycle_time=cycle_time+1
	
	--LOG("}}}}}}}}}",repr(list[1].key))
	if key_sav!=list[1].key and cycle_on and cycle_map then cycle_on=false cycle_time=0 LOG("ooooooooooooooo") end
	key_sav=list[1].key
	
	if cycle_on and cycle_map then
		if cycle_time>imax then cycle_time=0 end
		local i=0
		for n,v in button do
		  if i<=imax then	
			if i==cycle_time then
				button[i]:SetAlpha(1, true)
			else
				button[i]:SetAlpha(0.3, true)
			end
			if cycle_map then KillThread(cycle_map.mdThread)  cycle_map:SetNeedsFrameUpdate(false) cycle_map.OnFrame = function(self, elapsedTime) end cycle_map:SetAlpha(1) end
			cycle_map:Show()
			cycle_map.mdThread = ForkThread(function()
				WaitSeconds(.6)
				if not cycle_map:IsHidden() then 
					Effect.FadeOut(cycle_map,.5,.6,0.1)
					WaitSeconds(.3)
				end
				cycle_on=false
			end)
			local cmd=cmd_sav[cycle_time]

			if cmd!=false and cmd!=nil then
				import('/lua/ui/game/commandmode.lua').StartCommandMode("build", {name = cmd})
				if i>0 then 
					button[cycle_time]:SetAlpha(1, true)
					cycle_map:Show()
				end
			end
			i=i+1
		  end
		end	
		cycle_map:Show()
	return
	else cycle_on=true cycle_time=0 end
	
	if cycle_map then KillThread(cycle_map.mdThread)   cycle_map:SetNeedsFrameUpdate(false) cycle_map.OnFrame = function(self, elapsedTime) end  cycle_map:Hide() cycle_map:Destroy() end

	cycle_map = Group(GetFrame(0))
	--LayoutHelpers.AtLeftTopIn(cycle_map,parent)
	cycle_map.Depth:Set(1000) --always on top
	cycle_map.Width:Set(400)
	cycle_map.Height:Set(150)
	cycle_map.Top:Set(function() return GetFrame(0).Bottom()*.65 end)
	cycle_map.Left:Set(function() return (GetFrame(0).Right()-cycle_map.Width())/2  end)
	cycle_map:DisableHitTest()
	cycle_map:Hide()
	
	cycle_Panel_tl = Bitmap(cycle_map)
	cycle_Panel_tl:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/cycle-panel-bg-tl.dds'))
	cycle_Panel_tl.Top:Set(cycle_map.Top)
	cycle_Panel_tl.Left:Set(cycle_map.Left)
	cycle_Panel_tl.Width:Set(40)

	cycle_Panel_bl = Bitmap(cycle_map)
	cycle_Panel_bl:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/cycle-panel-bg-bl.dds'))
	cycle_Panel_bl.Bottom:Set(cycle_map.Bottom)
	cycle_Panel_bl.Left:Set(cycle_map.Left)
	cycle_Panel_bl.Width:Set(40)
	
	cycle_Panel_l = Bitmap(cycle_map)
	cycle_Panel_l:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/cycle-panel-bg-l.dds'))
	cycle_Panel_l.Top:Set(cycle_Panel_tl.Bottom)
	cycle_Panel_l.Bottom:Set(cycle_Panel_bl.Top)
	cycle_Panel_l.Left:Set(cycle_map.Left)
	cycle_Panel_l.Width:Set(40)

	cycle_Panel_tr = Bitmap(cycle_map)
	cycle_Panel_tr:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/cycle-panel-bg-tr.dds'))
	cycle_Panel_tr.Top:Set(cycle_map.Top)
	cycle_Panel_tr.Right:Set(cycle_map.Right)
	cycle_Panel_tr.Width:Set(40)
	
	cycle_Panel_br = Bitmap(cycle_map)
	cycle_Panel_br:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/cycle-panel-bg-br.dds'))
	cycle_Panel_br.Bottom:Set(cycle_map.Bottom)
	cycle_Panel_br.Right:Set(cycle_map.Right)
	cycle_Panel_br.Width:Set(40)
	
	cycle_Panel_r = Bitmap(cycle_map)
	cycle_Panel_r:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/cycle-panel-bg-r.dds'))
	cycle_Panel_r.Top:Set(cycle_Panel_tr.Bottom)
	cycle_Panel_r.Bottom:Set(cycle_Panel_br.Top)
	cycle_Panel_r.Right:Set(cycle_map.Right)
	cycle_Panel_r.Width:Set(40)

	cycle_Panel_t = Bitmap(cycle_map)
	cycle_Panel_t:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/cycle-panel-bg-t.dds'))
	cycle_Panel_t.Top:Set(cycle_map.Top)
	cycle_Panel_t.Left:Set(cycle_Panel_l.Right)
	cycle_Panel_t.Right:Set(cycle_Panel_r.Left)

	cycle_Panel_b = Bitmap(cycle_map)
	cycle_Panel_b:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/cycle-panel-bg-b.dds'))
	cycle_Panel_b.Bottom:Set(cycle_map.Bottom)
	cycle_Panel_b.Left:Set(cycle_Panel_l.Right)
	cycle_Panel_b.Right:Set(cycle_Panel_r.Left)
	
	cycle_Panel_m = Bitmap(cycle_map)
	cycle_Panel_m:SetTexture(UIUtil.SkinnableFile(pathuiimg..'/cycle-panel-bg-m.dds'))
	cycle_Panel_m.Top:Set(cycle_Panel_t.Bottom)
	cycle_Panel_m.Bottom:Set(cycle_Panel_b.Top)
	cycle_Panel_m.Left:Set(cycle_Panel_l.Right)
	cycle_Panel_m.Right:Set(cycle_Panel_r.Left)
	
	cycle_map.mdThread = ForkThread(function()
		WaitSeconds(.6)
		if not cycle_map:IsHidden() then 
			Effect.FadeOut(cycle_map,.5,.6,0)
			WaitSeconds(.3)
		end
		cycle_on=false
    end)

	for name,list_cmd in list do
		local cmd=list_cmd.cmd	
		local cmd=apply_exceptions(list_cmd)
		--LOG('-------->cmd:', cmd)
		if cmd!=false and cmd!=nil then
			
			--use i = 0 so the first item on the list is chosen to build
			if i == 0 then
				import('/lua/ui/game/commandmode.lua').StartCommandMode("build", {name = cmd})
			end
			button[i] = Bitmap(cycle_map, UIUtil.SkinnableFile('/icons/units/' .. cmd .. '_build_btn_up.dds'))
			button[i].Height:Set(boutonH)
			button[i].Width:Set(boutonW)
			button[i].Depth:Set(1002)
			LayoutHelpers.AtLeftTopIn(button[i],cycle_map,29 + boutonH*i,18)
			cmd_sav[i]=cmd
			local bp = __blueprints[cmd]
			UIManager.addStrategicIconTo(button[i], bp)
			button[i]:SetAlpha(.3, true)
			i=i+1
		end
	end
	imax=i-1
	
	if imax == -1 then PlayErrorSound() end
	
	cycle_map.Height:Set(boutonH+36)
	cycle_map.Width:Set(i*boutonH+58)
	cycle_map:DisableHitTest(true)
	if cycle_time>=i then cycle_time=0 end

	local cmd=cmd_sav[cycle_time]

		if cmd!='' and cmd!=nil then
			if i>0 then 
				button[cycle_time]:SetAlpha(1, true)
				cycle_map:Show()
			end
		end
	if i<=0 then 
		cycle_time=0
		cycle_on=false
	end
end

function execute_all_list_command(list)
	for name,list_cmd in list do
		local cmd, upgrade=apply_exceptions(list_cmd)
		--LOG("==cmd===>",cmd)
		if cmd!=false and cmd!=nil then
			if upgrade then
				IssueBlueprintCommand("UNITCOMMAND_Upgrade", cmd, 1, false)
			else
				IssueBlueprintCommand("UNITCOMMAND_BuildFactory", cmd, 1)
			end
		end
	end
end


--builds a unit given a command a tech level and hotkey type
function buildhotkey(list) 
	--LOG("===========================")
	--LOG(repr(list))
	--LOG("==================")
	
	local selection = GetSelectedUnits()
	if selection then
		--if current selection is engineer (includes commander)
		if table.getsize(EntityCategoryFilterDown(categories.ENGINEER, selection)) > 0 then 
			-- do the cycle
			cycle_engineer(list)
		else -- buildqueue or normal applying all the command
			execute_all_list_command(list)
		end
	else
		PlayErrorSound()
	end	
end

--loads keymaps from file and copies them into table    TODO: check syntax while loading a table
function loadKeymaps()
	for i,name in keymapfile.keymaps do
		if keymapfile[name] then
			keymaps.data[name] = table.copy(validKeymap(keymapfile[name]))
			table.insert(keymaps.names,name)
		end
	end
	--WARN(repr(keymaps.data))
	--WARN(repr(keymaps.names))
end

--returns a valid key map for a given keymap
function validKeymap(keymap)
	local goodKeymap = {}
	for i,hotkey in keymap do
		if hotkey.key and hotkey.command then

			goodKeymap[i] = hotkey	
			
			if not hotkey.index then
				--WARN("CUSTOM KEYMAP: index field missing, going on anyways")
			end
			
			if not hotkey.category then
				--WARN("CUSTOM KEYMAP: category field missing, replacing by default field")
				goodKeymap[i].category = ''
			end

		else
			WARN("SCA UI MOD [CUSTOM KEYMAP]: critical field missing, ignoring the following key entry:")
			WARN(repr(hotkey))
		end
	end
	return goodKeymap
end
