-- This is a class for a generic resizable window

local Group = import('/lua/maui/group.lua').Group
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Button = import('/lua/maui/button.lua').Button
local UIUtil = import('/lua/ui/uiutil.lua')
local Dragger = import('/lua/maui/dragger.lua').Dragger
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Prefs = import('/lua/user/prefs.lua')
local pathui = '/mods/uimod/hook'
local profiles = GetPreference("profile.profiles") -- Gets our profiles table
local current = GetPreference("profile.current") -- gets the current profile
local Prefs = import('/lua/user/prefs.lua')--preferences
local pathuiimg='/mods/uimod/textures'
local sca_utils = '/mods/uimod/sca_utils/'
local UIManager= import(sca_utils .. 'uimanager.lua')
local UIMod= import(sca_utils .. 'uimod.lua')

ResizeGrp = Class(Group) {
    __init = function(self, parent, title, icon, pin, config, lockSize, lockPosition, xmin, ymin, limitmovement, idname)
        Group.__init(self, parent)

		local GameParent = GetFrame(0)

		--distance for border snapping
		local snapdist = 15
		--table of borders to snap to
		local snaptable = {mainborder = {Left = GameParent.Left(), Top = GameParent.Top(), Right = GameParent.Right(), Bottom = GameParent.Bottom()}}
		--name of border snapped to
		local snappedtoleft = false
		local snappedtoright = false
		local snappedtotop = false
		local snappedtobottom = false
		local snappedx = false
		local snappedy = false
		local distance = function(a, b)
			local dist = math.abs(a-b)
			return dist
		end
		
        self._resizeGroup = Group(self)
        LayoutHelpers.FillParent(self._resizeGroup, self)
        self._resizeGroup.Depth:Set(function() return self.Depth() + 100 end)
        self._resizeGroup:DisableHitTest()
        
        self._windowGroup = Group(self)
        LayoutHelpers.FillParent(self._windowGroup, self)
        self._windowGroup:DisableHitTest()
        
        self.tl = Bitmap(self._resizeGroup)
        self.tr = Bitmap(self._resizeGroup)
        self.bl = Bitmap(self._resizeGroup)
        self.br = Bitmap(self._resizeGroup)
        self.tm = Bitmap(self._resizeGroup)
        self.bm = Bitmap(self._resizeGroup)
        self.ml = Bitmap(self._resizeGroup)
        self.mr = Bitmap(self._resizeGroup)
        
        self._borderSize = 6
        self._sizeLock = false
        self._lockPosition = lockPosition or false
        self._lockSize = lockSize or false
        self._xMin = xmin or 0
        self._yMin = ymin or 0
        
        self.tl:SetSolidColor('ff415055')
        self.tr:SetSolidColor('ff415055')
        self.bl:SetSolidColor('ff415055')
        self.br:SetSolidColor('ff415055')
        self.tm:SetSolidColor('ff415055')
        self.bm:SetSolidColor('ff415055')
        self.ml:SetSolidColor('ff415055')
        self.mr:SetSolidColor('ff415055')
        
        --Set alpha of resize controls to 0 so that they still get resize events, but are not seen
        
        self.tl:SetAlpha(0)
        self.tr:SetAlpha(0)
        self.bl:SetAlpha(0)
        self.br:SetAlpha(0)
        self.tm:SetAlpha(0)
        self.bm:SetAlpha(0)
        self.ml:SetAlpha(0)
        self.mr:SetAlpha(0)
        
        self.tl.Height:Set(self._borderSize)
        self.tl.Width:Set(self._borderSize)
        self.tl.Top:Set(self.Top)
        self.tl.Left:Set(self.Left)
        
        self.tr.Height:Set(self._borderSize)
        self.tr.Width:Set(self._borderSize)
        self.tr.Top:Set(self.Top)
        self.tr.Right:Set(self.Right)
        
        self.bl.Height:Set(self._borderSize)
        self.bl.Width:Set(self._borderSize)
        self.bl.Bottom:Set(self.Bottom)
        self.bl.Left:Set(self.Left)
        
        self.br.Height:Set(self._borderSize)
        self.br.Width:Set(self._borderSize)
        self.br.Bottom:Set(self.Bottom)
        self.br.Right:Set(self.Right)
        
        self.tm.Height:Set(self._borderSize)
        self.tm.Left:Set(self.tl.Right)
        self.tm.Right:Set(self.tr.Left)
        self.tm.Top:Set(self.tl.Top)
        
        self.bm.Height:Set(self._borderSize)
        self.bm.Left:Set(self.bl.Right)
        self.bm.Right:Set(self.br.Left)
        self.bm.Top:Set(self.bl.Top)
        
        self.ml.Width:Set(self._borderSize)
        self.ml.Left:Set(self.tl.Left)
        self.ml.Top:Set(self.tl.Bottom)
        self.ml.Bottom:Set(self.bl.Top)
        
        self.mr.Width:Set(self._borderSize)
        self.mr.Right:Set(self.tr.Right)
        self.mr.Top:Set(self.tr.Bottom)
        self.mr.Bottom:Set(self.br.Top)
        
        self.window_tl = Bitmap(self._windowGroup, UIUtil.UIFile('/dialogs/def_window/bg_tl.dds'))
        self.window_tr = Bitmap(self._windowGroup, UIUtil.UIFile('/dialogs/def_window/bg_tr.dds'))
        self.window_tm = Bitmap(self._windowGroup, UIUtil.UIFile('/dialogs/def_window/bg_tm.dds'))
        self.window_ml = Bitmap(self._windowGroup, UIUtil.UIFile('/dialogs/def_window/bg_ml.dds'))
        self.window_m = Bitmap(self._windowGroup, UIUtil.UIFile('/dialogs/def_window/bg_m.dds'))
        self.window_mr = Bitmap(self._windowGroup, UIUtil.UIFile('/dialogs/def_window/bg_mr.dds'))
        self.window_bl = Bitmap(self._windowGroup, UIUtil.UIFile('/dialogs/def_window/bg_bl.dds'))
        self.window_bm = Bitmap(self._windowGroup, UIUtil.UIFile('/dialogs/def_window/bg_bm.dds'))
        self.window_br = Bitmap(self._windowGroup, UIUtil.UIFile('/dialogs/def_window/bg_br.dds'))
        
        self.window_tl.Top:Set(self.Top)
        self.window_tl.Left:Set(self.Left)
        
        self.window_tr.Top:Set(self.Top)
        self.window_tr.Right:Set(self.Right)
        
        self.window_bl.Bottom:Set(self.Bottom)
        self.window_bl.Left:Set(self.Left)
        
        self.window_br.Bottom:Set(self.Bottom)
        self.window_br.Right:Set(self.Right)
        
        self.window_tm.Left:Set(self.window_tl.Right)
        self.window_tm.Right:Set(self.window_tr.Left)
        self.window_tm.Top:Set(self.window_tl.Top)
        
        self.window_bm.Left:Set(self.window_bl.Right)
        self.window_bm.Right:Set(self.window_br.Left)
        self.window_bm.Top:Set(self.window_bl.Top)
        
        self.window_ml.Left:Set(self.window_tl.Left)
        self.window_ml.Top:Set(self.window_tl.Bottom)
        self.window_ml.Bottom:Set(self.window_bl.Top)
        
        self.window_mr.Right:Set(self.window_tr.Right)
        self.window_mr.Top:Set(self.window_tr.Bottom)
        self.window_mr.Bottom:Set(self.window_br.Top)
        
        self.window_m.Top:Set(self.window_tm.Bottom)
        self.window_m.Left:Set(self.window_ml.Right)
        self.window_m.Right:Set(self.window_mr.Left)
        self.window_m.Bottom:Set(self.window_bm.Top)
        
        self.TitleGroup = Group(self)
        self.TitleGroup.Top:Set(self.tm.Top)
        self.TitleGroup.Left:Set(self.tl.Left)
        self.TitleGroup.Right:Set(self.tr.Right)
        self.TitleGroup.Bottom:Set(self.bm.Top)
        self.TitleGroup.Depth:Set(function() return self._windowGroup.Depth() + 2 end)
        
        if icon then
            self._titleIcon = Bitmap(self.TitleGroup, icon)
            LayoutHelpers.AtLeftTopIn(self._titleIcon, self.TitleGroup, 2, 2)
        end
        
        if title then
            self._title = UIUtil.CreateText(self.TitleGroup, UIManager.trad(title), 18)
            if icon then
                self._title.Left:Set(function() return self._titleIcon.Right() + 5 end)
                LayoutHelpers.AtVerticalCenterIn(self._title, self._titleIcon)
            else
                LayoutHelpers.AtLeftTopIn(self._title, self.TitleGroup, 7, 7)
            end
		end

        if pin then
            self._pinBtn = Checkbox(self.TitleGroup,
                UIUtil.UIFile('/dialogs/pin/pin_up.dds'),
                UIUtil.UIFile('/dialogs/pin/pinned_up.dds'),
                UIUtil.UIFile('/dialogs/pin/pin_over.dds'),
                UIUtil.UIFile('/dialogs/pin/pinned_over.dds'),
                UIUtil.UIFile('/dialogs/pin/pin_dis.dds'),
                UIUtil.UIFile('/dialogs/pin/pinned_dis.dds'))
        LayoutHelpers.AtRightTopIn(self._pinBtn, self.TitleGroup, 7, 7)
            self._pinBtn.OnCheck = function(control, checked)
                self:OnPinCheck(checked)
            end
        end
        
        if config then
            self._configBtn = Button(self.TitleGroup,
                UIUtil.UIFile(pathuiimg..'/config_btn_up.dds'),
  --              UIUtil.UIFile('/dialogs/config_btn/config_btn_up.dds'),				
                UIUtil.UIFile('/dialogs/config_btn/config_btn_down.dds'),
                UIUtil.UIFile('/dialogs/config_btn/config_btn_over.dds'),
                UIUtil.UIFile('/dialogs/config_btn/config_btn_dis.dds'))
            if pin then
                LayoutHelpers.LeftOf(self._configBtn, self._pinBtn)
            else
        LayoutHelpers.AtRightTopIn(self._configBtn, self.TitleGroup, 7, 7)
            end
            self._configBtn.OnClick = function(control)
                self:OnConfigClick()
            end
        end
        
        self.ClientGroup = Group(self)
        self.ClientGroup.Top:Set(self.TitleGroup.Bottom)
        self.ClientGroup.Left:Set(self.ml.Right)
        self.ClientGroup.Height:Set(function() return self.bm.Top() - self.TitleGroup.Bottom() end)
        self.ClientGroup.Width:Set(function() return self.mr.Left() - self.ml.Right() end)
        self.ClientGroup.Right:Set(self.mr.Left)
        self.ClientGroup.Bottom:Set(self.bm.Top)
        self.ClientGroup.Depth:Set(function() return self.window_m.Depth() + 1 end)
        
		--resize window
		
        local function StartSizing(event, xControl, yControl)
 			--create the table of current marker positions
			local layout = "sca"
			for name, control in UIManager.ResizeTable do
				if control:IsHidden() == false then
					snaptable[name] = {Left = control.Left(), Top = control.Top(), Right = control.Right(), Bottom = control.Bottom()}
				end
			end
			local drag = Dragger()
            local x_max = true
            local y_max = true
            offX = event.MouseX - self.Left()
            offY = event.MouseY - self.Top()
            height = self.Height()
            width = self.Width()
			if event.MouseX < self.tl.Right() then
                x_max = false
            end
            if event.MouseY < self.tl.Bottom() then
                y_max = false
            end
            drag.OnMove = function(dragself, x, y)
                if Prefs.GetFromCurrentProfile("uimanager_sav")["snapCheckBox"]==1 then
				snappedx = false
				snappedy = false
					--snap to edges of other resizers
					if xControl then
						local newX
						for name, marker in snaptable do
							--is the current resizer or the 'name' resizer in the same y region of the screen? CHECK THIS, EASY TO MAKE A STUPID ERROR
							if name != idname then
								if (((y-offY) < (marker.Bottom + snapdist) and (y-offY) > (marker.Top - snapdist))
								or ((y-offY+height) < (marker.Bottom + snapdist) and (y-offY+height) > (marker.Top - snapdist))
								or ((marker.Bottom) > (y-offY - snapdist) and (marker.Bottom) < (y-offY+height + snapdist))
								or ((marker.Top) > (y-offY - snapdist) and (marker.Top) < (y-offY+height + snapdist))) then
									--is the current resizer near enough the 'name' resizer to snap to it? if so snap and mark what resizer it is snapped to
									if distance(x, marker.Left) < snapdist then
										if x_max then
											newX = math.min(math.max(marker.Left, self.Left() + self._xMin), GameParent.Right())
											newX = math.max(newX, self.Left() + self._title.Width() + (2*self.window_tl.Width()))
										else
											newX = math.min(math.max(marker.Left, 0), self.Right() - self._xMin)
										end
									snappedx = true
									elseif distance(x, marker.Right) < snapdist then
										if x_max then
											newX = math.min(math.max(marker.Right, self.Left() + self._xMin), GameParent.Right())
											newX = math.max(newX, self.Left() + self._title.Width() + (2*self.window_tl.Width()))
										else
											newX = math.min(math.max(marker.Right, 0), self.Right() - self._xMin)
										end
									snappedx = true
									end
								end
								if snappedx == false then
									if x_max then
										newX = math.min(math.max(x, self.Left() + self._xMin), GameParent.Right())
										newX = math.max(newX, self.Left() + self._title.Width() + (2*self.window_tl.Width()))
									else
										newX = math.min(math.max(x, 0), self.Right() - self._xMin)
									end
								end
							end
						end
						xControl:Set(newX)
					end
					if yControl then
						local newY 
						for name, marker in snaptable do
							--is the current resizer or the 'name' resizer in the same y region of the screen?
							if name != idname then
								if (((x-offX) > (marker.Left - snapdist) and (x-offX) < (marker.Right + snapdist))
								or ((x-offX+width) > (marker.Left - snapdist) and (x-offX+width) < (marker.Right + snapdist))
								or ((marker.Left) > (x-offX - snapdist) and (marker.Left) < (x-offX+width + snapdist))
								or ((marker.Right) > (x-offX - snapdist) and (marker.Right) < (x-offX+width + snapdist))) then
									--is the current resizer near enough the 'name' resizer to snap to it? if so snap and mark what resizer it is snapped to
									if distance(y, marker.Top) < snapdist then
										if y_max then
											newY = math.min(math.max(marker.Top, self.Top() + self._yMin), GameParent.Bottom())
											newY = math.max(newY, self.Top() + self.window_bm.Height() + self.window_tm.Height())
										else
											newY = math.min(math.max(marker.Top, 0), self.Bottom() - self._yMin)
										end
										snappedy = true
									elseif distance(y, marker.Bottom) < snapdist then
										if y_max then
											newY = math.min(math.max(marker.Bottom, self.Top() + self._yMin), GameParent.Bottom())
											newY = math.max(newY, self.Top() + self.window_bm.Height() + self.window_tm.Height())
										else
											newY = math.min(math.max(marker.Bottom, 0), self.Bottom() - self._yMin)
										end
										snappedy = true
									end
								end
								if snappedy == false then
									if y_max then
										newY = math.min(math.max(y, self.Top() + self._yMin), GameParent.Bottom())
										newY = math.max(newY, self.Top() + self.window_bm.Height() + self.window_tm.Height())
									else
										newY = math.min(math.max(y, 0), self.Bottom() - self._yMin)
									end
								end
							end
						end
						yControl:Set(newY)
					end
				else
					if xControl then
						local newX
						if x_max then
							newX = math.min(math.max(x, self.Left() + self._xMin), GameParent.Right())
							newX = math.max(newX, self.Left() + self._title.Width() + (2*self.window_tl.Width()))
						else
							newX = math.min(math.max(x, 0), self.Right() - self._xMin)
						end
						xControl:Set(newX)
					end
					if yControl then
						local newY 
						if y_max then
							newY = math.min(math.max(y, self.Top() + self._yMin), GameParent.Bottom())
							newY = math.max(newY, self.Top() + self.window_bm.Height() + self.window_tm.Height())
						else
							newY = math.min(math.max(y, 0), self.Bottom() - self._yMin)
						end
						yControl:Set(newY)
					end
                end
				self:OnResize(x, y, not self._sizeLock)
                if not self._sizeLock then
                    self._sizeLock = true
                end
            end
            drag.OnRelease = function(dragself)
                self._sizeLock = false
                self._resizeGroup:SetAlpha(0, true)
                GetCursor():Reset()
                drag:Destroy()
                self:OnResizeSet()
				snaptable = {mainborder = {Left = GameParent.Left(), Top = GameParent.Top(), Right = GameParent.Right(), Bottom = GameParent.Bottom()}}
            end
            drag.OnCancel = function(dragself)
                self._sizeLock = false
                GetCursor():Reset()
                drag:Destroy()
            end
            PostDragger(self:GetRootFrame(), event.KeyCode, drag)
        end
                
        local function RolloverHandler(control, event, xControl, yControl, cursor)
            if self._lockSize == true then
				return
			elseif self._lockSize == 'Horizontal' then
	            if not self._sizeLock then
	                if event.Type == 'MouseEnter' then
	                    self._resizeGroup:SetAlpha(1, true)
	                    GetCursor():SetTexture(UIUtil.GetCursor(cursor))
	                elseif event.Type == 'MouseExit' then
	                    self._resizeGroup:SetAlpha(0, true)
	                    GetCursor():Reset()
	                elseif event.Type == 'ButtonPress' then
	                    StartSizing(event, nil, yControl)
	                end
	            end
			elseif self._lockSize == 'Vertical' then
	            if not self._sizeLock then
	                if event.Type == 'MouseEnter' then
	                    self._resizeGroup:SetAlpha(1, true)
	                    GetCursor():SetTexture(UIUtil.GetCursor(cursor))
	                elseif event.Type == 'MouseExit' then
	                    self._resizeGroup:SetAlpha(0, true)
	                    GetCursor():Reset()
	                elseif event.Type == 'ButtonPress' then
	                    StartSizing(event, xControl, nil)
	                end
	            end
			else
	            if not self._sizeLock then
	                if event.Type == 'MouseEnter' then
	                    self._resizeGroup:SetAlpha(1, true)
	                    GetCursor():SetTexture(UIUtil.GetCursor(cursor))
	                elseif event.Type == 'MouseExit' then
	                    self._resizeGroup:SetAlpha(0, true)
	                    GetCursor():Reset()
	                elseif event.Type == 'ButtonPress' then
	                    StartSizing(event, xControl, yControl)
	                end
	            end
			end
        end
        
        self.br.HandleEvent = function(control, event)
            RolloverHandler(control, event, self.Right, self.Bottom, 'NW_SE')
        end
        self.bl.HandleEvent = function(control, event)
            RolloverHandler(control, event, self.Left, self.Bottom, 'NE_SW')
        end
        self.bm.HandleEvent = function(control, event)
            RolloverHandler(control, event, nil, self.Bottom, 'N_S')
        end
        self.tr.HandleEvent = function(control, event)
            RolloverHandler(control, event, self.Right, self.Top, 'NE_SW')
        end
        self.tl.HandleEvent = function(control, event)
            RolloverHandler(control, event, self.Left, self.Top, 'NW_SE')
        end
        self.tm.HandleEvent = function(control, event)
            RolloverHandler(control, event, nil, self.Top, 'N_S')
        end
        self.mr.HandleEvent = function(control, event)
            RolloverHandler(control, event, self.Right, nil, 'W_E')
        end
        self.ml.HandleEvent = function(control, event)
            RolloverHandler(control, event, self.Left, nil, 'W_E')
        end
        
		-- move window
		
        self.TitleGroup.HandleEvent = function(control, event)
            if not self._sizeLock then
                if event.Type == 'ButtonPress' then
					--create the table of current marker positions
					local layout = "sca"
					for name, control in UIManager.ResizeTable do
						if control:IsHidden() == false then
							snaptable[name] = {Left = control.Left(), Top = control.Top(), Right = control.Right(), Bottom = control.Bottom()}
						end
					end
					if self._lockPosition then return end
                    local drag = Dragger()
                    local offX = event.MouseX - self.Left()
                    local offY = event.MouseY - self.Top()
                    local height = self.Height()
                    local width = self.Width()
                    drag.OnMove = function(dragself, x, y)
						local newLeft
						local newTop
						if Prefs.GetFromCurrentProfile("uimanager_sav")["snapCheckBox"]==1 then
							snappedtoleft = false
							snappedtoright = false
							snappedtotop = false
							snappedtobottom = false
							--snap to edges of other resizers
							for name, marker in snaptable do
								--is the current resizer or the 'name' resizer in the same y region of the screen?
								if name != idname then
									--snap left/right side
									if (((y-offY) < (marker.Bottom + snapdist) and (y-offY) > (marker.Top - snapdist))
									or ((y-offY+height) < (marker.Bottom + snapdist) and (y-offY+height) > (marker.Top - snapdist))
									or ((marker.Bottom) > (y-offY - snapdist) and (marker.Bottom) < (y-offY+height + snapdist))
									or ((marker.Top) > (y-offY - snapdist) and (marker.Top) < (y-offY+height + snapdist))) then
										--is the current resizer near enough the 'name' resizer to snap to it? if so snap and mark what resizer it is snapped to
										--left
										if distance(x-offX, marker.Left) < snapdist then
											newLeft = (marker.Left)
											snappedtoleft = name
										elseif distance(x-offX, marker.Right) < snapdist then
											newLeft = (marker.Right)
											snappedtoright = name
										end
										--right
										if distance(x-offX + width, marker.Left) < snapdist then
											newLeft = (marker.Left - width)
											snappedtoleft = name
										elseif distance(x-offX + width, marker.Right) < snapdist then
											newLeft = (marker.Right - width)
											snappedtoright = name
										end	
									end
									--snap top/bottom
									if (((x-offX) > (marker.Left - snapdist) and (x-offX) < (marker.Right + snapdist))
									or ((x-offX+width) > (marker.Left - snapdist) and (x-offX+width) < (marker.Right + snapdist))
									or ((marker.Left) > (x-offX - snapdist) and (marker.Left) < (x-offX+width + snapdist))
									or ((marker.Right) > (x-offX - snapdist) and (marker.Right) < (x-offX+width + snapdist))) then
										--is the current resizer near enough the 'name' resizer to snap to it? if so snap and mark what resizer it is snapped to
										--top
										if distance(y-offY, marker.Top) < snapdist then
											newTop = (marker.Top)
											snappedtotop = name
										elseif distance(y-offY, marker.Bottom) < snapdist then
											newTop = (marker.Bottom)
											snappedtobottom = name
										end
										--bottom
										if distance(y-offY + height, marker.Top) < snapdist then
											newTop = (marker.Top - height)
											snappedtotop = name
										elseif distance(y-offY + height, marker.Bottom) < snapdist then
											newTop = (marker.Bottom - height)
											snappedtobottom = name
										end
									end
								end
							end

							-- if not snapped to anything then move normally
							if snappedtoleft == false and snappedtoright == false then
								newLeft = (x-offX)
							end
							
							if snappedtotop == false and snappedtobottom == false then
								newTop = (y-offY)
							end
						else
							--just move
							newLeft = (x-offX)
							newTop = (y-offY)
						end
						if limitmovement == true then
							--stop movement outside GameParent edges
							newLeft = (math.min(math.max(newLeft, 0), GameParent.Right() - width))
							newTop = (math.min(math.max(newTop, 0), GameParent.Bottom() - height))
						end
						self.Left:Set(newLeft)
						self.Top:Set(newTop)
						
                        local tempRight = self.Left() + width
                        local tempBottom = self.Top() + height
                        self.Right:Set(tempRight)
                        self.Bottom:Set(tempBottom)
                        self:OnMove(x, y, not self._sizeLock)
                        if not self._sizeLock then
                            GetCursor():SetTexture(UIUtil.GetCursor('MOVE_WINDOW'))
                            self._sizeLock = true
                        end
                    end
                    drag.OnRelease = function(dragself)
                        self._sizeLock = false
                        GetCursor():Reset()
                        drag:Destroy()
                        self:OnMoveSet()
						snaptable = {mainborder = {Left = GameParent.Left(), Top = GameParent.Top(), Right = GameParent.Right(), Bottom = GameParent.Bottom()}}
                    end
                    drag.OnCancel = function(dragself)
                        self._sizeLock = false
                        GetCursor():Reset()
                        drag:Destroy()
                    end
                    PostDragger(self:GetRootFrame(), event.KeyCode, drag)
                end
			end
        end
        
        self.HandleEvent = function(control, event)
            if event.Type == 'WheelRotation' then
                self:OnMouseWheel(event.WheelRotation)
            end
        end
        self.OnHide = function(control, hidden)
            control._resizeGroup:SetHidden(hidden)
            control:OnHideWindow(control, hidden)
        end
    end,
    
    GetClientGroup = function(self)
        return self.ClientGroup
    end,
    
    SetSizeLock = function(self, locked)
        self._lockSize = locked
    end,
    
    SetPositionLock = function(self, locked)
        self._lockPosition = locked
    end,
    
    SetMinimumResize = function(control, xDimension, yDimension)
        control._xMin = xDimension or 0
        control._yMin = yDimension or 0
    end,
    
    SetWindowAlpha = function(control, alpha)
        control._windowGroup:SetAlpha(alpha, true)
    end,
    
    IsPinned = function(control)
        if control._pinBtn then
            return control._pinBtn:IsChecked()
        else
            return false
        end
    end,
        
    OnDestroy = function(control)
        control._resizeGroup:Destroy()
    end,
    
    --
    -- The following are functions that can be overloaded
    --
    OnResize = function(control, x, y, firstFrame) end,
    OnResizeSet = function(control) end,
    
    OnMove = function(control, x, y, firstFrame) end,
    OnMoveSet = function(control) end,
    
    OnPinCheck = function(control, checked) end,
    OnConfigClick = function(control) end,
    
    OnMouseWheel = function(control, rotation) end,
    
    OnClose = function(control) end,
    OnHideWindow = function(control, hidden) end,
}